pub struct MPSAccelerationStructureGroup { /* private fields */ }MPSAccelerationStructureGroup only.Expand description
A group of acceleration structures which may be used together in an instance acceleration structure.
All acceleration structures in an instance acceleration structures must be created with the same group, although they do not all need to be used in the same instance acceleration structure. The acceleration structures in a group share internal GPU memory allocations, so the total number and size of acceleration structures that can be created with the same group is limited by the Metal deviceโs buffer size limits. Therefore, do not group acceleration structures unless they are likely to be used in the same instance acceleration structure.
See also Appleโs documentation
Implementations
Sourceimpl MPSAccelerationStructureGroup
impl MPSAccelerationStructureGroup
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
๐DeprecatedAvailable on crate feature MPSRayIntersector only.
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
MPSRayIntersector only.The Metal device this acceleration structure group was created with
pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>
MPSRayIntersector only.pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSRayIntersector only.Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesnโt recognize.
See Appleโs documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
๐Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable โunder the hoodโ.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Appleโs documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called โlightweight genericsโ, and thatโs because they arenโt exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you donโt need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl AsRef<AnyObject> for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl AsRef<MPSAccelerationStructureGroup> for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl AsRef<MPSAccelerationStructureGroup> for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl AsRef<NSObject> for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl AsRef<NSObject> for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl Borrow<AnyObject> for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl Borrow<AnyObject> for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl Borrow<NSObject> for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl Borrow<NSObject> for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl ClassType for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl ClassType for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceconst NAME: &'static str = "MPSAccelerationStructureGroup"
const NAME: &'static str = "MPSAccelerationStructureGroup"
Sourcetype ThreadKind = <<MPSAccelerationStructureGroup as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSAccelerationStructureGroup as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl Debug for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl Debug for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl Deref for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl Deref for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl Hash for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl Hash for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl Message for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl Message for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl NSObjectProtocol for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl NSObjectProtocol for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn description(&self) -> Retained<NSObject>
fn description(&self) -> Retained<NSObject>
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl PartialEq for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceimpl RefEncode for MPSAccelerationStructureGroup
Available on crate feature MPSRayIntersector only.
impl RefEncode for MPSAccelerationStructureGroup
MPSRayIntersector only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSAccelerationStructureGroup
MPSRayIntersector only.impl Eq for MPSAccelerationStructureGroup
MPSRayIntersector only.