MPSAccelerationStructureUsage

Struct MPSAccelerationStructureUsage 

#[repr(transparent)]
pub struct MPSAccelerationStructureUsage(pub NSUInteger);
👎Deprecated
Available on crate feature MPSAccelerationStructure only.
Options describing how an acceleration structure will be used

See also Apple’s documentation

Tuple Fields

0: NSUInteger
👎Deprecated

Implementations

Source

impl MPSAccelerationStructureUsage

pub const None: Self

👎Deprecated
Available on crate feature MPSRayIntersector only.

No usage options specified

pub const Refit: Self

👎Deprecated
Available on crate feature MPSRayIntersector only.

Enable support for refitting the acceleration structure after it has been built. This option may reduce raytracing performance so do not use it unless the acceleration structure will be refit.

pub const FrequentRebuild: Self

👎Deprecated
Available on crate feature MPSRayIntersector only.

Option indicating that the acceleration structure will be rebuilt frequently. In this case, the acceleration structure may choose a higher performance but lower quality acceleration structure construction algorithm. This option may reduce raytracing performance performance so do not use it unless reduced acceleration structure build time is worth reduced raytracing performance. This option may be useful if, for example, the user is interactively editing a live view of the scene.

pub const PreferGPUBuild: Self

👎Deprecated
Available on crate feature MPSRayIntersector only.

Prefer building the acceleration structure on the GPU. By default, the acceleration structure will be built on the GPU when possible. However, in some cases such as very small triangle counts, the acceleration structure may be built on the CPU. This option will force the acceleration structure to be always be built on the GPU whenever possible.

pub const PreferCPUBuild: Self

👎Deprecated
Available on crate feature MPSRayIntersector only.

Prefer building the acceleration structure on the CPU. By default, the acceleration structure will be built on the GPU when possible, which is typically much faster than building on the CPU. However, in some cases it may be preferable to build on the CPU such as to avoid framerate hitches when the GPU is rendering the user interface.

Source

impl MPSAccelerationStructureUsage

pub const fn empty() -> Self

Available on crate feature MPSRayIntersector only.

Get a flags value with all bits unset.

pub const fn all() -> Self

Available on crate feature MPSRayIntersector only.

Get a flags value with all known bits set.

pub const fn bits(&self) -> NSUInteger

Available on crate feature MPSRayIntersector only.

Get the underlying bits value.

The returned value is exactly the bits set in this flags value.

pub const fn from_bits(bits: NSUInteger) -> Option<Self>

Available on crate feature MPSRayIntersector only.

Convert from a bits value.

This method will return None if any unknown bits are set.

pub const fn from_bits_truncate(bits: NSUInteger) -> Self

Available on crate feature MPSRayIntersector only.

Convert from a bits value, unsetting any unknown bits.

pub const fn from_bits_retain(bits: NSUInteger) -> Self

Available on crate feature MPSRayIntersector only.

Convert from a bits value exactly.

pub fn from_name(name: &str) -> Option<Self>

Available on crate feature MPSRayIntersector only.

Get a flags value with the bits of a flag with the given name set.

This method will return None if name is empty or doesn’t correspond to any named flag.

pub const fn is_empty(&self) -> bool

Available on crate feature MPSRayIntersector only.

Whether all bits in this flags value are unset.

pub const fn is_all(&self) -> bool

Available on crate feature MPSRayIntersector only.

Whether all known bits in this flags value are set.

pub const fn intersects(&self, other: Self) -> bool

Available on crate feature MPSRayIntersector only.

Whether any set bits in a source flags value are also set in a target flags value.

pub const fn contains(&self, other: Self) -> bool

Available on crate feature MPSRayIntersector only.

Whether all set bits in a source flags value are also set in a target flags value.

pub fn insert(&mut self, other: Self)

Available on crate feature MPSRayIntersector only.

The bitwise or (|) of the bits in two flags values.

pub fn remove(&mut self, other: Self)

Available on crate feature MPSRayIntersector only.

The intersection of a source flags value with the complement of a target flags value (&!).

This method is not equivalent to self & !other when other has unknown bits set. remove won’t truncate other, but the ! operator will.

pub fn toggle(&mut self, other: Self)

Available on crate feature MPSRayIntersector only.

The bitwise exclusive-or (^) of the bits in two flags values.

pub fn set(&mut self, other: Self, value: bool)

Available on crate feature MPSRayIntersector only.

Call insert when value is true or remove when value is false.

pub const fn intersection(self, other: Self) -> Self

Available on crate feature MPSRayIntersector only.

The bitwise and (&) of the bits in two flags values.

pub const fn union(self, other: Self) -> Self

Available on crate feature MPSRayIntersector only.

The bitwise or (|) of the bits in two flags values.

pub const fn difference(self, other: Self) -> Self

Available on crate feature MPSRayIntersector only.

The intersection of a source flags value with the complement of a target flags value (&!).

This method is not equivalent to self & !other when other has unknown bits set. difference won’t truncate other, but the ! operator will.

pub const fn symmetric_difference(self, other: Self) -> Self

Available on crate feature MPSRayIntersector only.

The bitwise exclusive-or (^) of the bits in two flags values.

pub const fn complement(self) -> Self

Available on crate feature MPSRayIntersector only.

The bitwise negation (!) of the bits in a flags value, truncating the result.

Source

impl MPSAccelerationStructureUsage

pub const fn iter(&self) -> Iter<MPSAccelerationStructureUsage>

Available on crate feature MPSRayIntersector only.

Yield a set of contained flags values.

Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.

pub const fn iter_names(&self) -> IterNames<MPSAccelerationStructureUsage>

Available on crate feature MPSRayIntersector only.

Yield a set of contained named flags values.

This method is like iter, except only yields bits in contained named flags. Any unknown bits, or bits not corresponding to a contained flag will not be yielded.