#[repr(transparent)]pub struct MPSCNNConvolutionGradientOption(pub NSUInteger);MPSCNNConvolution only.Expand description
Tuple Fields
0: NSUIntegerImplementations
Sourceimpl MPSCNNConvolutionGradientOption
impl MPSCNNConvolutionGradientOption
pub const GradientWithData: Self
MPSNeuralNetwork only.pub const GradientWithWeightsAndBias: Self
MPSNeuralNetwork only.pub const All: Self
MPSNeuralNetwork only.Sourceimpl MPSCNNConvolutionGradientOption
impl MPSCNNConvolutionGradientOption
Sourcepub const fn empty() -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn empty() -> Self
MPSNeuralNetwork only.Get a flags value with all bits unset.
Sourcepub const fn all() -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn all() -> Self
MPSNeuralNetwork only.Get a flags value with all known bits set.
Sourcepub const fn bits(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub const fn bits(&self) -> NSUInteger
MPSNeuralNetwork only.Get the underlying bits value.
The returned value is exactly the bits set in this flags value.
Sourcepub const fn from_bits(bits: NSUInteger) -> Option<Self>
Available on crate feature MPSNeuralNetwork only.
pub const fn from_bits(bits: NSUInteger) -> Option<Self>
MPSNeuralNetwork only.Convert from a bits value.
This method will return None if any unknown bits are set.
Sourcepub const fn from_bits_truncate(bits: NSUInteger) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn from_bits_truncate(bits: NSUInteger) -> Self
MPSNeuralNetwork only.Convert from a bits value, unsetting any unknown bits.
Sourcepub const fn from_bits_retain(bits: NSUInteger) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn from_bits_retain(bits: NSUInteger) -> Self
MPSNeuralNetwork only.Convert from a bits value exactly.
Sourcepub fn from_name(name: &str) -> Option<Self>
Available on crate feature MPSNeuralNetwork only.
pub fn from_name(name: &str) -> Option<Self>
MPSNeuralNetwork only.Get a flags value with the bits of a flag with the given name set.
This method will return None if name is empty or doesn’t
correspond to any named flag.
Sourcepub const fn is_empty(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub const fn is_empty(&self) -> bool
MPSNeuralNetwork only.Whether all bits in this flags value are unset.
Sourcepub const fn is_all(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub const fn is_all(&self) -> bool
MPSNeuralNetwork only.Whether all known bits in this flags value are set.
Sourcepub const fn intersects(&self, other: Self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub const fn intersects(&self, other: Self) -> bool
MPSNeuralNetwork only.Whether any set bits in a source flags value are also set in a target flags value.
Sourcepub const fn contains(&self, other: Self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub const fn contains(&self, other: Self) -> bool
MPSNeuralNetwork only.Whether all set bits in a source flags value are also set in a target flags value.
Sourcepub fn insert(&mut self, other: Self)
Available on crate feature MPSNeuralNetwork only.
pub fn insert(&mut self, other: Self)
MPSNeuralNetwork only.The bitwise or (|) of the bits in two flags values.
Sourcepub fn remove(&mut self, other: Self)
Available on crate feature MPSNeuralNetwork only.
pub fn remove(&mut self, other: Self)
MPSNeuralNetwork only.The intersection of a source flags value with the complement of a target flags
value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
remove won’t truncate other, but the ! operator will.
Sourcepub fn toggle(&mut self, other: Self)
Available on crate feature MPSNeuralNetwork only.
pub fn toggle(&mut self, other: Self)
MPSNeuralNetwork only.The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub fn set(&mut self, other: Self, value: bool)
Available on crate feature MPSNeuralNetwork only.
pub fn set(&mut self, other: Self, value: bool)
MPSNeuralNetwork only.Call insert when value is true or remove when value is false.
Sourcepub const fn intersection(self, other: Self) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn intersection(self, other: Self) -> Self
MPSNeuralNetwork only.The bitwise and (&) of the bits in two flags values.
Sourcepub const fn union(self, other: Self) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn union(self, other: Self) -> Self
MPSNeuralNetwork only.The bitwise or (|) of the bits in two flags values.
Sourcepub const fn difference(self, other: Self) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn difference(self, other: Self) -> Self
MPSNeuralNetwork only.The intersection of a source flags value with the complement of a target flags
value (&!).
This method is not equivalent to self & !other when other has unknown bits set.
difference won’t truncate other, but the ! operator will.
Sourcepub const fn symmetric_difference(self, other: Self) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn symmetric_difference(self, other: Self) -> Self
MPSNeuralNetwork only.The bitwise exclusive-or (^) of the bits in two flags values.
Sourcepub const fn complement(self) -> Self
Available on crate feature MPSNeuralNetwork only.
pub const fn complement(self) -> Self
MPSNeuralNetwork only.The bitwise negation (!) of the bits in a flags value, truncating the result.
Sourceimpl MPSCNNConvolutionGradientOption
impl MPSCNNConvolutionGradientOption
Sourcepub const fn iter(&self) -> Iter<MPSCNNConvolutionGradientOption>
Available on crate feature MPSNeuralNetwork only.
pub const fn iter(&self) -> Iter<MPSCNNConvolutionGradientOption>
MPSNeuralNetwork only.Yield a set of contained flags values.
Each yielded flags value will correspond to a defined named flag. Any unknown bits will be yielded together as a final flags value.
Sourcepub const fn iter_names(&self) -> IterNames<MPSCNNConvolutionGradientOption>
Available on crate feature MPSNeuralNetwork only.
pub const fn iter_names(&self) -> IterNames<MPSCNNConvolutionGradientOption>
MPSNeuralNetwork only.Yield a set of contained named flags values.
This method is like iter, except only yields bits in contained named flags.
Any unknown bits, or bits not corresponding to a contained flag will not be yielded.