MPSCNNConvolutionGradientState

Struct MPSCNNConvolutionGradientState 

pub struct MPSCNNConvolutionGradientState { /* private fields */ }
Available on crate features MPSCNNConvolution and MPSCore and MPSNNGradientState and MPSState only.
The MPSCNNConvolutionGradientState is returned by resultStateForSourceImage:sourceStates method on MPSCNNConvolution object. Note that resultStateForSourceImage:sourceStates:destinationImage creates the object on autoreleasepool. It will be consumed by MPSCNNConvolutionGradient. This is also used by MPSCNNConvolutionTranspose encode call that returns MPSImage on left hand side to correctly size the destination. Note that state objects are not usable across batches i.e. when batch is done you should nuke the state object and create new one for next batch.

This state exposes the gradient with respect to weights and biases, as computed by the MPSCNNConvolutionGradient kernel, as a metal buffer to be used during weights and biases update. The standard weights and biases update formula is:

weights(t+1) = f(weights(t), gradientForWeights(t)) and biases(t+1) = f(biases(t), gradientForBiases(t)),

where the weights(t)/biases(t) are the wegihts and the biases at step t that are provided by data source provider used to create MPSCNNConvolution and MPSCNNConvoltuionGradient objects. There are multiple ways user can update weights and biases as described below:

  1. For check pointing, i.e. updating weights/biases and storing: once the command buffer on which MPSCNNConvolutionGradient is enqueued is done (e.g. in command buffer completion callback), the application can simply use float* delta_w = (float*)((char*)[gradientForWeights contents]); float* delta_b = (float*)((char*)[gradientForBiases contents]); to update the weights and biases in the data provider directly. The application can instead provide a metal kernel that reads from gradientForWeights and gradientForBiases buffer and the buffer created using data provided by the data source to do any kind of update it will like to do, then read back the updated weights/biases and store to the data source. Note that lifetime of the gradientForWeights and gradientForBiases buffer is the same as the MPSCNNConvolutionGradientState. So it’s the applications’s responsibility to make sure the buffer is alive (retained) when the update kernel is running if the command buffer doesn’t retain the buffer. Also, in order to gaurantee that the buffer is correctly synchronized for CPU side access, it is the application’s responsibility to call [gradientState synchronizeOnCommandBuffer:] before accessing data from the buffer.

  2. For a CPU side update, once the weights and biases in the data source provider are updated as above, the original MPSCNNConvolution and MPSCNNConvolutionGradient objects need to be updated with the new weigths and biases by calling the -(void) reloadWeightsAndBiasesFromDataSource method. Again application needs to call [gradientState synchronizeOnCommandBuffer:] before touching data on CPU side.

  3. The above CPU side update requires command buffer to be done. If the application doesn’t want to update its data source provider object and would prefer to directly enqueue an update of the internal MPSCNNConvolution and MPSCNNConvolutionGradient weights/biases buffers on the GPU without CPU side involvement, it needs to do following: i) get gradientForWeights and gradientForBiases buffers from this gradient state object and set it as source of update kernel ii) create a temporary buffer, dest, of same size and set it as destination of update kernel iii) enqueue update kernel on command buffer iv) call reloadWeightsAndBiasesWithCommandBuffer:dest:weightsOffset:biasesOffset on MPSCNNConvolution and MPSCNNConvolutionGradient objects. This will reload the weights from application’s update kernel in dest on GPU without CPU side involvement.

See also Apple’s documentation

Implementations

Source

impl MPSCNNConvolutionGradientState

pub unsafe fn gradientForWeights( &self, ) -> Retained<ProtocolObject<dyn MTLBuffer>>

Available on crate feature MPSNeuralNetwork only.

A buffer that contains the loss function gradients with respect to weights. Each value in the buffer is a float. The layout of the gradients with respect to the weights is the same as the weights layout provided by data source i.e. it can be interpreted as 4D array

gradientForWeights[outputFeatureChannels][kernelHeight][kernelWidth][inputFeatureChannels/groups] For depthwise convolution it will be (since we only support channel multiplier of 1 currently) gradientForWeights[outputFeatureChannels][kernelHeight][kernelWidth]

pub unsafe fn gradientForBiases( &self, ) -> Retained<ProtocolObject<dyn MTLBuffer>>

Available on crate feature MPSNeuralNetwork only.

A buffer that contains the loss function gradients with respect to biases.

pub unsafe fn convolution(&self) -> Retained<MPSCNNConvolution>

Available on crate features MPSNeuralNetwork and MPSCNNKernel and MPSKernel only.

The convolution filter that produced the state. For child MPSCNNConvolutionTrasposeGradientState object, convolution below refers to MPSCNNConvolution object that produced MPSCNNConvolutionGradientState object which was used to create MPSCNNConvolutionTransposeGradientState object. See resultStateForSourceImage:sourceStates method of MPSCNNConvolutionTranspose below.

pub unsafe fn gradientForWeightsLayout(&self) -> MPSCNNConvolutionWeightsLayout

Available on crate feature MPSNeuralNetwork only.

Layout of gradient with respect to weights in gradientForWeights buffer. Currently only MPSCNNConvolutionWeightsLayoutOHWI is supported.

Source

impl MPSCNNConvolutionGradientState

Methods declared on superclass MPSState.

pub unsafe fn temporaryStateWithCommandBuffer_bufferSize( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, buffer_size: usize, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a MPSState holding a temporary MTLBuffer

Parameter cmdBuf: The command buffer against which the temporary resource is allocated

Parameter bufferSize: The size of the buffer in bytes

pub unsafe fn temporaryStateWithCommandBuffer_textureDescriptor( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, descriptor: &MTLTextureDescriptor, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a MPSState holding a temporary MTLTexture

Parameter cmdBuf: The command buffer against which the temporary resource is allocated

Parameter descriptor: A descriptor for the new temporary texture

pub unsafe fn temporaryStateWithCommandBuffer( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a new autoreleased temporary state object without underlying resource

Parameter cmdBuf: The command buffer with which the temporary resource is associated

pub unsafe fn initWithDevice_bufferSize( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, buffer_size: usize, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithDevice_textureDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, descriptor: &MTLTextureDescriptor, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithResource( this: Allocated<Self>, resource: Option<&ProtocolObject<dyn MTLResource>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a MPSState with a non-temporary MTLResource

Parameter resource: A MTLBuffer or MTLTexture. May be nil.

Safety
  • resource may need to be synchronized.
  • resource may be unretained, you must ensure it is kept alive while in use.
pub unsafe fn init(this: Allocated<Self>) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithDevice_resourceList( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, resource_list: &MPSStateResourceList, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Initialize a non-temporary state to hold a number of textures and buffers

The allocation of each resource will be deferred until it is needed. This occurs when -resource or -resourceAtIndex: is called.

Parameter resourceList: The list of resources to create.

pub unsafe fn temporaryStateWithCommandBuffer_resourceList( command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, resource_list: &MPSStateResourceList, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Initialize a temporary state to hold a number of textures and buffers

The textures occur first in sequence

pub unsafe fn initWithResources( this: Allocated<Self>, resources: Option<&NSArray<ProtocolObject<dyn MTLResource>>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a state object with a list of MTLResources

Because MPS prefers deferred allocation of resources your application should use -initWithTextures:bufferSizes:bufferCount: whenever possible. This method is useful for cases when the MTLResources must be initialized by the CPU.

Safety
  • resources generic may need to be synchronized.
  • resources generic may be unretained, you must ensure it is kept alive while in use.
Source

impl MPSCNNConvolutionGradientState

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = MPSState>

pub unsafe fn resourceCount(&self) -> NSUInteger

Return the number of MTLResource objects held by the state

pub unsafe fn resourceAtIndex_allocateMemory( &self, index: NSUInteger, allocate_memory: bool, ) -> Option<Retained<ProtocolObject<dyn MTLResource>>>

Get the MTLResource at the indicated index

By convention, except where otherwise documented, the MTLResources held by the MPSState are private to the MPSState object, owned by the MPSState subclass author. If the MPSState subclass author is MPS, then the identity (e.g. texture vs. buffer) and information contained in the resource should be considered implementation dependent. It may change by operating system version or device. If you are the author of the subclass then it is for your own use, and MPS will not look at it, except perhaps so as to pass it to a custom kernel. Otherwise, the method is made available to facilitate debugging and to allow you to write your own state objects. Provide accessors to read this information in a defined format.

Parameter index: The index of the MTLResource to retrieve

Parameter allocateMemory: It is very important to avoid allocating memory to hold MTLResources until it is absolutely necessary, especially when working with temporary MPSStates. When allocateMemory is set to NO and the resource has not yet been allocated, nil will be returned instead. If you just need information about the resource such as buffer size or MTLTexture properties, but not the resource itself, please use -bufferSizeAtIndex: or -textureInfoAtIndex: instead, as these will not force the creation of the MTLResource.

pub unsafe fn readCount(&self) -> NSUInteger

pub unsafe fn setReadCount(&self, read_count: NSUInteger)

Setter for readCount.

pub unsafe fn isTemporary(&self) -> bool

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn bufferSizeAtIndex(&self, index: NSUInteger) -> NSUInteger

Return the buffer size of the MTLBuffer at index or 0 if it is not a MTLBuffer

Does not force allocation of the MTLResource

pub unsafe fn textureInfoAtIndex( &self, index: NSUInteger, ) -> MPSStateTextureInfo

Return the texture size {width,height,depth} or {0,0,0} if it is not a MTLTexture

Does not force allocation of the MTLResource

pub unsafe fn resourceTypeAtIndex( &self, index: NSUInteger, ) -> MPSStateResourceType

Return YES if the resource at index is a buffer

Does not force allocation of the MTLResource

pub unsafe fn synchronizeOnCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

Flush any copy of MTLResources held by the state from the device’s caches, and invalidate any CPU caches if needed.

This will call [id <MTLBlitEncoder

synchronizeResource: ] on the state’s MTLResources. This is necessary for all MTLStorageModeManaged resources. For other resources, including temporary resources (these are all MTLStorageModePrivate), nothing is done.

Parameter commandBuffer: The commandbuffer on which to synchronize

pub unsafe fn resourceSize(&self) -> NSUInteger

Get the number of bytes used to allocate underyling MTLResources

This is the size of the backing store of underlying MTLResources. It does not include all storage used by the object, for example the storage used to hold the MPSState instantiation and MTLTexture or MTLBuffer is not included. It only measures the size of the allocation used to hold the texels in the texture or bytes in the buffer. This value is subject to change between different devices and operating systems.

Except when -initWithResource: is used, most MPSStates are allocated without a backing store. The backing store is allocated lazily when it is needed, typically when the .texture property is called. Consequently, in most cases, it should be inexpensive to make a MPSImage to see how much memory it will need, and release it if it is too large.

This method may fail in certain circumstances, such as when the MPSImage is created with -initWithTexture:featureChannels:, in which case 0 will be returned.

pub unsafe fn destinationImageDescriptorForSourceImages_sourceStates_forKernel_suggestedDescriptor( &self, source_images: &NSArray<MPSImage>, source_states: Option<&NSArray<MPSState>>, kernel: &MPSKernel, in_descriptor: &MPSImageDescriptor, ) -> Retained<MPSImageDescriptor>

Available on crate features MPSImage and MPSKernel only.

Determine padding and sizing of result images

A MPSState has the opportunity to reconfigure the MPSImageDescriptor used to create the filter result state and set the MPSKernel.offset to the correct value. By default, the MPSState does not modify the descriptor.

There is a order of operations defined for who may update the descriptor:

  1. Default padding code runs based on the MPSNNPaddingMethod in the MPSCNNKernel.padding. This creates the descriptor and picks a starting value for the MPSCNNKernel.offset.
  2. MPSStates are called in order to apply this function and update the offset.
  3. The MPSNNPadding custom padding method of the same name is called.

Some code that may prove helpful:

                  const int centeringPolicy = 0;  // When kernelSize is even: 0 pad bottom right. 1 pad top left.    Centers the kernel for even sized kernels.

                  typedef enum Style{
                      StyleValidOnly = -1,
                      StyleSame = 0,
                      StyleFull = 1
                  }Style;

                  // Typical destination size in one dimension for forward filters (most filters)
                  static int DestSize( int sourceSize, int stride, int filterWindowSize, Style style ){
                      sourceSize += style * (filterWindowSize - 1);       // adjust how many pixels we are allowed to read
                      return (sourceSize + stride - 1) / stride;          // sourceSize / stride, round up
                  }

                  // Typical destination size in one dimension for reverse filters (e.g. convolution transpose)
                  static int DestSizeReverse( int sourceSize, int stride, int filterWindowSize, Style style ){
                      return (sourceSize-1) * stride +        // center tap for the last N-1 results. Take stride into account
                              1 +                             // center tap for the first result
                              style * (filterWindowSize-1);   // add or subtract (or ignore) the filter extent
                  }

                  // Find the MPSOffset in one dimension
                  static int Offset( int sourceSize, int stride, int filterWindowSize, Style style ){
                      // The correction needed to adjust from position of left edge to center per MPSOffset definition
                      int correction = filterWindowSize / 2;

                      // exit if all we want is to start consuming pixels at the left edge of the image.
                      if( 0 )
                          return correction;

                      // Center the area consumed in the source image:
                      // Calculate the size of the destination image
                      int destSize = DestSize( sourceSize, stride, filterWindowSize, style ); // use DestSizeReverse here instead as appropriate

                      // calculate extent of pixels we need to read in source to populate the destination
                      int readSize = (destSize-1) * stride + filterWindowSize;

                      // calculate number of missing pixels in source
                      int extraSize = readSize - sourceSize;

                      // number of missing pixels on left side
                      int leftExtraPixels = (extraSize + centeringPolicy) / 2;

                      // account for the fact that the offset is based on the center pixel, not the left edge
                      return correction - leftExtraPixels;
                  }

Parameter sourceImages: The list of source images to be used

Parameter sourceStates: The list of source states to be used

Parameter kernel: The MPSKernel the padding method will be applied to. Set the kernel.offset

Parameter inDescriptor: MPS will prepare a starting guess based on the padding policy (exclusive of MPSNNPaddingMethodCustom) set for the object. You should adjust the offset and image size accordingly. It is on an autoreleasepool.

Returns: The MPSImageDescriptor to use to make a MPSImage to capture the results from the filter. The MPSImageDescriptor is assumed to be on an autoreleasepool. Your method must also set the kernel.offset property.

pub unsafe fn resource( &self, ) -> Option<Retained<ProtocolObject<dyn MTLResource>>>

👎Deprecated

Get the private MTLResource underlying the MPSState

When the state is not directly initialized with a MTLResource, the actuall MTLResource creation is deferred. Especially with temporary resources, it is important to delay this creation as late as possible to avoid increasing the memory footprint. The memory is returned for reuse when the readCount = 0. Calling the -resource method will force the resource to be allocated, so you should not use it lightly, for purposes such as finding the MTLPixelFormat of a texture in the state.

By convention, except where otherwise documented, the MTLResources held by the MPSState are private to the MPSState object, owned by the MPSState subclass author. If the MPSState subclass author is MPS, then the identity (e.g. texture vs. buffer) and information contained in the resource should be considered implementation dependent. It may change by operating system version or device. If you are the author of the subclass then it is for your own use, and MPS will not look at it, except perhaps so as to pass it to a custom kernel. Otherwise, the method is made available to facilitate debugging and to allow you to write your own state objects.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}