pub struct MPSCNNLossDataDescriptor { /* private fields */ }MPSCNNLoss only.Expand description
Dependencies: This depends on Metal.framework.
The MPSCNNLossDataDescriptor specifies a loss data descriptor. The same descriptor can be used to initialize both the labels and the optional weights data.
See also Apple’s documentation
Implementations
Sourceimpl MPSCNNLossDataDescriptor
impl MPSCNNLossDataDescriptor
Sourcepub unsafe fn layout(&self) -> MPSDataLayout
Available on crate features MPSNeuralNetwork and MPSCore and MPSImage only.
pub unsafe fn layout(&self) -> MPSDataLayout
MPSNeuralNetwork and MPSCore and MPSImage only.Data layout of loss data. See MPSImage.h for more information.
This parameter specifies the layout of loss data.
Sourcepub unsafe fn size(&self) -> MTLSize
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn size(&self) -> MTLSize
MPSNeuralNetwork only.Size of loss data: (width, height, feature channels}.
This parameter specifies the size of loss data.
Sourcepub unsafe fn bytesPerRow(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn bytesPerRow(&self) -> NSUInteger
MPSNeuralNetwork only.Row bytes of loss data.
This parameter specifies the row bytes of loss data.
Sourcepub unsafe fn setBytesPerRow(&self, bytes_per_row: NSUInteger)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setBytesPerRow(&self, bytes_per_row: NSUInteger)
MPSNeuralNetwork only.Setter for bytesPerRow.
Sourcepub unsafe fn bytesPerImage(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn bytesPerImage(&self) -> NSUInteger
MPSNeuralNetwork only.Slice bytes of loss data.
This parameter specifies the slice bytes of loss data.
Sourcepub unsafe fn setBytesPerImage(&self, bytes_per_image: NSUInteger)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setBytesPerImage(&self, bytes_per_image: NSUInteger)
MPSNeuralNetwork only.Setter for bytesPerImage.
pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>
MPSNeuralNetwork only.Sourcepub unsafe fn cnnLossDataDescriptorWithData_layout_size(
data: &NSData,
layout: MPSDataLayout,
size: MTLSize,
) -> Option<Retained<MPSCNNLossDataDescriptor>>
Available on crate features MPSNeuralNetwork and MPSCore and MPSImage only.
pub unsafe fn cnnLossDataDescriptorWithData_layout_size( data: &NSData, layout: MPSDataLayout, size: MTLSize, ) -> Option<Retained<MPSCNNLossDataDescriptor>>
MPSNeuralNetwork and MPSCore and MPSImage only.Make a descriptor loss data. The bytesPerRow and bytesPerImage are automatically calculated assuming a dense array. If it is not a dense array, adjust bytesPerRow and bytesPerImage to the right value by changing properties.
Parameter data: The per-element loss data. The data must be in floating point format.
Parameter layout: The data layout of loss data.
Parameter size: The size of loss data.
Returns: A valid MPSCNNLossDataDescriptor object or nil, if failure.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl AsRef<AnyObject> for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl AsRef<MPSCNNLossDataDescriptor> for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSCNNLossDataDescriptor> for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl AsRef<NSObject> for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl AsRef<NSObject> for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl Borrow<AnyObject> for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Borrow<AnyObject> for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl Borrow<NSObject> for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Borrow<NSObject> for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl ClassType for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl ClassType for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceconst NAME: &'static str = "MPSCNNLossDataDescriptor"
const NAME: &'static str = "MPSCNNLossDataDescriptor"
Sourcetype ThreadKind = <<MPSCNNLossDataDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSCNNLossDataDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl CopyingHelper for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourcetype Result = MPSCNNLossDataDescriptor
type Result = MPSCNNLossDataDescriptor
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Debug for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl Deref for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Deref for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl Hash for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Hash for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl Message for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Message for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl NSCopying for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl NSCopying for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl NSObjectProtocol for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl NSObjectProtocol for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl PartialEq for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceimpl RefEncode for MPSCNNLossDataDescriptor
Available on crate feature MPSNeuralNetwork only.
impl RefEncode for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.impl Eq for MPSCNNLossDataDescriptor
MPSNeuralNetwork only.