MPSCNNLossDescriptor

Struct MPSCNNLossDescriptor 

pub struct MPSCNNLossDescriptor { /* private fields */ }
Available on crate feature MPSCNNLoss only.
Dependencies: This depends on Metal.framework.

The MPSCNNLossDescriptor specifies a loss filter descriptor. The same descriptor can be used to initialize both the MPSCNNLoss and the MPSNNLossGradient filters.

See also Apple’s documentation

Implementations

Source

impl MPSCNNLossDescriptor

pub unsafe fn lossType(&self) -> MPSCNNLossType

Available on crate features MPSNeuralNetwork and MPSCNNTypes only.

The type of a loss filter.

This parameter specifies the type of a loss filter.

pub unsafe fn setLossType(&self, loss_type: MPSCNNLossType)

Available on crate features MPSNeuralNetwork and MPSCNNTypes only.

Setter for lossType.

pub unsafe fn reductionType(&self) -> MPSCNNReductionType

Available on crate features MPSNeuralNetwork and MPSCNNTypes only.

The type of a reduction operation performed in the loss filter.

This parameter specifies the type of a reduction operation performed in the loss filter.

pub unsafe fn setReductionType(&self, reduction_type: MPSCNNReductionType)

Available on crate features MPSNeuralNetwork and MPSCNNTypes only.

Setter for reductionType.

pub unsafe fn reduceAcrossBatch(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

If set to YES then the reduction operation is applied also across the batch-index dimension, ie. the loss value is summed over images in the batch and the result of the reduction is written on the first loss image in the batch while the other loss images will be set to zero. If set to NO, then no reductions are performed across the batch dimension and each image in the batch will contain the loss value associated with that one particular image. NOTE: If reductionType == MPSCNNReductionTypeNone, then this flag has no effect on results, that is no reductions are done in this case. NOTE: If reduceAcrossBatch is set to YES and reductionType == MPSCNNReductionTypeMean then the final forward loss value is computed by first summing over the components and then by dividing the result with: number of feature channels * width * height * number of images in the batch. The default value is NO.

pub unsafe fn setReduceAcrossBatch(&self, reduce_across_batch: bool)

Available on crate feature MPSNeuralNetwork only.

Setter for reduceAcrossBatch.

pub unsafe fn weight(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

The scale factor to apply to each element of a result.

Each element of a result is multiplied by the weight value. The default value is 1.0f.

pub unsafe fn setWeight(&self, weight: c_float)

Available on crate feature MPSNeuralNetwork only.

Setter for weight.

pub unsafe fn labelSmoothing(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

The label smoothing parameter. The default value is 0.0f.

This parameter is valid only for the loss functions of the following type(s): MPSCNNLossFunctionTypeSoftmaxCrossEntropy, MPSCNNLossFunctionTypeSigmoidCrossEntropy.

MPSCNNLossFunctionTypeSoftmaxCrossEntropy: given labels (ground truth), it is applied in the following way: labels = labelSmoothing > 0 ? labels * (1 - labelSmoothing) + labelSmoothing / numberOfClasses : labels

MPSCNNLossFunctionTypeSigmoidCrossEntropy: given labels (ground truth), it is applied in the following way: labels = labelSmoothing > 0 ? labels * (1 - labelSmoothing) + 0.5 * labelSmoothing : labels

pub unsafe fn setLabelSmoothing(&self, label_smoothing: c_float)

Available on crate feature MPSNeuralNetwork only.

Setter for labelSmoothing.

pub unsafe fn numberOfClasses(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of classes parameter. The default value is 1.

This parameter is valid only for the loss functions of the following type(s): MPSCNNLossFunctionTypeSoftmaxCrossEntropy.

Given labels (ground truth), it is applied in the following way: labels = labelSmoothing > 0 ? labels * (1 - labelSmoothing) + labelSmoothing / numberOfClasses : labels

pub unsafe fn setNumberOfClasses(&self, number_of_classes: NSUInteger)

Available on crate feature MPSNeuralNetwork only.

Setter for numberOfClasses.

pub unsafe fn epsilon(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

The epsilon parameter. The default value is 1e-7.

This parameter is valid only for the loss functions of the following type(s): MPSCNNLossTypeLog.

Given predictions and labels (ground truth), it is applied in the following way: -(labels * log(predictions + epsilon)) - ((1 - labels) * log(1 - predictions + epsilon))

pub unsafe fn setEpsilon(&self, epsilon: c_float)

Available on crate feature MPSNeuralNetwork only.

Setter for epsilon.

pub unsafe fn delta(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

The delta parameter. The default value is 1.0f.

This parameter is valid only for the loss functions of the following type(s): MPSCNNLossTypeHuber.

Given predictions and labels (ground truth), it is applied in the following way: if (|predictions - labels| < = delta, loss = 0.5f * predictions^2 if (|predictions - labels| > delta, loss = 0.5 * delta^2 + delta * (|predictions - labels| - delta)

pub unsafe fn setDelta(&self, delta: c_float)

Available on crate feature MPSNeuralNetwork only.

Setter for delta.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn cnnLossDescriptorWithType_reductionType( loss_type: MPSCNNLossType, reduction_type: MPSCNNReductionType, ) -> Retained<MPSCNNLossDescriptor>

Available on crate features MPSNeuralNetwork and MPSCNNTypes only.

Make a descriptor for a MPSCNNLoss or MPSNNLossGradient object.

Parameter lossType: The type of a loss filter.

Parameter reductionType: The type of a reduction operation to apply. This argument is ignored in the MPSNNLossGradient filter.

Returns: A valid MPSCNNLossDescriptor object or nil, if failure.

Source

impl MPSCNNLossDescriptor

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}