MPSCNNLossLabels

Struct MPSCNNLossLabels 

pub struct MPSCNNLossLabels { /* private fields */ }
Available on crate features MPSCNNLoss and MPSCore and MPSState only.
Dependencies: This depends on Metal.framework.

The MPSCNNLossLabels is used to hold the per-element weights buffer used by both MPSCNNLoss forward filter and MPSNNLossGradient backward filter. The MPSCNNLoss forward filter populates the MPSCNNLossLabels object and the MPSNNLossGradient backward filter consumes the state object.

See also Apple’s documentation

Implementations

Source

impl MPSCNNLossLabels

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Use one of the interfaces below instead.

pub unsafe fn initWithDevice_labelsDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, labels_descriptor: &MPSCNNLossDataDescriptor, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Set labels (aka targets, ground truth) for the MPSCNNLossLabels object.

The labels and weights data are copied into internal storage. The computed loss can either be a scalar value (in batch mode, a single value per image in a batch) or it can be one value per feature channel. Thus, the size of the loss image must either match the size of the input source image or be {1, 1, 1}, which results in a scalar value. In this convinience initializer, the assumed size of the loss image is {1, 1, 1}.

Parameter device: Device the state resources will be created on.

Parameter labelsDescriptor: Describes the labels data. This includes:

  • The per-element labels data. The data must be in floating point format.
  • Data layout of labels data. See MPSImage.h for more information.
  • Size of labels data: (width, height, feature channels}.
  • Optionally, row bytes of labels data.
  • Optionally, slice bytes of labels data.
pub unsafe fn initWithDevice_lossImageSize_labelsDescriptor_weightsDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, loss_image_size: MTLSize, labels_descriptor: &MPSCNNLossDataDescriptor, weights_descriptor: Option<&MPSCNNLossDataDescriptor>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Set labels (aka targets, ground truth) and weights for the MPSCNNLossLabels object. Weights are optional.

The labels and weights data are copied into internal storage.

Parameter device: Device the state resources will be created on.

Parameter lossImageSize: The size of the resulting loss image: { width, height, featureChannels }. The computed loss can either be a scalar value (in batch mode, a single value per image in a batch) or it can be one value per feature channel. Thus, the size of the loss image must either match the size of the input source image or be {1, 1, 1}, which results in a scalar value.

Parameter labelsDescriptor: Describes the labels data. This includes:

  • The per-element labels data. The data must be in floating point format.
  • Data layout of labels data. See MPSImage.h for more information.
  • Size of labels data: (width, height, feature channels}.
  • Optionally, row bytes of labels data.
  • Optionally, slice bytes of labels data.

Parameter weightsDescriptor: Describes the weights data. This includes:

  • The per-element weights data. The data must be in floating point format.
  • Data layout of weights data. See MPSImage.h for more information.
  • Size of weights data: (width, height, feature channels}.
  • Optionally, row bytes of weights data.
  • Optionally, slice bytes of weights data. This parameter is optional. If you are using a single weight, please use the weight property of the MPSCNNLossDescriptor object.
pub unsafe fn initWithDevice_lossImageSize_labelsImage_weightsImage( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, loss_image_size: MTLSize, labels_image: &MPSImage, weights_image: Option<&MPSImage>, ) -> Retained<Self>

Available on crate features MPSNeuralNetwork and MPSImage only.

Set labels (aka targets, ground truth) and weights for the MPSCNNLossLabels object. Weights are optional.

The labels and weights images are retained - it is the users responsibility to make sure that they contain the right data when the loss filter is run on the device.

Parameter device: Device the state resources will be created on.

Parameter lossImageSize: The size of the resulting loss image: { width, height, featureChannels }. The computed loss can either be a scalar value (in batch mode, a single value per image in a batch) or it can be one value per feature channel. Thus, the size of the loss image must either match the size of the input source image or be {1, 1, 1}, which results in a scalar value.

Parameter labelsImage: Describes the labels data.

Parameter weightsImage: Describes the weights data. This parameter is optional. If you are using a single weight, please use the weight property of the MPSCNNLossDescriptor object.

pub unsafe fn lossImage(&self) -> Retained<MPSImage>

Available on crate features MPSNeuralNetwork and MPSImage only.

Loss image accessor method.

Returns: An autoreleased MPSImage object, containing the loss data. The loss data is populated in the -encode call, thus the contents are undefined until you -encode the filter.

In order to guarantee that the image is correctly synchronized for CPU side access, it is the application’s responsibility to call the [gradientState synchronizeOnCommandBuffer:] method before accessing the data in the image.

pub unsafe fn labelsImage(&self) -> Retained<MPSImage>

Available on crate features MPSNeuralNetwork and MPSImage only.

Labels image accessor method.

Returns: An autoreleased MPSImage object, containing the labels data. The labels data is populated in the -initWithDevice call.

In order to guarantee that the image is correctly synchronized for CPU side access, it is the application’s responsibility to call the [gradientState synchronizeOnCommandBuffer:] method before accessing the data in the image.

pub unsafe fn weightsImage(&self) -> Retained<MPSImage>

Available on crate features MPSNeuralNetwork and MPSImage only.

Weights image accessor method.

Returns: An autoreleased MPSImage object, containing the weights data. The weights data is populated in the -initWithDevice call.

In order to guarantee that the image is correctly synchronized for CPU side access, it is the application’s responsibility to call the [gradientState synchronizeOnCommandBuffer:] method before accessing the data in the image.

Source

impl MPSCNNLossLabels

Methods declared on superclass MPSState.

pub unsafe fn temporaryStateWithCommandBuffer_bufferSize( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, buffer_size: usize, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a MPSState holding a temporary MTLBuffer

Parameter cmdBuf: The command buffer against which the temporary resource is allocated

Parameter bufferSize: The size of the buffer in bytes

pub unsafe fn temporaryStateWithCommandBuffer_textureDescriptor( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, descriptor: &MTLTextureDescriptor, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a MPSState holding a temporary MTLTexture

Parameter cmdBuf: The command buffer against which the temporary resource is allocated

Parameter descriptor: A descriptor for the new temporary texture

pub unsafe fn temporaryStateWithCommandBuffer( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a new autoreleased temporary state object without underlying resource

Parameter cmdBuf: The command buffer with which the temporary resource is associated

pub unsafe fn initWithDevice_bufferSize( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, buffer_size: usize, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithDevice_textureDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, descriptor: &MTLTextureDescriptor, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithResource( this: Allocated<Self>, resource: Option<&ProtocolObject<dyn MTLResource>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a MPSState with a non-temporary MTLResource

Parameter resource: A MTLBuffer or MTLTexture. May be nil.

Safety
  • resource may need to be synchronized.
  • resource may be unretained, you must ensure it is kept alive while in use.
pub unsafe fn initWithDevice_resourceList( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, resource_list: &MPSStateResourceList, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Initialize a non-temporary state to hold a number of textures and buffers

The allocation of each resource will be deferred until it is needed. This occurs when -resource or -resourceAtIndex: is called.

Parameter resourceList: The list of resources to create.

pub unsafe fn temporaryStateWithCommandBuffer_resourceList( command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, resource_list: &MPSStateResourceList, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Initialize a temporary state to hold a number of textures and buffers

The textures occur first in sequence

pub unsafe fn initWithResources( this: Allocated<Self>, resources: Option<&NSArray<ProtocolObject<dyn MTLResource>>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Create a state object with a list of MTLResources

Because MPS prefers deferred allocation of resources your application should use -initWithTextures:bufferSizes:bufferCount: whenever possible. This method is useful for cases when the MTLResources must be initialized by the CPU.

Safety
  • resources generic may need to be synchronized.
  • resources generic may be unretained, you must ensure it is kept alive while in use.
Source

impl MPSCNNLossLabels

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = MPSState>

pub unsafe fn resourceCount(&self) -> NSUInteger

Return the number of MTLResource objects held by the state

pub unsafe fn resourceAtIndex_allocateMemory( &self, index: NSUInteger, allocate_memory: bool, ) -> Option<Retained<ProtocolObject<dyn MTLResource>>>

Get the MTLResource at the indicated index

By convention, except where otherwise documented, the MTLResources held by the MPSState are private to the MPSState object, owned by the MPSState subclass author. If the MPSState subclass author is MPS, then the identity (e.g. texture vs. buffer) and information contained in the resource should be considered implementation dependent. It may change by operating system version or device. If you are the author of the subclass then it is for your own use, and MPS will not look at it, except perhaps so as to pass it to a custom kernel. Otherwise, the method is made available to facilitate debugging and to allow you to write your own state objects. Provide accessors to read this information in a defined format.

Parameter index: The index of the MTLResource to retrieve

Parameter allocateMemory: It is very important to avoid allocating memory to hold MTLResources until it is absolutely necessary, especially when working with temporary MPSStates. When allocateMemory is set to NO and the resource has not yet been allocated, nil will be returned instead. If you just need information about the resource such as buffer size or MTLTexture properties, but not the resource itself, please use -bufferSizeAtIndex: or -textureInfoAtIndex: instead, as these will not force the creation of the MTLResource.

pub unsafe fn readCount(&self) -> NSUInteger

pub unsafe fn setReadCount(&self, read_count: NSUInteger)

Setter for readCount.

pub unsafe fn isTemporary(&self) -> bool

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn bufferSizeAtIndex(&self, index: NSUInteger) -> NSUInteger

Return the buffer size of the MTLBuffer at index or 0 if it is not a MTLBuffer

Does not force allocation of the MTLResource

pub unsafe fn textureInfoAtIndex( &self, index: NSUInteger, ) -> MPSStateTextureInfo

Return the texture size {width,height,depth} or {0,0,0} if it is not a MTLTexture

Does not force allocation of the MTLResource

pub unsafe fn resourceTypeAtIndex( &self, index: NSUInteger, ) -> MPSStateResourceType

Return YES if the resource at index is a buffer

Does not force allocation of the MTLResource

pub unsafe fn synchronizeOnCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

Flush any copy of MTLResources held by the state from the device’s caches, and invalidate any CPU caches if needed.

This will call [id <MTLBlitEncoder

synchronizeResource: ] on the state’s MTLResources. This is necessary for all MTLStorageModeManaged resources. For other resources, including temporary resources (these are all MTLStorageModePrivate), nothing is done.

Parameter commandBuffer: The commandbuffer on which to synchronize

pub unsafe fn resourceSize(&self) -> NSUInteger

Get the number of bytes used to allocate underyling MTLResources

This is the size of the backing store of underlying MTLResources. It does not include all storage used by the object, for example the storage used to hold the MPSState instantiation and MTLTexture or MTLBuffer is not included. It only measures the size of the allocation used to hold the texels in the texture or bytes in the buffer. This value is subject to change between different devices and operating systems.

Except when -initWithResource: is used, most MPSStates are allocated without a backing store. The backing store is allocated lazily when it is needed, typically when the .texture property is called. Consequently, in most cases, it should be inexpensive to make a MPSImage to see how much memory it will need, and release it if it is too large.

This method may fail in certain circumstances, such as when the MPSImage is created with -initWithTexture:featureChannels:, in which case 0 will be returned.

pub unsafe fn destinationImageDescriptorForSourceImages_sourceStates_forKernel_suggestedDescriptor( &self, source_images: &NSArray<MPSImage>, source_states: Option<&NSArray<MPSState>>, kernel: &MPSKernel, in_descriptor: &MPSImageDescriptor, ) -> Retained<MPSImageDescriptor>

Available on crate features MPSImage and MPSKernel only.

Determine padding and sizing of result images

A MPSState has the opportunity to reconfigure the MPSImageDescriptor used to create the filter result state and set the MPSKernel.offset to the correct value. By default, the MPSState does not modify the descriptor.

There is a order of operations defined for who may update the descriptor:

  1. Default padding code runs based on the MPSNNPaddingMethod in the MPSCNNKernel.padding. This creates the descriptor and picks a starting value for the MPSCNNKernel.offset.
  2. MPSStates are called in order to apply this function and update the offset.
  3. The MPSNNPadding custom padding method of the same name is called.

Some code that may prove helpful:

                  const int centeringPolicy = 0;  // When kernelSize is even: 0 pad bottom right. 1 pad top left.    Centers the kernel for even sized kernels.

                  typedef enum Style{
                      StyleValidOnly = -1,
                      StyleSame = 0,
                      StyleFull = 1
                  }Style;

                  // Typical destination size in one dimension for forward filters (most filters)
                  static int DestSize( int sourceSize, int stride, int filterWindowSize, Style style ){
                      sourceSize += style * (filterWindowSize - 1);       // adjust how many pixels we are allowed to read
                      return (sourceSize + stride - 1) / stride;          // sourceSize / stride, round up
                  }

                  // Typical destination size in one dimension for reverse filters (e.g. convolution transpose)
                  static int DestSizeReverse( int sourceSize, int stride, int filterWindowSize, Style style ){
                      return (sourceSize-1) * stride +        // center tap for the last N-1 results. Take stride into account
                              1 +                             // center tap for the first result
                              style * (filterWindowSize-1);   // add or subtract (or ignore) the filter extent
                  }

                  // Find the MPSOffset in one dimension
                  static int Offset( int sourceSize, int stride, int filterWindowSize, Style style ){
                      // The correction needed to adjust from position of left edge to center per MPSOffset definition
                      int correction = filterWindowSize / 2;

                      // exit if all we want is to start consuming pixels at the left edge of the image.
                      if( 0 )
                          return correction;

                      // Center the area consumed in the source image:
                      // Calculate the size of the destination image
                      int destSize = DestSize( sourceSize, stride, filterWindowSize, style ); // use DestSizeReverse here instead as appropriate

                      // calculate extent of pixels we need to read in source to populate the destination
                      int readSize = (destSize-1) * stride + filterWindowSize;

                      // calculate number of missing pixels in source
                      int extraSize = readSize - sourceSize;

                      // number of missing pixels on left side
                      int leftExtraPixels = (extraSize + centeringPolicy) / 2;

                      // account for the fact that the offset is based on the center pixel, not the left edge
                      return correction - leftExtraPixels;
                  }

Parameter sourceImages: The list of source images to be used

Parameter sourceStates: The list of source states to be used

Parameter kernel: The MPSKernel the padding method will be applied to. Set the kernel.offset

Parameter inDescriptor: MPS will prepare a starting guess based on the padding policy (exclusive of MPSNNPaddingMethodCustom) set for the object. You should adjust the offset and image size accordingly. It is on an autoreleasepool.

Returns: The MPSImageDescriptor to use to make a MPSImage to capture the results from the filter. The MPSImageDescriptor is assumed to be on an autoreleasepool. Your method must also set the kernel.offset property.

pub unsafe fn resource( &self, ) -> Option<Retained<ProtocolObject<dyn MTLResource>>>

👎Deprecated

Get the private MTLResource underlying the MPSState

When the state is not directly initialized with a MTLResource, the actuall MTLResource creation is deferred. Especially with temporary resources, it is important to delay this creation as late as possible to avoid increasing the memory footprint. The memory is returned for reuse when the readCount = 0. Calling the -resource method will force the resource to be allocated, so you should not use it lightly, for purposes such as finding the MTLPixelFormat of a texture in the state.

By convention, except where otherwise documented, the MTLResources held by the MPSState are private to the MPSState object, owned by the MPSState subclass author. If the MPSState subclass author is MPS, then the identity (e.g. texture vs. buffer) and information contained in the resource should be considered implementation dependent. It may change by operating system version or device. If you are the author of the subclass then it is for your own use, and MPS will not look at it, except perhaps so as to pass it to a custom kernel. Otherwise, the method is made available to facilitate debugging and to allow you to write your own state objects.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}