MPSImageDescriptor

Struct MPSImageDescriptor 

pub struct MPSImageDescriptor { /* private fields */ }
Available on crate feature MPSImage only.
Dependencies: This depends on Metal.framework

A MPSImageDescriptor object describes a attributes of MPSImage and is used to create one (see MPSImage discussion below)

See also Apple’s documentation

Implementations

Source

impl MPSImageDescriptor

pub unsafe fn width(&self) -> NSUInteger

Available on crate feature MPSCore only.

The width of the CNN image.

The formal width of the CNN image in pixels. Default = 1.

pub unsafe fn setWidth(&self, width: NSUInteger)

Available on crate feature MPSCore only.

Setter for width.

pub unsafe fn height(&self) -> NSUInteger

Available on crate feature MPSCore only.

The height of the CNN image.

The formal height of the CNN image in pixels. Default = 1.

pub unsafe fn setHeight(&self, height: NSUInteger)

Available on crate feature MPSCore only.

Setter for height.

pub unsafe fn featureChannels(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of feature channels per pixel. Default = 1.

pub unsafe fn setFeatureChannels(&self, feature_channels: NSUInteger)

Available on crate feature MPSCore only.

Setter for featureChannels.

pub unsafe fn numberOfImages(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of images for batch processing. Default = 1.

pub unsafe fn setNumberOfImages(&self, number_of_images: NSUInteger)

Available on crate feature MPSCore only.

Setter for numberOfImages.

pub unsafe fn pixelFormat(&self) -> MTLPixelFormat

Available on crate feature MPSCore only.

The MTLPixelFormat expected for the underlying texture.

pub unsafe fn channelFormat(&self) -> MPSImageFeatureChannelFormat

Available on crate features MPSCore and MPSCoreTypes only.

The storage format to use for each channel in the image.

pub unsafe fn setChannelFormat( &self, channel_format: MPSImageFeatureChannelFormat, )

Available on crate features MPSCore and MPSCoreTypes only.

Setter for channelFormat.

pub unsafe fn cpuCacheMode(&self) -> MTLCPUCacheMode

Available on crate feature MPSCore only.

Options to specify CPU cache mode of texture resource. Default = MTLCPUCacheModeDefaultCache

pub unsafe fn setCpuCacheMode(&self, cpu_cache_mode: MTLCPUCacheMode)

Available on crate feature MPSCore only.

Setter for cpuCacheMode.

pub unsafe fn storageMode(&self) -> MTLStorageMode

Available on crate feature MPSCore only.

To specify storage mode of texture resource.

Storage mode options:

            Default =   MTLStorageModeShared on iOS
                        MTLStorageModeManaged on Mac OSX
            MTLStorageModeShared not supported on Mac OSX.
            See Metal headers for synchronization requirements when using StorageModeManaged
pub unsafe fn setStorageMode(&self, storage_mode: MTLStorageMode)

Available on crate feature MPSCore only.

Setter for storageMode.

pub unsafe fn usage(&self) -> MTLTextureUsage

Available on crate feature MPSCore only.

Description of texture usage. Default = MTLTextureUsageShaderRead/Write

pub unsafe fn setUsage(&self, usage: MTLTextureUsage)

Available on crate feature MPSCore only.

Setter for usage.

pub unsafe fn imageDescriptorWithChannelFormat_width_height_featureChannels( channel_format: MPSImageFeatureChannelFormat, width: NSUInteger, height: NSUInteger, feature_channels: NSUInteger, ) -> Retained<Self>

Available on crate features MPSCore and MPSCoreTypes only.

Create a MPSImageDescriptor for a single read/write cnn image.

pub unsafe fn imageDescriptorWithChannelFormat_width_height_featureChannels_numberOfImages_usage( channel_format: MPSImageFeatureChannelFormat, width: NSUInteger, height: NSUInteger, feature_channels: NSUInteger, number_of_images: NSUInteger, usage: MTLTextureUsage, ) -> Retained<Self>

Available on crate features MPSCore and MPSCoreTypes only.

Create a MPSImageDescriptor for a read/write cnn image with option to set usage and batch size (numberOfImages).

pub unsafe fn copyWithZone(&self, zone: *mut NSZone) -> Retained<Self>

Available on crate feature MPSCore only.
Safety

zone must be a valid pointer or null.

Source

impl MPSImageDescriptor

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSCore only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSCore only.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}