MPSImageEdgeMode

Struct MPSImageEdgeMode 

#[repr(transparent)]
pub struct MPSImageEdgeMode(pub NSUInteger);
Available on crate feature MPSCoreTypes only.
Tuple Fields

0: NSUInteger

Implementations

Source

impl MPSImageEdgeMode

pub const Zero: Self

Available on crate feature MPSCore only.

Out of bound pixels are (0,0,0,1) for image with pixel format without alpha channel and (0,0,0,0) for image with pixel format that has an alpha channel

pub const Clamp: Self

Available on crate feature MPSCore only.

Out of bound pixels are clamped to nearest edge pixel

pub const Mirror: Self

Available on crate feature MPSCore only.

Out of bound pixels are mirrored wrt. the nearest edge pixel center - ie. the edge of the image is not repeated. NOTE: The only filter that currently supports this mode is MPSNNPad- using this with other filters results in undefined behavior.

pub const MirrorWithEdge: Self

Available on crate feature MPSCore only.

Out of bound pixels are mirrored wrt. the nearest edge pixel nearest border - ie. the edge of the image is repeated. NOTE: The only filter that currently supports this mode is MPSNNPad- using this with other filters results in undefined behavior.

pub const Constant: Self

Available on crate feature MPSCore only.

Out of bound pixels are filled with a constant value defined by the filter. NOTE: The only filter that currently supports this mode is MPSNNPad- using this with other filters results in undefined behavior.