#[repr(transparent)]pub struct MPSImageFeatureChannelFormat(pub NSUInteger);Available on crate feature
MPSCoreTypes only.Expand description
Tuple Fields
0: NSUIntegerImplementations
Sourceimpl MPSImageFeatureChannelFormat
impl MPSImageFeatureChannelFormat
Sourcepub const None: Self
Available on crate feature MPSCore only.
pub const None: Self
MPSCore only.No format. This can mean according to context invalid format or any format. In the latter case, it is an invitation to MPS to pick a format.
Sourcepub const Unorm8: Self
Available on crate feature MPSCore only.
pub const Unorm8: Self
MPSCore only.uint8_t with value [0,255] encoding [0,1.0]
Sourcepub const Unorm16: Self
Available on crate feature MPSCore only.
pub const Unorm16: Self
MPSCore only.uint16_t with value [0,65535] encoding [0,1.0]
Sourcepub const Float16: Self
Available on crate feature MPSCore only.
pub const Float16: Self
MPSCore only.IEEE-754 16-bit floating-point value. “half precision” Representable normal range is +-[2**-14, 65504], 0, Infinity, NaN. 11 bits of precision + exponent.
Sourcepub const Float32: Self
Available on crate feature MPSCore only.
pub const Float32: Self
MPSCore only.IEEE-754 32-bit floating-point value. “single precision” (standard float type in C) 24 bits of precision + exponent
Sourcepub const _reserved0: Self
Available on crate feature MPSCore only.
pub const _reserved0: Self
MPSCore only.Reserved for later expansion