pub struct MPSImageFindKeypoints { /* private fields */ }MPSCore and MPSImageKeypoint and MPSKernel only.Expand description
The MPSImageFindKeypoints kernel is used to find a list of keypoints whose values are >= minimumPixelThresholdValue in MPSImageKeypointRangeInfo. The keypoints are generated for a specified region in the image. The pixel format of the source image must be MTLPixelFormatR8Unorm.
See also Apple’s documentation
Implementations
Sourceimpl MPSImageFindKeypoints
impl MPSImageFindKeypoints
Sourcepub unsafe fn keypointRangeInfo(&self) -> MPSImageKeypointRangeInfo
Available on crate feature MPSImage only.
pub unsafe fn keypointRangeInfo(&self) -> MPSImageKeypointRangeInfo
MPSImage only.Return a structure describing the keypoint range info
Returns a MPSImageKeypointRangeInfo structure
Sourcepub unsafe fn initWithDevice_info(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
info: NonNull<MPSImageKeypointRangeInfo>,
) -> Retained<Self>
Available on crate feature MPSImage only.
pub unsafe fn initWithDevice_info( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, info: NonNull<MPSImageKeypointRangeInfo>, ) -> Retained<Self>
MPSImage only.Specifies information to find keypoints in an image.
Parameter device: The device the filter will run on
Parameter info: Pointer to the MPSImageKeypointRangeInfo struct
Returns: A valid MPSImageFindKeypoints object or nil, if failure.
Safety
info must be a valid pointer.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSImage only.Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSImage only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSImage only.NSSecureCoding compatability
While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.
Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel
Parameter device: The MTLDevice on which to make the MPSKernel
Returns: A new MPSKernel object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn encodeToCommandBuffer_sourceTexture_regions_numberOfRegions_keypointCountBuffer_keypointCountBufferOffset_keypointDataBuffer_keypointDataBufferOffset(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source: &ProtocolObject<dyn MTLTexture>,
regions: NonNull<MTLRegion>,
number_of_regions: NSUInteger,
keypoint_count_buffer: &ProtocolObject<dyn MTLBuffer>,
keypoint_count_buffer_offset: NSUInteger,
keypoint_data_buffer: &ProtocolObject<dyn MTLBuffer>,
keypoint_data_buffer_offset: NSUInteger,
)
Available on crate feature MPSImage only.
pub unsafe fn encodeToCommandBuffer_sourceTexture_regions_numberOfRegions_keypointCountBuffer_keypointCountBufferOffset_keypointDataBuffer_keypointDataBufferOffset( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source: &ProtocolObject<dyn MTLTexture>, regions: NonNull<MTLRegion>, number_of_regions: NSUInteger, keypoint_count_buffer: &ProtocolObject<dyn MTLBuffer>, keypoint_count_buffer_offset: NSUInteger, keypoint_data_buffer: &ProtocolObject<dyn MTLBuffer>, keypoint_data_buffer_offset: NSUInteger, )
MPSImage only.Encode the filter to a command buffer using a MTLComputeCommandEncoder.
The filter will not begin to execute until after the command buffer has been enqueued and committed.
Parameter commandBuffer: A valid MTLCommandBuffer.
Parameter source: A valid MTLTexture containing the source image for the filter.
Parameter regions: An array of rectangles that describe regions in the image.
The list of keypoints is generated for each individual rectangle specifed.
Parameter keypointCountBuffer: The list of keypoints for each specified region
Parameter keypointCountBufferOffset: Byte offset into keypointCountBufferOffset buffer at which to write the keypoint results.
Must be a multiple of 32 bytes.
Parameter keypointDataBuffer: A valid MTLBuffer to receive the keypoint data results for each rectangle.
The keypoint data for keypoints in each rectangle are stored consecutively.
The keypoint data for each rectangle starts at the following offset:
MPSImageKeypointRangeInfo.maximumKeyPoints * rectangle index
Parameter keypointDataBufferOffset: Byte offset into keypointData buffer at which to write the keypoint results.
Must be a multiple of 32 bytes.
Safety
sourcemay need to be synchronized.sourcemay be unretained, you must ensure it is kept alive while in use.regionsmust be a valid pointer.keypoint_count_buffermay need to be synchronized.keypoint_count_buffermay be unretained, you must ensure it is kept alive while in use.keypoint_count_buffercontents should be of the correct type.keypoint_data_buffermay need to be synchronized.keypoint_data_buffermay be unretained, you must ensure it is kept alive while in use.keypoint_data_buffercontents should be of the correct type.
Sourceimpl MPSImageFindKeypoints
Methods declared on superclass MPSKernel.
impl MPSImageFindKeypoints
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSImage only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSImage only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl AsRef<AnyObject> for MPSImageFindKeypoints
MPSImage only.Sourceimpl AsRef<MPSImageFindKeypoints> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl AsRef<MPSImageFindKeypoints> for MPSImageFindKeypoints
MPSImage only.Sourceimpl AsRef<MPSKernel> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl AsRef<MPSKernel> for MPSImageFindKeypoints
MPSImage only.Sourceimpl AsRef<NSObject> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl AsRef<NSObject> for MPSImageFindKeypoints
MPSImage only.Sourceimpl Borrow<AnyObject> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Borrow<AnyObject> for MPSImageFindKeypoints
MPSImage only.Sourceimpl Borrow<MPSKernel> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Borrow<MPSKernel> for MPSImageFindKeypoints
MPSImage only.Sourceimpl Borrow<NSObject> for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Borrow<NSObject> for MPSImageFindKeypoints
MPSImage only.Sourceimpl ClassType for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl ClassType for MPSImageFindKeypoints
MPSImage only.Sourceconst NAME: &'static str = "MPSImageFindKeypoints"
const NAME: &'static str = "MPSImageFindKeypoints"
Sourcetype ThreadKind = <<MPSImageFindKeypoints as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSImageFindKeypoints as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl CopyingHelper for MPSImageFindKeypoints
MPSImage only.Sourcetype Result = MPSImageFindKeypoints
type Result = MPSImageFindKeypoints
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Debug for MPSImageFindKeypoints
MPSImage only.Sourceimpl Deref for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Deref for MPSImageFindKeypoints
MPSImage only.Sourceimpl Hash for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Hash for MPSImageFindKeypoints
MPSImage only.Sourceimpl Message for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl Message for MPSImageFindKeypoints
MPSImage only.Sourceimpl NSCoding for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl NSCoding for MPSImageFindKeypoints
MPSImage only.Sourceimpl NSCopying for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl NSCopying for MPSImageFindKeypoints
MPSImage only.Sourceimpl NSObjectProtocol for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl NSObjectProtocol for MPSImageFindKeypoints
MPSImage only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl NSSecureCoding for MPSImageFindKeypoints
MPSImage only.Sourceimpl PartialEq for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl PartialEq for MPSImageFindKeypoints
MPSImage only.Sourceimpl RefEncode for MPSImageFindKeypoints
Available on crate feature MPSImage only.
impl RefEncode for MPSImageFindKeypoints
MPSImage only.Sourceconst ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSImageFindKeypoints
MPSImage only.impl Eq for MPSImageFindKeypoints
MPSImage only.