pub struct MPSImageHistogram { /* private fields */ }MPSCore and MPSImageHistogram and MPSKernel only.Expand description
The MPSImageHistogram computes the histogram of an image.
See also Apple’s documentation
Implementations
Sourceimpl MPSImageHistogram
impl MPSImageHistogram
Sourcepub unsafe fn clipRectSource(&self) -> MTLRegion
Available on crate feature MPSImage only.
pub unsafe fn clipRectSource(&self) -> MTLRegion
MPSImage only.The source rectangle to use when reading data.
A MTLRegion that indicates which part of the source to read. If the clipRectSource does not lie completely within the source image, the intersection of the image bounds and clipRectSource will be used. The clipRectSource replaces the MPSUnaryImageKernel offset parameter for this filter. The latter is ignored. Default: MPSRectNoClip, use the entire source texture.
Sourcepub unsafe fn setClipRectSource(&self, clip_rect_source: MTLRegion)
Available on crate feature MPSImage only.
pub unsafe fn setClipRectSource(&self, clip_rect_source: MTLRegion)
MPSImage only.Setter for clipRectSource.
Sourcepub unsafe fn zeroHistogram(&self) -> bool
Available on crate feature MPSImage only.
pub unsafe fn zeroHistogram(&self) -> bool
MPSImage only.Zero-initalize the histogram results
Indicates that the memory region in which the histogram results are to be written in the histogram buffer are to be zero-initialized or not. Default: YES.
Sourcepub unsafe fn setZeroHistogram(&self, zero_histogram: bool)
Available on crate feature MPSImage only.
pub unsafe fn setZeroHistogram(&self, zero_histogram: bool)
MPSImage only.Setter for zeroHistogram.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSImage only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSImage only.NSSecureCoding compatability
While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.
Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel
Parameter device: The MTLDevice on which to make the MPSKernel
Returns: A new MPSKernel object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn encodeToCommandBuffer_sourceTexture_histogram_histogramOffset(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source: &ProtocolObject<dyn MTLTexture>,
histogram: &ProtocolObject<dyn MTLBuffer>,
histogram_offset: NSUInteger,
)
Available on crate feature MPSImage only.
pub unsafe fn encodeToCommandBuffer_sourceTexture_histogram_histogramOffset( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source: &ProtocolObject<dyn MTLTexture>, histogram: &ProtocolObject<dyn MTLBuffer>, histogram_offset: NSUInteger, )
MPSImage only.Encode the filter to a command buffer using a MTLComputeCommandEncoder.
The filter will not begin to execute until after the command buffer has been enqueued and committed.
Parameter commandBuffer: A valid MTLCommandBuffer.
Parameter source: A valid MTLTexture containing the source image for the filter
Parameter histogram: A valid MTLBuffer to receive the histogram results.
Parameter histogramOffset: Byte offset into histogram buffer at which to write the histogram results. Must be a multiple of 32 bytes.
The histogram results / channel are stored together. The number of channels for which
histogram results are stored is determined by the number of channels in the image.
If histogramInfo.histogramForAlpha is false and the source image is RGBA then only histogram
results for RGB channels are stored.
The histogram results are stored in the histogram buffer as follows:
- histogram results for the R channel for all bins followed by
- histogram results for the G channel for all bins followed by
- histogram results for the B channel for all bins followed by
- histogram results for the A channel for all bins
Safety
sourcemay need to be synchronized.sourcemay be unretained, you must ensure it is kept alive while in use.histogrammay need to be synchronized.histogrammay be unretained, you must ensure it is kept alive while in use.histogramcontents should be of the correct type.
Sourcepub unsafe fn histogramSizeForSourceFormat(
&self,
source_format: MTLPixelFormat,
) -> usize
Available on crate feature MPSImage only.
pub unsafe fn histogramSizeForSourceFormat( &self, source_format: MTLPixelFormat, ) -> usize
MPSImage only.The amount of space in the output MTLBuffer the histogram will take up.
This convenience function calculates the minimum amount of space needed in the output histogram for the results. The MTLBuffer should be at least this length, longer if histogramOffset is non-zero.
Parameter sourceFormat: The MTLPixelFormat of the source image. This is
the source parameter of -encodeToCommandBuffer:
sourceTexture:histogram:histogramOffset
Returns: The number of bytes needed to store the result histograms.
Sourceimpl MPSImageHistogram
Methods declared on superclass MPSKernel.
impl MPSImageHistogram
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithDevice(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
Available on crate feature MPSImage only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSImage only.Standard init with default properties per filter type
Parameter device: The device that the filter will be used on. May not be NULL.
Returns: a pointer to the newly initialized object. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSImage only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSImage only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSImageHistogram
Available on crate feature MPSImage only.
impl AsRef<AnyObject> for MPSImageHistogram
MPSImage only.Sourceimpl AsRef<MPSImageHistogram> for MPSImageHistogram
Available on crate feature MPSImage only.
impl AsRef<MPSImageHistogram> for MPSImageHistogram
MPSImage only.Sourceimpl AsRef<MPSKernel> for MPSImageHistogram
Available on crate feature MPSImage only.
impl AsRef<MPSKernel> for MPSImageHistogram
MPSImage only.Sourceimpl AsRef<NSObject> for MPSImageHistogram
Available on crate feature MPSImage only.
impl AsRef<NSObject> for MPSImageHistogram
MPSImage only.Sourceimpl Borrow<AnyObject> for MPSImageHistogram
Available on crate feature MPSImage only.
impl Borrow<AnyObject> for MPSImageHistogram
MPSImage only.Sourceimpl Borrow<MPSKernel> for MPSImageHistogram
Available on crate feature MPSImage only.
impl Borrow<MPSKernel> for MPSImageHistogram
MPSImage only.Sourceimpl Borrow<NSObject> for MPSImageHistogram
Available on crate feature MPSImage only.
impl Borrow<NSObject> for MPSImageHistogram
MPSImage only.Sourceimpl ClassType for MPSImageHistogram
Available on crate feature MPSImage only.
impl ClassType for MPSImageHistogram
MPSImage only.Sourceconst NAME: &'static str = "MPSImageHistogram"
const NAME: &'static str = "MPSImageHistogram"
Sourcetype ThreadKind = <<MPSImageHistogram as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSImageHistogram as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSImageHistogram
Available on crate feature MPSImage only.
impl CopyingHelper for MPSImageHistogram
MPSImage only.Sourcetype Result = MPSImageHistogram
type Result = MPSImageHistogram
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSImageHistogram
Available on crate feature MPSImage only.
impl Debug for MPSImageHistogram
MPSImage only.Sourceimpl Deref for MPSImageHistogram
Available on crate feature MPSImage only.
impl Deref for MPSImageHistogram
MPSImage only.Sourceimpl Hash for MPSImageHistogram
Available on crate feature MPSImage only.
impl Hash for MPSImageHistogram
MPSImage only.Sourceimpl Message for MPSImageHistogram
Available on crate feature MPSImage only.
impl Message for MPSImageHistogram
MPSImage only.Sourceimpl NSCoding for MPSImageHistogram
Available on crate feature MPSImage only.
impl NSCoding for MPSImageHistogram
MPSImage only.Sourceimpl NSCopying for MPSImageHistogram
Available on crate feature MPSImage only.
impl NSCopying for MPSImageHistogram
MPSImage only.Sourceimpl NSObjectProtocol for MPSImageHistogram
Available on crate feature MPSImage only.
impl NSObjectProtocol for MPSImageHistogram
MPSImage only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSImageHistogram
Available on crate feature MPSImage only.
impl NSSecureCoding for MPSImageHistogram
MPSImage only.Sourceimpl PartialEq for MPSImageHistogram
Available on crate feature MPSImage only.
impl PartialEq for MPSImageHistogram
MPSImage only.Sourceimpl RefEncode for MPSImageHistogram
Available on crate feature MPSImage only.
impl RefEncode for MPSImageHistogram
MPSImage only.Sourceconst ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSImageHistogram
MPSImage only.impl Eq for MPSImageHistogram
MPSImage only.