MPSImageReduceRowSum

Struct MPSImageReduceRowSum 

pub struct MPSImageReduceRowSum { /* private fields */ }
Available on crate features MPSCore and MPSImageKernel and MPSImageReduce and MPSKernel only.
The MPSImageReduceRowSum performs a reduction operation returning the sum for each row of an image

See also Apple’s documentation

Implementations

Source

impl MPSImageReduceRowSum

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSImage only.

Specifies information to apply the reduction operation on an image.

Parameter device: The device the filter will run on

Returns: A valid MPSImageReduce object or nil, if failure.

Source

impl MPSImageReduceRowSum

Methods declared on superclass MPSUnaryImageKernel.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSImage only.

NSSecureCoding compatability

While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.

Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel

Parameter device: The MTLDevice on which to make the MPSKernel

Returns: A new MPSKernel object, or nil if failure.

Safety

a_decoder possibly has further requirements.

Source

impl MPSImageReduceRowSum

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSImage only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSImageReduceRowSum

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSImage only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSImage only.

Methods from Deref<Target = MPSImageReduceUnary>

pub unsafe fn clipRectSource(&self) -> MTLRegion

Available on crate feature MPSImage only.

The source rectangle to use when reading data.

A MTLRegion that indicates which part of the source to read. If the clipRectSource does not lie completely within the source image, the intersection of the image bounds and clipRectSource will be used. The clipRectSource replaces the MPSUnaryImageKernel offset parameter for this filter. The latter is ignored. Default: MPSRectNoClip, use the entire source texture.

The clipRect specified in MPSUnaryImageKernel is used to control the origin in the destination texture where the min, max values are written. The clipRect.width must be >=2. The clipRect.height must be >= 1.

pub unsafe fn setClipRectSource(&self, clip_rect_source: MTLRegion)

Available on crate feature MPSImage only.

Setter for clipRectSource.

Methods from Deref<Target = MPSUnaryImageKernel>

pub unsafe fn offset(&self) -> MPSOffset

Available on crate features MPSImage and MPSCoreTypes only.

The position of the destination clip rectangle origin relative to the source buffer.

The offset is defined to be the position of clipRect.origin in source coordinates. Default: {0,0,0}, indicating that the top left corners of the clipRect and source image align.

See Also: MetalPerformanceShaders.hsubsubsection_mpsoffset

pub unsafe fn setOffset(&self, offset: MPSOffset)

Available on crate features MPSImage and MPSCoreTypes only.

Setter for offset.

pub unsafe fn clipRect(&self) -> MTLRegion

Available on crate feature MPSImage only.

An optional clip rectangle to use when writing data. Only the pixels in the rectangle will be overwritten.

A MTLRegion that indicates which part of the destination to overwrite. If the clipRect does not lie completely within the destination image, the intersection between clip rectangle and destination bounds is used. Default: MPSRectNoClip (MPSKernel::MPSRectNoClip) indicating the entire image.

See Also: MetalPerformanceShaders.hsubsubsection_clipRect

pub unsafe fn setClipRect(&self, clip_rect: MTLRegion)

Available on crate feature MPSImage only.

Setter for clipRect.

pub unsafe fn edgeMode(&self) -> MPSImageEdgeMode

Available on crate features MPSImage and MPSCoreTypes only.

The MPSImageEdgeMode to use when texture reads stray off the edge of an image

Most MPSKernel objects can read off the edge of the source image. This can happen because of a negative offset property, because the offset + clipRect.size is larger than the source image or because the filter looks at neighboring pixels, such as a Convolution or morphology filter. Default: usually MPSImageEdgeModeZero. (Some MPSKernel types default to MPSImageEdgeModeClamp, because MPSImageEdgeModeZero is either not supported or would produce unexpected results.)

See Also: MetalPerformanceShaders.hsubsubsection_edgemode

pub unsafe fn setEdgeMode(&self, edge_mode: MPSImageEdgeMode)

Available on crate features MPSImage and MPSCoreTypes only.

Setter for edgeMode.

pub unsafe fn encodeToCommandBuffer_inPlaceTexture_fallbackCopyAllocator( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, texture: NonNull<NonNull<ProtocolObject<dyn MTLTexture>>>, copy_allocator: MPSCopyAllocator, ) -> bool

Available on crate features MPSImage and block2 only.

This method attempts to apply the MPSKernel in place on a texture.

In-place operation means that the same texture is used both to hold the input image and the results. Operating in-place can be an excellent way to reduce resource utilization, and save time and energy. While simple Metal kernels can not operate in place because textures can not be readable and writable at the same time, some MPSKernels can operate in place because they use multi-pass algorithms. Whether a MPSKernel can operate in-place can depend on current hardware, operating system revision and the parameters and properties passed to it. You should never assume that a MPSKernel will continue to work in place, even if you have observed it doing so before.

If the operation succeeds in-place, YES is returned. If the in-place operation fails and no copyAllocator is provided, then NO is returned. Without a fallback MPSCopyAllocator, in neither case is the pointer held at *texture modified.

Failure during in-place operation is very common and will occur inconsistently across different hardware platforms and OS releases. Without a fallback MPSCopyAllocator, operating in place may require significant error handling code to accompany each call to -encodeToCommandBuffer:…, complicating your code.

You may find it simplifies your code to provide a fallback MPSCopyAllocator so that the operation can proceed reliably even when it can not proceed in-place. When an in-place filter fails, the MPSCopyAllocator (if any) will be invoked to create a new texture in which to write the results, allowing the filter to proceed reliably out-of-place. The original texture will be released, replaced with a pointer to the new texture and YES will be returned. If the allocator returns an invalid texture, it is released, *texture remains unmodified and NO is returned. Please see the MPSCopyAllocator definition for a sample allocator implementation.

Sample usage with a copy allocator:

  id <MTLTexture> inPlaceTex = ...;
  MPSImageSobel *sobelFiler = [[MPSImageSobel alloc] initWithDevice: myDevice];

  // With a fallback MPSCopyAllocator, failure should only occur in exceptional
  // conditions such as MTLTexture allocation failure or programmer error.
  // That is, the operation is roughly as robust as the MPSCopyAllocator.
  // Depending on the quality of that, we might decide we are justified here
  // in not checking the return value.
  [sobelFilter encodeToCommandBuffer: myCommandBuffer
                      inPlaceTexture: &inPlaceTex  // may be replaced!
               fallbackCopyAllocator: myAllocator];

  // If myAllocator was not called:
  //
  //      inPlaceTex holds the original texture with the result pixels in it
  //
  // else,
  //
  //      1) myAllocator creates a new texture.
  //      2) The new texture pixel data is overwritten by MPSUnaryImageKernel.
  //      3) The old texture passed in *inPlaceTex is released once.
  //      4) *inPlaceTex = the new texture
  //
  // In either case, the caller should now hold one reference to the texture now held in
  // inPlaceTex, whether it was replaced or not. Most of the time that means that nothing
  // further needs to be done here, and you can proceed to the next image encoding operation.
  // However, if other agents held references to the original texture, they still hold them
  // and may need to be alerted that the texture has been replaced so that they can retain
  // the new texture and release the old one.

  [sobelFilter release];  // if not ARC, clean up the MPSImageSobel object

Note: Image filters that look at neighboring pixel values may actually consume more memory when operating in place than out of place. Many such operations are tiled internally to save intermediate texture storage, but can not tile when operating in place. The memory savings for tiling is however very short term, typically the lifetime of the MTLCommandBuffer.

Attempt to apply a MPSKernel to a texture in place.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter texture: A pointer to a valid MTLTexture containing source image. On success, the image contents and possibly texture itself will be replaced with the result image.

Parameter copyAllocator: An optional block to allocate a new texture to hold the results, in case in-place operation is not possible. The allocator may use a different MTLPixelFormat or size than the original texture. You may enqueue operations on the provided MTLCommandBuffer using the provided MTLComputeCommandEncoder to initialize the texture contents.

Returns: On success, YES is returned. The texture may have been replaced with a new texture if a copyAllocator was provided. On failure, NO is returned. The texture is unmodified.

Safety
  • texture must be a valid pointer.
  • copy_allocator must be a valid pointer or null.
pub unsafe fn encodeToCommandBuffer_sourceTexture_destinationTexture( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_texture: &ProtocolObject<dyn MTLTexture>, destination_texture: &ProtocolObject<dyn MTLTexture>, )

Available on crate feature MPSImage only.

Encode a MPSKernel into a command Buffer. The operation shall proceed out-of-place.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter sourceTexture: A valid MTLTexture containing the source image.

Parameter destinationTexture: A valid MTLTexture to be overwritten by result image. DestinationTexture may not alias sourceTexture.

Safety
  • source_texture may need to be synchronized.
  • source_texture may be unretained, you must ensure it is kept alive while in use.
  • destination_texture may need to be synchronized.
  • destination_texture may be unretained, you must ensure it is kept alive while in use.
pub unsafe fn encodeToCommandBuffer_sourceImage_destinationImage( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_image: &MPSImage, destination_image: &MPSImage, )

Available on crate feature MPSImage only.

Encode a MPSKernel into a command Buffer. The operation shall proceed out-of-place.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter sourceImage: A valid MPSImage containing the source image.

Parameter destinationImage: A valid MPSImage to be overwritten by result image. DestinationImage may not alias sourceImage.

pub unsafe fn sourceRegionForDestinationSize( &self, destination_size: MTLSize, ) -> MPSRegion

Available on crate features MPSImage and MPSCoreTypes only.

sourceRegionForDestinationSize: is used to determine which region of the sourceTexture will be read by encodeToCommandBuffer:sourceTexture:destinationTexture (and similar) when the filter runs. This information may be needed if the source image is broken into multiple textures. The size of the full (untiled) destination image is provided. The region of the full (untiled) source image that will be read is returned. You can then piece together an appropriate texture containing that information for use in your tiled context.

The function will consult the MPSUnaryImageKernel offset and clipRect parameters, to determine the full region read by the function. Other parameters such as sourceClipRect, kernelHeight and kernelWidth will be consulted as necessary. All properties should be set to intended values prior to calling sourceRegionForDestinationSize:.

Caution: This function operates using global image coordinates, but -encodeToCommandBuffer:… uses coordinates local to the source and destination image textures. Consequently, the offset and clipRect attached to this object will need to be updated using a global to local coordinate transform before -encodeToCommandBuffer:… is called.

Determine the region of the source texture that will be read for a encode operation

Parameter destinationSize: The size of the full virtual destination image.

Returns: The area in the virtual source image that will be read.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}