MPSInstanceAccelerationStructure

Struct MPSInstanceAccelerationStructure 

pub struct MPSInstanceAccelerationStructure { /* private fields */ }
👎Deprecated
Available on crate features MPSAccelerationStructure and MPSCore and MPSInstanceAccelerationStructure and MPSKernel only.
An acceleration structure built over instances of other acceleration structures

Instancing can be used to reduce memory usage in scenes that contain many copies of the same object(s) or to combine multiple acceleration structures such as a static and dynamic acceleration structure into a two-level instance hierarchy.

The typical pattern for creating an instance acceleration structure is as follows. First, create individual bottom-level acceleration structures. Then assign these acceleration structures to the accelerationStructures property of an instance acceleration structure.

All of the acceleration structures in the instance hierarchy must share the same MPSAccelerationStructureGroup. Furthermore, all of the bottom-level acceleration structures must share the same vertex buffer, index buffer, etc. although they may have different offsets within those buffers.

     MPSAccelerationStructureGroup *group = nil;
     group = [[MPSAccelerationStructureGroup alloc] initWithDevice:device];

     MPSInstanceAccelerationStructure *instanceAccel = nil;
     instanceAccel = [[MPSInstanceAccelerationStructure alloc] initWithGroup:group];

     NSMutableArray *accelerationStructures = [NSMutableArray array];
     instanceAccel.accelerationStructures = accelerationStructures;

     instanceAccel.instanceCount = instanceCount;

     for (ObjectType *objectType in objectTypes) {
         MPSTriangleAccelerationStructure *triAccel = nil;
         triAccel = [[MPSTriangleAccelerationStructure alloc] initWithGroup:group];

         triAccel.vertexBuffer = objectType.vertexBuffer;
         triAccel.vertexBufferOffset = objectType.vertexBufferOffset;
         triAccel.triangleCount = objectType.triangleCount;

         [triAccel rebuild];

         [accelerationStructures addObject:triAccel];
     }

Next, create a buffer containing the acceleration structure index for each instance, and another acceleration structure containing the transformation matrix for each instance:

     NSUInteger instanceBufferLength = sizeof(uint32_t) * instanceCount;
     
     id <MTLBuffer> instanceBuffer =
         [device newBufferWithLength:instanceBufferLength
                             options:MTLResourceStorageModeManaged];
     
     memcpy(instanceBuffer.contents, instances,
         instanceBufferLength);
     [instanceBuffer
         didModifyRange:NSMakeRange(0, instanceBufferLength)];
     
     instanceAccel.instanceBuffer = instanceBuffer;

     // Similar for transformation matrix buffer

Finally, rebuild the instance acceleration structure:

     [instanceAccel rebuild];

Refitting and Rebuilding Bottom-Level Acceleration Structures: when a bottom level acceleration structure is rebuild or refit, its’ bounding box may change. Therefore, the instance acceleration structure also needs to be rebuilt or refit.

Copying and Serializing Instance Acceleration Structures: When an instance acceleration structure is copied or serialized, the bottom level acceleration structures are not copied or serialized. These must be copied or serialized along with the instance acceleration structure and assigned to the new instance acceleration structure. This also applies to buffer properties such as the instance buffer, transformation buffer, etc.

Performance Guidelines:

  • Use instancing to reduce memory usage: if there are many copies of the same object(s) in a scene, using instances of the same object can reduce memory usage and acceleration structure build time. Rebuilding or refitting the top level acceleration structure can also be much faster than rebuilding a large single level acceleration structure.

  • Consider flattening your instance hierarchy into a single acceleration structure if the increased memory usage and acceleration structure build time are not a concern. Intersecting a two level acceleration structure can have a significant performance cost so only use it when necessary. Which technique to use depends on the scene and use case. For example, in a rendering application, it may be best to use an instance hierarchy for interactive scene editing and preview and flattening the instance hierarchy for the final render. For smaller scenes, it may also be sufficient to refit a flattened acceleration structure rather than rebuilding an instance hierarchy.

  • If there is only a single object in the scene, intersect its acceleration structure directly instead of using an instance hierarchy.

  • Consider dividing objects into static and dynamic acceleration structures. If dynamic objects require the acceleration structure to be rebuilt frequently, create a high quality static acceleration structure and a lower quality but faster to build dynamic acceleration structure. These two acceleration structures can then be combined with a two level acceleration structure. Use MPSTransformTypeIdentity to reduce the overhead of this technique. Whether this technique is more efficient than rebuilding the entire acceleration structure depends on the scene.

See MPSAccelerationStructure for more information

See also Apple’s documentation

Implementations

Source

impl MPSInstanceAccelerationStructure

pub unsafe fn accelerationStructures( &self, ) -> Option<Retained<NSArray<MPSPolygonAccelerationStructure>>>

👎Deprecated
Available on crate features MPSRayIntersector and MPSPolygonAccelerationStructure only.

Acceleration structures available for use in this instance acceleration structure. Each instance must provide an index into this array in the instance buffer as well as a transformation matrix in the transform buffer. All acceleration structures must share a single vertex buffer, optional index buffer, and optional mask buffer, though they may have different offsets within each buffer, and all acceleration structures must share the same acceleration structure group. If a polygon acceleration structure is rebuilt or refit, the instance acceleration structure must subsequently be rebuilt or refit.

pub unsafe fn setAccelerationStructures( &self, acceleration_structures: Option<&NSArray<MPSPolygonAccelerationStructure>>, )

👎Deprecated
Available on crate features MPSRayIntersector and MPSPolygonAccelerationStructure only.
pub unsafe fn instanceBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Buffer containing the 32 bit unsigned integer index into the acceleration structure array for each instance

pub unsafe fn setInstanceBuffer( &self, instance_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for instanceBuffer.

Safety
  • instance_buffer may need to be synchronized.
  • instance_buffer may be unretained, you must ensure it is kept alive while in use.
  • instance_buffer contents should be of the correct type.
pub unsafe fn instanceBufferOffset(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the instance buffer. Defaults to 0 bytes. Must be aligned to 4 bytes.

pub unsafe fn setInstanceBufferOffset(&self, instance_buffer_offset: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.
pub unsafe fn transformBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Buffer containing one column major matrix_float4x4 transformation matrix per instance

pub unsafe fn setTransformBuffer( &self, transform_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for transformBuffer.

Safety
  • transform_buffer may need to be synchronized.
  • transform_buffer may be unretained, you must ensure it is kept alive while in use.
  • transform_buffer contents should be of the correct type.
pub unsafe fn transformBufferOffset(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the transform buffer. Defaults to 0 bytes. Must be aligned to the stride of the transform type.

pub unsafe fn setTransformBufferOffset( &self, transform_buffer_offset: NSUInteger, )

👎Deprecated
Available on crate feature MPSRayIntersector only.
pub unsafe fn transformType(&self) -> MPSTransformType

👎Deprecated
Available on crate feature MPSRayIntersector only.

Instance transform type. Defaults to MPSTransformTypeFloat4x4. Changes to this property require rebuilding the acceleration structure.

pub unsafe fn setTransformType(&self, transform_type: MPSTransformType)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for transformType.

pub unsafe fn maskBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Mask buffer containing one uint32_t mask per instance. May be nil.

pub unsafe fn setMaskBuffer( &self, mask_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for maskBuffer.

Safety
  • mask_buffer may need to be synchronized.
  • mask_buffer may be unretained, you must ensure it is kept alive while in use.
  • mask_buffer contents should be of the correct type.
pub unsafe fn maskBufferOffset(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the mask buffer. Defaults to 0 bytes. Must be aligned to 4 bytes.

pub unsafe fn setMaskBufferOffset(&self, mask_buffer_offset: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for maskBufferOffset.

pub unsafe fn instanceCount(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Number of instances. Changes to this property require rebuilding the acceleration structure.

pub unsafe fn setInstanceCount(&self, instance_count: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for instanceCount.

Source

impl MPSInstanceAccelerationStructure

Methods declared on superclass MPSAccelerationStructure.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

👎Deprecated
Available on crate feature MPSRayIntersector only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Initialize the acceleration structure with a Metal device

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Initialize the acceleration structure with an NSCoder and a Metal device. Buffer properties such as the vertex buffer, instance buffer, etc. are set to nil. Encode and decode these buffers along with the acceleration structure instead.

Safety

a_decoder possibly has further requirements.

pub unsafe fn initWithGroup( this: Allocated<Self>, group: &MPSAccelerationStructureGroup, ) -> Retained<Self>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

Initialize the acceleration structure with an acceleration structure group, if the acceleration structure will be used in an instance hierarchy.

The Metal device is determined from the acceleration structure group. All acceleration structures in the instance hierarchy must share the same group.

pub unsafe fn initWithCoder_group( this: Allocated<Self>, a_decoder: &NSCoder, group: &MPSAccelerationStructureGroup, ) -> Option<Retained<Self>>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

Initialize the acceleration structure with an NSCoder and an acceleration structure group, if the acceleration structure will be used in an instance hierarchy. All acceleration structures in the instance hierarchy must share the same group. Buffer properties such as the vertex buffer, instance buffer, etc. are set to nil. Encode and decode these buffers along with the acceleration structure instead.

Safety

a_decoder possibly has further requirements.

Source

impl MPSInstanceAccelerationStructure

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSRayIntersector only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSInstanceAccelerationStructure

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSRayIntersector only.

Methods from Deref<Target = MPSAccelerationStructure>

pub unsafe fn group(&self) -> Retained<MPSAccelerationStructureGroup>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

The group this acceleration structure was created with

pub unsafe fn status(&self) -> MPSAccelerationStructureStatus

👎Deprecated
Available on crate feature MPSRayIntersector only.

Status indicating whether the acceleration structure has finished building

pub unsafe fn usage(&self) -> MPSAccelerationStructureUsage

👎Deprecated
Available on crate feature MPSRayIntersector only.

Acceleration structure usage options. Changes to this property require rebuilding the acceleration structure. Defaults to MPSAccelerationStructureUsageNone.

pub unsafe fn setUsage(&self, usage: MPSAccelerationStructureUsage)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for usage.

pub unsafe fn rebuild(&self)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Rebuild the acceleration structure

This method must be called before any intersection tests can be scheduled with this acceleration structure. Before calling this method, fill out the properties of the acceleration structure such as vertex buffer, instance buffer, etc. The acceleration structure should be rebuilt when its contents (e.g. vertices in a triangle acceleration structure) have been modified significantly and must be rebuilt when properties such as triangle count, vertex stride, etc. have changed. When the contents of the acceleration structure have only been modified slightly, it may be cheaper to refit the acceleration structure instead.

This method blocks until the acceleration structure has been rebuilt. Until the rebuild has completed, the acceleration structure cannot be copied, encoded with NSSecureCoding, rebuilt, or refit. Before this method can be called, any pending GPU writes to the vertex buffer, index buffer, etc. must be completed (and, for managed buffers, synchronized). Any prior intersection tests must also be completed before the acceleration structure can be rebuilt.

pub unsafe fn rebuildWithCompletionHandler( &self, completion_handler: MPSAccelerationStructureCompletionHandler, )

👎Deprecated
Available on crate features MPSRayIntersector and block2 only.

Rebuild the acceleration structure asynchronously

This method must be called before any intersection tests can be scheduled with this acceleration structure. Before calling this method, fill out the properties of the acceleration structure such as vertex buffer, instance buffer, etc. The acceleration structure should be rebuilt when its contents (e.g. vertices in a triangle acceleration structure) have been modified significantly and must be rebuilt when properties such as triangle count, vertex stride, etc. have changed. When the contents of the acceleration structure have only been modified slightly, it may be cheaper to refit the acceleration structure instead.

Until the rebuild has completed, the acceleration structure cannot be copied, encoded with NSSecureCoding, rebuilt, or refit. Before this method can be called, any pending GPU writes to the vertex buffer, index buffer, etc. must be completed (and, for managed buffers, synchronized). Any prior intersection tests must also be completed before the acceleration structure can be rebuilt.

Safety

completion_handler must be a valid pointer.

pub unsafe fn encodeRefitToCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Refit the existing acceleration structure to new data

This method is used to refit the acceleration structure to new vertex data, index data, instance data, etc. while preserving the existing acceleration structure topology. This is typically much faster than a full rebuild of the acceleration structure. Refitting can also be pipelined with other GPU work such as ray intersection.

Until the command buffer has completed, the acceleration structure cannot be copied, encoded with NSSecureCoding, or rebuilt. Changes to properties such as the triangle count or instance count might not be reflected. These changes require that the acceleration structure be rebuilt instead. The acceleration structure must be rebuilt at least once before this method can be called.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Create a a copy of this acceleration structure

The acceleration structure may be copied to a different Metal device. Buffer properties of the acceleration structure such as the vertex buffer, instance, buffer, etc. are set to nil. Copy these buffers to the new Metal device and assign them to the new acceleration structure instead. Do not copy the acceleration structure until any prior refit or rebuild operations have completed.

Parameter zone: This parameter is ignored. Memory zones are no longer used by Objective-C.

Parameter device: New Metal device

Safety

zone must be a valid pointer or null.

pub unsafe fn copyWithZone_group( &self, zone: *mut NSZone, group: &MPSAccelerationStructureGroup, ) -> Retained<Self>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

Create a a copy of this acceleration structure

The acceleration structure may be copied with a different acceleration structure group. Buffer properties of the acceleration structure such as the vertex buffer, instance buffer, etc. are set to nil. Copy these buffers with the new Metal device and assign them to the new acceleration structure instead. Do not copy the acceleration structure until any prior refit or rebuild operations have completed.

Parameter zone: This parameter is ignored. Memory zones are no longer used by Objective-C.

Parameter group: New acceleration structure group

Safety

zone must be a valid pointer or null.

pub unsafe fn encodeWithCoder(&self, coder: &NSCoder)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Encode the acceleration structure with an NSCoder

Buffer properties such as the vertex buffer, index buffer, etc. are not be encoded. Encode and decode these buffers along with the acceleration structure instead. Do not encode the acceleration structure until any prior refit or rebuild operations have completed.

Parameter coder: An archiver object

Safety

coder possibly has further requirements.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}