MPSIntersectionDistance

Struct MPSIntersectionDistance 

#[repr(C)]
pub struct MPSIntersectionDistance { pub distance: c_float, }
Available on crate features MPSRayIntersector and MPSRayIntersectorTypes only.
Returned intersection result which contains the distance from the ray origin to the intersection point

This type is available from the Metal Shading Language by including the MetalPerformanceShaders/MetalPerformanceShaders.h header.

See also Apple’s documentation

Fields

distance: c_float

Distance from the ray origin to the intersection point along the ray direction vector such that intersection = ray.origin + ray.direction * distance. Is negative if there is no intersection. If the intersection type is MPSIntersectionTypeAny, is a positive value for a hit or a negative value for a miss.