#[repr(C)]pub struct MPSIntersectionDistancePrimitiveIndex {
pub distance: c_float,
pub primitiveIndex: c_uint,
}Available on crate features
MPSRayIntersector and MPSRayIntersectorTypes only.Expand description
Intersection result which contains the distance from the ray origin to the intersection point and the index of the intersected primitive
This type is available from the Metal Shading Language by including the MetalPerformanceShaders/MetalPerformanceShaders.h header.
See also Apple’s documentation
Fields
distance: c_floatDistance from the ray origin to the intersection point along the ray direction vector such that intersection = ray.origin + ray.direction * distance. Is negative if there is no intersection. If the intersection type is MPSIntersectionTypeAny, is a positive value for a hit or a negative value for a miss.
primitiveIndex: c_uintIndex of the intersected primitive. Undefined if the ray does not intersect a primitive or if the intersection type is MPSIntersectionTypeAny.