pub struct MPSMatrixBatchNormalizationGradient { /* private fields */ }MPSCore and MPSKernel and MPSMatrix and MPSMatrixBatchNormalization and MPSMatrixTypes only.Expand description
Dependencies: This depends on Metal.framework.
A kernel to compute the gradient of the batch normalization operation.
A MPSMatrixBatchNormalizationGradient object computes the results of backpropagating the gradients of a loss function with respect to the outputs of an MPSMatrixBatchNormalization object. The corresponding properties and data used by the MPSMatrixBatchNormalizationGradient object should correspond to those used by the forward MPSMatrixBatchNormalization object for which the gradient is being computed.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixBatchNormalizationGradient
impl MPSMatrixBatchNormalizationGradient
Sourcepub unsafe fn sourceNumberOfFeatureVectors(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceNumberOfFeatureVectors(&self) -> NSUInteger
MPSNeuralNetwork only.The number of input vectors which make up the input array.
Sourcepub unsafe fn setSourceNumberOfFeatureVectors(
&self,
source_number_of_feature_vectors: NSUInteger,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSourceNumberOfFeatureVectors( &self, source_number_of_feature_vectors: NSUInteger, )
MPSNeuralNetwork only.Setter for sourceNumberOfFeatureVectors.
Sourcepub unsafe fn sourceInputFeatureChannels(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceInputFeatureChannels(&self) -> NSUInteger
MPSNeuralNetwork only.The number of feature channels in the input vectors.
Sourcepub unsafe fn setSourceInputFeatureChannels(
&self,
source_input_feature_channels: NSUInteger,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSourceInputFeatureChannels( &self, source_input_feature_channels: NSUInteger, )
MPSNeuralNetwork only.Setter for sourceInputFeatureChannels.
Sourcepub unsafe fn epsilon(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn epsilon(&self) -> c_float
MPSNeuralNetwork only.A small term added to the variance when normalizing the input.
Sourcepub unsafe fn setEpsilon(&self, epsilon: c_float)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setEpsilon(&self, epsilon: c_float)
MPSNeuralNetwork only.Setter for epsilon.
Sourcepub unsafe fn setNeuronType_parameterA_parameterB_parameterC(
&self,
neuron_type: MPSCNNNeuronType,
parameter_a: c_float,
parameter_b: c_float,
parameter_c: c_float,
)
Available on crate features MPSNeuralNetwork and MPSCNNNeuronType only.
pub unsafe fn setNeuronType_parameterA_parameterB_parameterC( &self, neuron_type: MPSCNNNeuronType, parameter_a: c_float, parameter_b: c_float, parameter_c: c_float, )
MPSNeuralNetwork and MPSCNNNeuronType only.Specifies a neuron activation function to be used.
This method can be used to add a neuron activation funtion of given type with associated scalar parameters A, B, and C that are shared across all output values. Note that this method can only be used to specify neurons which are specified by three (or fewer) parameters shared across all output values (or channels, in CNN nomenclature). It is an error to call this method for neuron activation functions like MPSCNNNeuronTypePReLU, which require per-channel parameter values. An MPSMatrixBatchNormalizationGradient kernel is initialized with a default neuron function of MPSCNNNeuronTypeNone.
Parameter neuronType: Type of neuron activation function. For full list see MPSCNNNeuronType.h
Parameter parameterA: parameterA of neuron activation that is shared across all output values.
Parameter parameterB: parameterB of neuron activation that is shared across all output values.
Parameter parameterC: parameterC of neuron activation that is shared across all output values.
Sourcepub unsafe fn neuronType(&self) -> MPSCNNNeuronType
Available on crate features MPSNeuralNetwork and MPSCNNNeuronType only.
pub unsafe fn neuronType(&self) -> MPSCNNNeuronType
MPSNeuralNetwork and MPSCNNNeuronType only.Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method
Sourcepub unsafe fn neuronParameterA(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn neuronParameterA(&self) -> c_float
MPSNeuralNetwork only.Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method
Sourcepub unsafe fn neuronParameterB(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn neuronParameterB(&self) -> c_float
MPSNeuralNetwork only.Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method
Sourcepub unsafe fn neuronParameterC(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn neuronParameterC(&self) -> c_float
MPSNeuralNetwork only.Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSNeuralNetwork only.Sourcepub unsafe fn encodeToCommandBuffer_gradientMatrix_inputMatrix_meanVector_varianceVector_gammaVector_betaVector_resultGradientForDataMatrix_resultGradientForGammaVector_resultGradientForBetaVector(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
gradient_matrix: &MPSMatrix,
input_matrix: &MPSMatrix,
mean_vector: &MPSVector,
variance_vector: &MPSVector,
gamma_vector: Option<&MPSVector>,
beta_vector: Option<&MPSVector>,
result_gradient_for_data_matrix: &MPSMatrix,
result_gradient_for_gamma_vector: Option<&MPSVector>,
result_gradient_for_beta_vector: Option<&MPSVector>,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeToCommandBuffer_gradientMatrix_inputMatrix_meanVector_varianceVector_gammaVector_betaVector_resultGradientForDataMatrix_resultGradientForGammaVector_resultGradientForBetaVector( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, gradient_matrix: &MPSMatrix, input_matrix: &MPSMatrix, mean_vector: &MPSVector, variance_vector: &MPSVector, gamma_vector: Option<&MPSVector>, beta_vector: Option<&MPSVector>, result_gradient_for_data_matrix: &MPSMatrix, result_gradient_for_gamma_vector: Option<&MPSVector>, result_gradient_for_beta_vector: Option<&MPSVector>, )
MPSNeuralNetwork only.Encode a MPSMatrixBatchNormalizationGradient object to a command buffer and compute its gradient with respect to its input data.
Parameter commandBuffer: The commandBuffer on which to encode the operation.
Parameter gradientMatrix: A matrix whose values represent the gradient of a
loss function with respect to the results of a forward
MPSMatrixBatchNormalization operation.
Parameter inputMatrix: A matrix containing the inputs to a forward MPSMatrixBatchNormalization
operation for which the gradient values are to be computed.
Parameter meanVector: A vector containing the batch mean values. Should contain either the specified
values used to compute the forward result, or the computed values resulting from
the forward kernel execution.
Parameter varianceVector: A vector containing the batch variance values. Should contain either the specified
values used to compute the forward result, or the computed values resulting from
the forward kernel execution.
Parameter gammaVector: A vector containing the gamma terms. Should be the same values as used
when computing the forward result.
Parameter betaVector: A vector containing the beta terms. Should be the same values as used when
computing the forward result.
Parameter resultGradientForDataMatrix: The matrix containing the resulting gradient values.
Parameter resultGradientForGammaVector: If non-NULL the vector containing gradients for the gamma
terms.
Parameter resultGradientForBetaVector: If non-NULL the vector containing gradients for the beta
terms.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSNeuralNetwork only.NSSecureCoding compatability
See
MPSKernel#initWithCoder.
Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixBatchNormalizationGradient
Parameter device: The MTLDevice on which to make the MPSMatrixBatchNormalizationGradient object.
Returns: A new MPSMatrixBatchNormalizationGradient object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
MPSNeuralNetwork only.Make a copy of this kernel for a new device -
See: MPSKernel
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: A pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Sourceimpl MPSMatrixBatchNormalizationGradient
Methods declared on superclass MPSKernel.
impl MPSMatrixBatchNormalizationGradient
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSNeuralNetwork only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSMatrixBinaryKernel>
Sourcepub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
pub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
The origin, relative to [0, 0] in the primary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setPrimarySourceMatrixOrigin(
&self,
primary_source_matrix_origin: MTLOrigin,
)
pub unsafe fn setPrimarySourceMatrixOrigin( &self, primary_source_matrix_origin: MTLOrigin, )
Setter for primarySourceMatrixOrigin.
Sourcepub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
pub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
The origin, relative to [0, 0] in the secondary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setSecondarySourceMatrixOrigin(
&self,
secondary_source_matrix_origin: MTLOrigin,
)
pub unsafe fn setSecondarySourceMatrixOrigin( &self, secondary_source_matrix_origin: MTLOrigin, )
Setter for secondarySourceMatrixOrigin.
Sourcepub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
Setter for resultMatrixOrigin.
Sourcepub unsafe fn batchStart(&self) -> NSUInteger
pub unsafe fn batchStart(&self) -> NSUInteger
The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.
Sourcepub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
Setter for batchStart.
Sourcepub unsafe fn batchSize(&self) -> NSUInteger
pub unsafe fn batchSize(&self) -> NSUInteger
The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.
Sourcepub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
Setter for batchSize.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<AnyObject> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl AsRef<MPSKernel> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSKernel> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl AsRef<MPSMatrixBatchNormalizationGradient> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSMatrixBatchNormalizationGradient> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<NSObject> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<NSObject> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl Borrow<AnyObject> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<AnyObject> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl Borrow<MPSKernel> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSKernel> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<NSObject> for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<NSObject> for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl ClassType for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl ClassType for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceconst NAME: &'static str = "MPSMatrixBatchNormalizationGradient"
const NAME: &'static str = "MPSMatrixBatchNormalizationGradient"
Sourcetype Super = MPSMatrixBinaryKernel
type Super = MPSMatrixBinaryKernel
Sourcetype ThreadKind = <<MPSMatrixBatchNormalizationGradient as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixBatchNormalizationGradient as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl CopyingHelper for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourcetype Result = MPSMatrixBatchNormalizationGradient
type Result = MPSMatrixBatchNormalizationGradient
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Debug for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl Deref for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Deref for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl Hash for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Hash for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl Message for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl Message for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl NSCoding for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl NSCoding for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl NSCopying for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl NSCopying for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl NSObjectProtocol for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl NSObjectProtocol for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl NSSecureCoding for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl PartialEq for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl PartialEq for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceimpl RefEncode for MPSMatrixBatchNormalizationGradient
Available on crate feature MPSNeuralNetwork only.
impl RefEncode for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.Sourceconst ENCODING_REF: Encoding = <MPSMatrixBinaryKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSMatrixBinaryKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.impl Eq for MPSMatrixBatchNormalizationGradient
MPSNeuralNetwork only.