pub struct MPSMatrixBinaryKernel { /* private fields */ }MPSCore and MPSKernel and MPSMatrixTypes only.Expand description
Dependencies: This depends on Metal.framework
A MPSMatrixBinaryKernel consumes two MPSMatrix objects and produces one MPSMatrix object.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixBinaryKernel
impl MPSMatrixBinaryKernel
Sourcepub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the primary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setPrimarySourceMatrixOrigin(
&self,
primary_source_matrix_origin: MTLOrigin,
)
Available on crate feature MPSMatrix only.
pub unsafe fn setPrimarySourceMatrixOrigin( &self, primary_source_matrix_origin: MTLOrigin, )
MPSMatrix only.Setter for primarySourceMatrixOrigin.
Sourcepub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the secondary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setSecondarySourceMatrixOrigin(
&self,
secondary_source_matrix_origin: MTLOrigin,
)
Available on crate feature MPSMatrix only.
pub unsafe fn setSecondarySourceMatrixOrigin( &self, secondary_source_matrix_origin: MTLOrigin, )
MPSMatrix only.Setter for secondarySourceMatrixOrigin.
Sourcepub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
Available on crate feature MPSMatrix only.
pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
MPSMatrix only.Setter for resultMatrixOrigin.
Sourcepub unsafe fn batchStart(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn batchStart(&self) -> NSUInteger
MPSMatrix only.The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.
Sourcepub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
MPSMatrix only.Setter for batchStart.
Sourcepub unsafe fn batchSize(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn batchSize(&self) -> NSUInteger
MPSMatrix only.The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.
Sourcepub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
MPSMatrix only.Setter for batchSize.
Sourceimpl MPSMatrixBinaryKernel
Methods declared on superclass MPSKernel.
impl MPSMatrixBinaryKernel
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithDevice(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSMatrix only.Standard init with default properties per filter type
Parameter device: The device that the filter will be used on. May not be NULL.
Returns: a pointer to the newly initialized object. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSMatrix only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSMatrix only.NSSecureCoding compatability
While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.
Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel
Parameter device: The MTLDevice on which to make the MPSKernel
Returns: A new MPSKernel object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl AsRef<AnyObject> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl AsRef<MPSKernel> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl AsRef<MPSKernel> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
Available on crate features MPSMatrixBatchNormalization and MPSNeuralNetwork and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
MPSMatrixBatchNormalization and MPSNeuralNetwork and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixFullyConnected
Available on crate features MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixFullyConnected
MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
Available on crate features MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixLogSoftMaxGradient
Available on crate features MPSMatrixSoftMax and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixLogSoftMaxGradient
MPSMatrixSoftMax and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixNeuronGradient
Available on crate features MPSMatrixNeuron and MPSNeuralNetwork and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixNeuronGradient
MPSMatrixNeuron and MPSNeuralNetwork and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
Available on crate features MPSMatrixSoftMax and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
MPSMatrixSoftMax and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSolveCholesky
Available on crate features MPSMatrixSolve and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSolveCholesky
MPSMatrixSolve and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSolveLU
Available on crate features MPSMatrixSolve and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSolveLU
MPSMatrixSolve and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSolveTriangular
Available on crate features MPSMatrixSolve and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSolveTriangular
MPSMatrixSolve and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixVectorMultiplication
Available on crate features MPSMatrixMultiplication and MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixVectorMultiplication
MPSMatrixMultiplication and MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<NSObject> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl AsRef<NSObject> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl Borrow<AnyObject> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Borrow<AnyObject> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl Borrow<MPSKernel> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Borrow<MPSKernel> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
Available on crate features MPSMatrixBatchNormalization and MPSNeuralNetwork and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixBatchNormalizationGradient
MPSMatrixBatchNormalization and MPSNeuralNetwork and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixFullyConnected
Available on crate features MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixFullyConnected
MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
Available on crate features MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
MPSMatrixFullyConnected and MPSNeuralNetwork and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixLogSoftMaxGradient
Available on crate features MPSMatrixSoftMax and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixLogSoftMaxGradient
MPSMatrixSoftMax and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixNeuronGradient
Available on crate features MPSMatrixNeuron and MPSNeuralNetwork and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixNeuronGradient
MPSMatrixNeuron and MPSNeuralNetwork and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
Available on crate features MPSMatrixSoftMax and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
MPSMatrixSoftMax and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSolveCholesky
Available on crate features MPSMatrixSolve and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSolveCholesky
MPSMatrixSolve and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSolveLU
Available on crate features MPSMatrixSolve and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSolveLU
MPSMatrixSolve and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSolveTriangular
Available on crate features MPSMatrixSolve and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSolveTriangular
MPSMatrixSolve and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixVectorMultiplication
Available on crate features MPSMatrixMultiplication and MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixVectorMultiplication
MPSMatrixMultiplication and MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<NSObject> for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Borrow<NSObject> for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl ClassType for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl ClassType for MPSMatrixBinaryKernel
MPSMatrix only.Sourceconst NAME: &'static str = "MPSMatrixBinaryKernel"
const NAME: &'static str = "MPSMatrixBinaryKernel"
Sourcetype ThreadKind = <<MPSMatrixBinaryKernel as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixBinaryKernel as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl CopyingHelper for MPSMatrixBinaryKernel
MPSMatrix only.Sourcetype Result = MPSMatrixBinaryKernel
type Result = MPSMatrixBinaryKernel
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Debug for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl Deref for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Deref for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl Hash for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Hash for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl Message for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl Message for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl NSCoding for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl NSCoding for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl NSCopying for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl NSCopying for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl NSObjectProtocol for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl NSObjectProtocol for MPSMatrixBinaryKernel
MPSMatrix only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl NSSecureCoding for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl PartialEq for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl PartialEq for MPSMatrixBinaryKernel
MPSMatrix only.Sourceimpl RefEncode for MPSMatrixBinaryKernel
Available on crate feature MPSMatrix only.
impl RefEncode for MPSMatrixBinaryKernel
MPSMatrix only.Sourceconst ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixBinaryKernel
MPSMatrix only.impl Eq for MPSMatrixBinaryKernel
MPSMatrix only.