pub struct MPSMatrixCopy { /* private fields */ }MPSCore and MPSKernel and MPSMatrixCombination only.Expand description
Implementations
Sourceimpl MPSMatrixCopy
impl MPSMatrixCopy
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSMatrix only.Sourcepub unsafe fn initWithDevice_copyRows_copyColumns_sourcesAreTransposed_destinationsAreTransposed(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
copy_rows: NSUInteger,
copy_columns: NSUInteger,
sources_are_transposed: bool,
destinations_are_transposed: bool,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithDevice_copyRows_copyColumns_sourcesAreTransposed_destinationsAreTransposed( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, copy_rows: NSUInteger, copy_columns: NSUInteger, sources_are_transposed: bool, destinations_are_transposed: bool, ) -> Retained<Self>
MPSMatrix only.Initialize a copy operator
Parameter copyRows: The number of rows to copy for each copy operation
Parameter copyColumns: The number of matrix columns to copy in each copy operation
Parameter sourcesAreTransposed: If YES, the sources are in column major storage order
Parameter destinationsAreTransposed: If YES, the destinations are in column major storage order
Sourcepub unsafe fn copyRows(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn copyRows(&self) -> NSUInteger
MPSMatrix only.The number of rows to copy for each copy operation
Sourcepub unsafe fn copyColumns(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn copyColumns(&self) -> NSUInteger
MPSMatrix only.The number of columns to copy for each copy operation
Sourcepub unsafe fn sourcesAreTransposed(&self) -> bool
Available on crate feature MPSMatrix only.
pub unsafe fn sourcesAreTransposed(&self) -> bool
MPSMatrix only.If YES, the sources are in row major storage order
Sourcepub unsafe fn destinationsAreTransposed(&self) -> bool
Available on crate feature MPSMatrix only.
pub unsafe fn destinationsAreTransposed(&self) -> bool
MPSMatrix only.If YES, the destinations are in row major storage order
Sourcepub unsafe fn encodeToCommandBuffer_copyDescriptor(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
copy_descriptor: &MPSMatrixCopyDescriptor,
)
Available on crate feature MPSMatrix only.
pub unsafe fn encodeToCommandBuffer_copyDescriptor( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, copy_descriptor: &MPSMatrixCopyDescriptor, )
MPSMatrix only.Encode the copy operations to the command buffer
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.
Parameter copyDescriptor: The descriptor that defines the copy operator
Sourcepub unsafe fn encodeToCommandBuffer_copyDescriptor_rowPermuteIndices_rowPermuteOffset_columnPermuteIndices_columnPermuteOffset(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
copy_descriptor: &MPSMatrixCopyDescriptor,
row_permute_indices: Option<&MPSVector>,
row_permute_offset: NSUInteger,
column_permute_indices: Option<&MPSVector>,
column_permute_offset: NSUInteger,
)
Available on crate feature MPSMatrix only.
pub unsafe fn encodeToCommandBuffer_copyDescriptor_rowPermuteIndices_rowPermuteOffset_columnPermuteIndices_columnPermuteOffset( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, copy_descriptor: &MPSMatrixCopyDescriptor, row_permute_indices: Option<&MPSVector>, row_permute_offset: NSUInteger, column_permute_indices: Option<&MPSVector>, column_permute_offset: NSUInteger, )
MPSMatrix only.Encode the copy operations to the command buffer. This of the encode version support permuting the outputs with custom vectors of indices. The permutations are defined on the destination indices and are the same for each copy operation.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.
Parameter copyDescriptor: The descriptor that defines the copy operator
Parameter rowPermuteIndices: If not nil then the output row index is
‘rowPermuteIndices[i] + rowOffset’ instead of ‘i + rowOffset’,
where ‘i’ is the local row index of the copy operation.
Note: if destinationsAreTransposed is set to YES then the destination
transpose is performed before permutations.
Parameter rowPermuteOffset: Offset in numbers to apply to the ‘rowPermuteIndices’ vector.
Parameter columnPermuteIndices: If not nil then the output column index is
‘columnPermuteIndices[i] + columnOffset’ instead of ‘i + columnOffset’,
where ‘i’ is the local column index of the copy operation.
Note: if destinationsAreTransposed is set to YES then the destination
transpose is performed before permutations.
Parameter columnPermuteOffset: Offset in numbers to apply to the ‘columnPermuteIndices’ vector.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSMatrix only.NSSecureCoding compatability
See
MPSKernel#initWithCoder.
Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixLookUpAndCopy
Parameter device: The MTLDevice on which to make the MPSMatrixLookUpAndCopy
Returns: A new MPSMatrixLookUpAndCopy object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourceimpl MPSMatrixCopy
Methods declared on superclass MPSKernel.
impl MPSMatrixCopy
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSMatrix only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl AsRef<AnyObject> for MPSMatrixCopy
MPSMatrix only.Sourceimpl AsRef<MPSKernel> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl AsRef<MPSKernel> for MPSMatrixCopy
MPSMatrix only.Sourceimpl AsRef<MPSMatrixCopy> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixCopy> for MPSMatrixCopy
MPSMatrix only.Sourceimpl AsRef<NSObject> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl AsRef<NSObject> for MPSMatrixCopy
MPSMatrix only.Sourceimpl Borrow<AnyObject> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Borrow<AnyObject> for MPSMatrixCopy
MPSMatrix only.Sourceimpl Borrow<MPSKernel> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Borrow<MPSKernel> for MPSMatrixCopy
MPSMatrix only.Sourceimpl Borrow<NSObject> for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Borrow<NSObject> for MPSMatrixCopy
MPSMatrix only.Sourceimpl ClassType for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl ClassType for MPSMatrixCopy
MPSMatrix only.Sourceconst NAME: &'static str = "MPSMatrixCopy"
const NAME: &'static str = "MPSMatrixCopy"
Sourcetype ThreadKind = <<MPSMatrixCopy as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixCopy as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl CopyingHelper for MPSMatrixCopy
MPSMatrix only.Sourcetype Result = MPSMatrixCopy
type Result = MPSMatrixCopy
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Debug for MPSMatrixCopy
MPSMatrix only.Sourceimpl Deref for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Deref for MPSMatrixCopy
MPSMatrix only.Sourceimpl Hash for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Hash for MPSMatrixCopy
MPSMatrix only.Sourceimpl Message for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl Message for MPSMatrixCopy
MPSMatrix only.Sourceimpl NSCoding for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl NSCoding for MPSMatrixCopy
MPSMatrix only.Sourceimpl NSCopying for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl NSCopying for MPSMatrixCopy
MPSMatrix only.Sourceimpl NSObjectProtocol for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl NSObjectProtocol for MPSMatrixCopy
MPSMatrix only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl NSSecureCoding for MPSMatrixCopy
MPSMatrix only.Sourceimpl PartialEq for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl PartialEq for MPSMatrixCopy
MPSMatrix only.Sourceimpl RefEncode for MPSMatrixCopy
Available on crate feature MPSMatrix only.
impl RefEncode for MPSMatrixCopy
MPSMatrix only.Sourceconst ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixCopy
MPSMatrix only.impl Eq for MPSMatrixCopy
MPSMatrix only.