pub struct MPSMatrixCopyDescriptor { /* private fields */ }MPSMatrixCombination only.Expand description
A list of copy operations
The MPSMatrixCopy filter can do multiple copy operations. For RNN filters, these copies are often small, and are more efficient when grouped together. The MPSMatriceCopyDescriptor provides a container to list the operations. The operations occur in any order, and may not alias.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixCopyDescriptor
impl MPSMatrixCopyDescriptor
Sourcepub unsafe fn descriptorWithSourceMatrix_destinationMatrix_offsets(
source_matrix: &MPSMatrix,
destination_matrix: &MPSMatrix,
offsets: MPSMatrixCopyOffsets,
) -> Retained<Self>
Available on crate features MPSMatrix and MPSCore only.
pub unsafe fn descriptorWithSourceMatrix_destinationMatrix_offsets( source_matrix: &MPSMatrix, destination_matrix: &MPSMatrix, offsets: MPSMatrixCopyOffsets, ) -> Retained<Self>
MPSMatrix and MPSCore only.convenience allocator for single copies
Sourcepub unsafe fn initWithDevice_count(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
count: NSUInteger,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithDevice_count( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, count: NSUInteger, ) -> Retained<Self>
MPSMatrix only.initialize a MPSMatrixCopyDescriptor with default values.
Use -setCopyOperationAtIndex:sourceMatrix:destinationMatrix:copyOffsets to initialize. All indices must be initialized before use.
Parameter device: The device on which the copy will be performed
Parameter count: The number of copy operations the object will encode
Returns: A MPSMatrixCopyDescriptor. It still needs to be initialized with -setCopyOperationAtIndex:sourceMatrix:destinationMatrix:copyOffsets
Sourcepub unsafe fn setCopyOperationAtIndex_sourceMatrix_destinationMatrix_offsets(
&self,
index: NSUInteger,
source_matrix: &MPSMatrix,
destination_matrix: &MPSMatrix,
offsets: MPSMatrixCopyOffsets,
)
Available on crate features MPSMatrix and MPSCore only.
pub unsafe fn setCopyOperationAtIndex_sourceMatrix_destinationMatrix_offsets( &self, index: NSUInteger, source_matrix: &MPSMatrix, destination_matrix: &MPSMatrix, offsets: MPSMatrixCopyOffsets, )
MPSMatrix and MPSCore only.Initialize a MPSMatrixCopyDescriptor using offsets generated on the CPU
This is for one at a time intialization of the copy operations
Parameter index: The index of the copy operation
Parameter sourceMatrix: The source matrix for this copy operation
Parameter destinationMatrix: The destination matrix for this copy operation
Parameter offsets: The offsets to use for the copy operation
Sourcepub unsafe fn initWithSourceMatrices_destinationMatrices_offsetVector_offset(
this: Allocated<Self>,
source_matrices: &NSArray<MPSMatrix>,
destination_matrices: &NSArray<MPSMatrix>,
offsets: Option<&MPSVector>,
byte_offset: NSUInteger,
) -> Retained<Self>
Available on crate features MPSMatrix and MPSCore only.
pub unsafe fn initWithSourceMatrices_destinationMatrices_offsetVector_offset( this: Allocated<Self>, source_matrices: &NSArray<MPSMatrix>, destination_matrices: &NSArray<MPSMatrix>, offsets: Option<&MPSVector>, byte_offset: NSUInteger, ) -> Retained<Self>
MPSMatrix and MPSCore only.Initialize a MPSMatrixCopyDescriptor using offsets generated on the GPU
Use this method when the offsets needed are coming from GPU based computation.
Parameter sourceMatrices: A list of matrices from which the matrix data is read
Parameter destinationMatrices: A list of matrices to which to write the data. The count
must match the number of source matrices.
Parameter offsets: A MPSVector of type MPSDataTypeUInt32 containing the list of
offsets, stored as a packed array of MPSMatrixCopyOffsets.
Parameter byteOffset: A byte offset into the offsets vector where the data starts in ‘offsets’.
This value must be a multiple of 16.
Returns: A valid MPSMatrixCopyDescriptor to represent the list of copy operations
pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>
MPSMatrix only.Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl AsRef<AnyObject> for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl AsRef<MPSMatrixCopyDescriptor> for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixCopyDescriptor> for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl AsRef<NSObject> for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl AsRef<NSObject> for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl Borrow<AnyObject> for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl Borrow<AnyObject> for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl Borrow<NSObject> for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl Borrow<NSObject> for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl ClassType for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl ClassType for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceconst NAME: &'static str = "MPSMatrixCopyDescriptor"
const NAME: &'static str = "MPSMatrixCopyDescriptor"
Sourcetype ThreadKind = <<MPSMatrixCopyDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixCopyDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl Debug for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl Debug for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl Deref for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl Deref for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl Hash for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl Hash for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl Message for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl Message for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl NSObjectProtocol for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl NSObjectProtocol for MPSMatrixCopyDescriptor
MPSMatrix only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl PartialEq for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceimpl RefEncode for MPSMatrixCopyDescriptor
Available on crate feature MPSMatrix only.
impl RefEncode for MPSMatrixCopyDescriptor
MPSMatrix only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixCopyDescriptor
MPSMatrix only.impl Eq for MPSMatrixCopyDescriptor
MPSMatrix only.