#[repr(C)]pub struct MPSMatrixCopyOffsets {
pub sourceRowOffset: u32,
pub sourceColumnOffset: u32,
pub destinationRowOffset: u32,
pub destinationColumnOffset: u32,
}Available on crate features
MPSMatrix and MPSMatrixCombination only.Expand description
A description of each copy operation
See also Apple’s documentation
Fields
sourceRowOffset: u32offset to start of source region to read in rows
sourceColumnOffset: u32offset to start of source region to read in columns
destinationRowOffset: u32offset to start of destination region to read in rows
destinationColumnOffset: u32offset to start of destination region to read in columns