pub struct MPSMatrixCopyToImage { /* private fields */ }MPSCore and MPSImageCopy and MPSKernel only.Expand description
The MPSMatrixCopyToImage copies matrix data to a MPSImage. The operation is the reverse of MPSImageCopyToMatrix.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixCopyToImage
impl MPSMatrixCopyToImage
Sourcepub unsafe fn sourceMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSImage only.
pub unsafe fn sourceMatrixOrigin(&self) -> MTLOrigin
MPSImage only.The origin, relative to [0, 0] in the source matrix. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setSourceMatrixOrigin(&self, source_matrix_origin: MTLOrigin)
Available on crate feature MPSImage only.
pub unsafe fn setSourceMatrixOrigin(&self, source_matrix_origin: MTLOrigin)
MPSImage only.Setter for sourceMatrixOrigin.
Sourcepub unsafe fn sourceMatrixBatchIndex(&self) -> NSUInteger
Available on crate feature MPSImage only.
pub unsafe fn sourceMatrixBatchIndex(&self) -> NSUInteger
MPSImage only.The index of the source matrix in the batch. This property is modifiable and defaults to 0 at initialization time.
Sourcepub unsafe fn setSourceMatrixBatchIndex(
&self,
source_matrix_batch_index: NSUInteger,
)
Available on crate feature MPSImage only.
pub unsafe fn setSourceMatrixBatchIndex( &self, source_matrix_batch_index: NSUInteger, )
MPSImage only.Setter for sourceMatrixBatchIndex.
Sourcepub unsafe fn dataLayout(&self) -> MPSDataLayout
Available on crate feature MPSImage only.
pub unsafe fn dataLayout(&self) -> MPSDataLayout
MPSImage only.The data layout to use
Returns the data layout. When copying from a MPSMatrix to a MPSImage, this describes the order in which the image values are to be stored in the buffer associated with the MPSMatrix. Default: MPSDataLayoutFeatureChannelsxHeightxWidth
Sourcepub unsafe fn initWithDevice_dataLayout(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
data_layout: MPSDataLayout,
) -> Retained<Self>
Available on crate feature MPSImage only.
pub unsafe fn initWithDevice_dataLayout( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, data_layout: MPSDataLayout, ) -> Retained<Self>
MPSImage only.Initialize a MPSMatrixCopyToImage object on a device
Parameter device: The device the kernel will run on
Parameter dataLayout: The data layout
Returns: A valid MPSMatrixCopyToImage object or nil, if failure.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSImage only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSImage only.NSSecureCoding compatability
While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.
Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel
Parameter device: The MTLDevice on which to make the MPSKernel
Returns: A new MPSKernel object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn encodeToCommandBuffer_sourceMatrix_destinationImage(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source_matrix: &MPSMatrix,
destination_image: &MPSImage,
)
Available on crate features MPSImage and MPSMatrix only.
pub unsafe fn encodeToCommandBuffer_sourceMatrix_destinationImage( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_matrix: &MPSMatrix, destination_image: &MPSImage, )
MPSImage and MPSMatrix only.Encode a kernel that copies a MPSMatrix to a MPSImage into a command buffer using a MTLComputeCommandEncoder.
The kernel copies feature channels from sourceMatrix to the destinationImage. The kernel will not begin to execute until after the command buffer has been enqueued and committed.
NOTE: The sourceMatrix.dataType must match the feature channel data type in destinationImage.
Parameter commandBuffer: A valid MTLCommandBuffer.
Parameter sourceMatrix: A valid MPSMatrix or MPSTemporaryMatrix object describing the source matrix.
Parameter destinationImage: A valid MPSImage describing the image to copy to.
Sourcepub unsafe fn encodeBatchToCommandBuffer_sourceMatrix_destinationImages(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source_matrix: &MPSMatrix,
destination_images: &MPSImageBatch,
)
Available on crate features MPSImage and MPSMatrix and MPSNDArray only.
pub unsafe fn encodeBatchToCommandBuffer_sourceMatrix_destinationImages( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_matrix: &MPSMatrix, destination_images: &MPSImageBatch, )
MPSImage and MPSMatrix and MPSNDArray only.Encode a kernel that copies a MPSMatrix to a MPSImageBatch into a command buffer using a MTLComputeCommandEncoder.
The kernel copies feature channels from sourceImage to the buffer associated with destinationMatrix. The kernel will not begin to execute until after the command buffer has been enqueued and committed. Each image will be copied to its own row in the matrix, starting with row destinationMatrixOrigin.x.
NOTE: The destinationMatrix.dataType must match the feature channel data type in sourceImage. NOTE: All the images in the source batch should be of the same size and have numberOfImages = 1.
Parameter commandBuffer: A valid MTLCommandBuffer.
Parameter sourceMatrix: A valid MPSMatrix or MPSTemporaryMatrix object describing the source matrix.
Parameter destinationImages: A valid MPSImageBatch describing the images to copy to.
Sourceimpl MPSMatrixCopyToImage
Methods declared on superclass MPSKernel.
impl MPSMatrixCopyToImage
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithDevice(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
Available on crate feature MPSImage only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSImage only.Standard init with default properties per filter type
Parameter device: The device that the filter will be used on. May not be NULL.
Returns: a pointer to the newly initialized object. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSImage only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSImage only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl AsRef<AnyObject> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl AsRef<MPSKernel> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl AsRef<MPSKernel> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl AsRef<MPSMatrixCopyToImage> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl AsRef<MPSMatrixCopyToImage> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl AsRef<NSObject> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl AsRef<NSObject> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl Borrow<AnyObject> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Borrow<AnyObject> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl Borrow<MPSKernel> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Borrow<MPSKernel> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl Borrow<NSObject> for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Borrow<NSObject> for MPSMatrixCopyToImage
MPSImage only.Sourceimpl ClassType for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl ClassType for MPSMatrixCopyToImage
MPSImage only.Sourceconst NAME: &'static str = "MPSMatrixCopyToImage"
const NAME: &'static str = "MPSMatrixCopyToImage"
Sourcetype ThreadKind = <<MPSMatrixCopyToImage as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixCopyToImage as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl CopyingHelper for MPSMatrixCopyToImage
MPSImage only.Sourcetype Result = MPSMatrixCopyToImage
type Result = MPSMatrixCopyToImage
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Debug for MPSMatrixCopyToImage
MPSImage only.Sourceimpl Deref for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Deref for MPSMatrixCopyToImage
MPSImage only.Sourceimpl Hash for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Hash for MPSMatrixCopyToImage
MPSImage only.Sourceimpl Message for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl Message for MPSMatrixCopyToImage
MPSImage only.Sourceimpl NSCoding for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl NSCoding for MPSMatrixCopyToImage
MPSImage only.Sourceimpl NSCopying for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl NSCopying for MPSMatrixCopyToImage
MPSImage only.Sourceimpl NSObjectProtocol for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl NSObjectProtocol for MPSMatrixCopyToImage
MPSImage only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl NSSecureCoding for MPSMatrixCopyToImage
MPSImage only.Sourceimpl PartialEq for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl PartialEq for MPSMatrixCopyToImage
MPSImage only.Sourceimpl RefEncode for MPSMatrixCopyToImage
Available on crate feature MPSImage only.
impl RefEncode for MPSMatrixCopyToImage
MPSImage only.Sourceconst ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixCopyToImage
MPSImage only.impl Eq for MPSMatrixCopyToImage
MPSImage only.