MPSMatrixDescriptor

Struct MPSMatrixDescriptor 

pub struct MPSMatrixDescriptor { /* private fields */ }
Available on crate feature MPSMatrix only.
Dependencies: This depends on Metal.framework

A MPSMatrixDescriptor describes the sizes, strides, and data type of a an array of 2-dimensional matrices. All storage is assumed to be in “matrix-major”. See the description for MPSMatrix for further details.

See also Apple’s documentation

Implementations

Source

impl MPSMatrixDescriptor

pub unsafe fn rows(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of rows in a matrix.

pub unsafe fn setRows(&self, rows: NSUInteger)

Available on crate feature MPSCore only.

Setter for rows.

pub unsafe fn columns(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of columns in a matrix.

pub unsafe fn setColumns(&self, columns: NSUInteger)

Available on crate feature MPSCore only.

Setter for columns.

pub unsafe fn matrices(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of matrices.

pub unsafe fn dataType(&self) -> MPSDataType

Available on crate features MPSCore and MPSCoreTypes only.

The type of the data which makes up the values of the matrix.

pub unsafe fn setDataType(&self, data_type: MPSDataType)

Available on crate features MPSCore and MPSCoreTypes only.

Setter for dataType.

pub unsafe fn rowBytes(&self) -> NSUInteger

Available on crate feature MPSCore only.

The stride, in bytes, between corresponding elements of consecutive rows. Must be a multiple of the element size.

pub unsafe fn setRowBytes(&self, row_bytes: NSUInteger)

Available on crate feature MPSCore only.

Setter for rowBytes.

pub unsafe fn matrixBytes(&self) -> NSUInteger

Available on crate feature MPSCore only.

The stride, in bytes, between corresponding elements of consecutive matrices. Must be a multiple of rowBytes.

pub unsafe fn matrixDescriptorWithDimensions_columns_rowBytes_dataType( rows: NSUInteger, columns: NSUInteger, row_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>

👎Deprecated
Available on crate features MPSCore and MPSCoreTypes only.

Create a MPSMatrixDescriptor with the specified dimensions and data type.

Parameter rows: The number of rows of the matrix.

Parameter columns: The number of columns of the matrix.

Parameter rowBytes: The number of bytes between starting elements of consecutive rows. Must be a multiple of the element size.

Parameter dataType: The type of the data to be stored in the matrix.

For performance considerations the optimal row stride may not necessarily be equal to the number of columns in the matrix. The MPSMatrix class provides a method which may be used to determine this value, see the rowBytesForColumns API in the MPSMatrix class. The number of matrices described is initialized to 1.

pub unsafe fn matrixDescriptorWithRows_columns_rowBytes_dataType( rows: NSUInteger, columns: NSUInteger, row_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>

Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn matrixDescriptorWithRows_columns_matrices_rowBytes_matrixBytes_dataType( rows: NSUInteger, columns: NSUInteger, matrices: NSUInteger, row_bytes: NSUInteger, matrix_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>

Available on crate features MPSCore and MPSCoreTypes only.

Create a MPSMatrixDescriptor with the specified dimensions and data type.

Parameter rows: The number of rows of a single matrix.

Parameter columns: The number of columns of a single matrix.

Parameter matrices: The number of matrices in the MPSMatrix object.

Parameter rowBytes: The number of bytes between starting elements of consecutive rows. Must be a multiple of the element size.

Parameter matrixBytes: The number of bytes between starting elements of consecutive matrices. Must be a multiple of rowBytes.

Parameter dataType: The type of the data to be stored in the matrix.

For performance considerations the optimal row stride may not necessarily be equal to the number of columns in the matrix. The MPSMatrix class provides a method which may be used to determine this value, see the rowBytesForColumns API in the MPSMatrix class.

pub unsafe fn rowBytesFromColumns_dataType( columns: NSUInteger, data_type: MPSDataType, ) -> usize

👎Deprecated
Available on crate features MPSCore and MPSCoreTypes only.

Return the recommended row stride, in bytes, for a given number of columns.

Parameter columns: The number of columns in the matrix for which the recommended row stride, in bytes, is to be determined.

Parameter dataType: The type of matrix data values.

To achieve best performance the optimal stride between rows of a matrix is not necessarily equivalent to the number of columns. This method returns the row stride, in bytes, which gives best performance for a given number of columns. Using this row stride to construct your array is recommended, but not required (provided that the stride used is still large enough to allocate a full row of data).

pub unsafe fn rowBytesForColumns_dataType( columns: NSUInteger, data_type: MPSDataType, ) -> usize

Available on crate features MPSCore and MPSCoreTypes only.
Source

impl MPSMatrixDescriptor

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSCore only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSCore only.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}