pub struct MPSMatrixFindTopK { /* private fields */ }MPSCore and MPSKernel and MPSMatrixFindTopK and MPSMatrixTypes only.Expand description
Dependencies: This depends on Metal.framework.
A kernel that find top-K values and their corresponding indices withing a row of a matrix
A MPSMatrixFindTopK object computes finds the ‘k’ largest values within a row of a matrix and returns the value found and the index of the entry in the source matrix. This operation is performed independently on the rows and matrices in batch of the source matrix.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixFindTopK
impl MPSMatrixFindTopK
Sourcepub unsafe fn sourceRows(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn sourceRows(&self) -> NSUInteger
MPSMatrix only.The number of rows to consider from the source in the operation. This property is modifiable and defaults to NSUIntegerMax and the number is adjusted dynamically at kernel encode time (see encodeToCommandBuffer) to fit into the source matrix available starting from sourceMatrixOrigin.x, indicating that by default the whole source matrix is used. If a different size is desired then this should be modified prior to encoding the kernel. It is the user’s responsibility to ensure that the resultIndexMatrix and resultValueMatrix parameters in encodeToCommandBuffer are large enough to accommodate the results of this operation, otherwise the results of the encode call are undefined. NOTE: sourceMatrixOrigin and resultMatrixOrigin from MPSMatrixUnaryKernel can be used to control the starting points in the source and destination at kernel encode time (see encodeToCommandBuffer).
Sourcepub unsafe fn setSourceRows(&self, source_rows: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setSourceRows(&self, source_rows: NSUInteger)
MPSMatrix only.Setter for sourceRows.
Sourcepub unsafe fn sourceColumns(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn sourceColumns(&self) -> NSUInteger
MPSMatrix only.The number of columns to consider from the source in the operation. This property is modifiable and defaults to NSUIntegerMax and the number is adjusted dynamically at kernel encode time (see encodeToCommandBuffer) to fit into the source matrix available starting from sourceMatrixOrigin.y, indicating that by default the whole source matrix is used. If a different size is desired then this should be modified prior to encoding the kernel. It is the user’s responsibility to ensure that the resultIndexMatrix and resultValueMatrix parameters in encodeToCommandBuffer are large enough to accommodate the results of this operation, otherwise the results of the encode call are undefined. NOTE: sourceMatrixOrigin and resultMatrixOrigin from MPSMatrixUnaryKernel can be used to control the starting points in the source and destination at kernel encode time (see encodeToCommandBuffer).
Sourcepub unsafe fn setSourceColumns(&self, source_columns: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setSourceColumns(&self, source_columns: NSUInteger)
MPSMatrix only.Setter for sourceColumns.
Sourcepub unsafe fn indexOffset(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn indexOffset(&self) -> NSUInteger
MPSMatrix only.Specifies a number that will be added to all the indices written to resultIndexMatrix in encodeToCommandBuffer. This value can be used to offset later computations for example by adding the value for the source matrix column offset sourceMatrixOrigin.y. Example: Let numberOfTopKValues be 3, let the source be the following:
source = [ 6.0, 3.0, 8.0, 1.0, 9.0, 4.0, 5.0 ]
and let the sourceMatrixOrigin.y = 2.
Then if indexOffset = 2 then the result value and result index matrices will be:
result values = [ 9.0, 8.0, 5.0 ] result indices = [ 4 , 2 , 6 ],
which gives the user indices into the original source matrix.
On the other hand if the indexOffset = 0 then the results are as follows:
result values = [ 9.0, 8.0, 5.0 ] result indices = [ 2 , 0 , 4 ],
which on the other hand gives the user indices into the submatrix starting from sourceMatrixOrigin.y == 2.
This property is modifiable and defaults to 0. If a different behavior is desired then this should be modified prior to encoding the kernel.
Sourcepub unsafe fn setIndexOffset(&self, index_offset: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setIndexOffset(&self, index_offset: NSUInteger)
MPSMatrix only.Setter for indexOffset.
Sourcepub unsafe fn numberOfTopKValues(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn numberOfTopKValues(&self) -> NSUInteger
MPSMatrix only.The number of highest values (and their indices) to be found in each row by the kernel. This property is initialized in the kernel initialization call initWithDevice, but can be modified before encoding the kernel. Must be less or equal to 16 and requesting more values results in undefined behavior. It is the user’s responsibility to ensure that the resultIndexMatrix and resultValueMatrix parameters in encodeToCommandBuffer are large enough to accommodate the results of this operation, otherwise the results of the encode call are undefined.
Sourcepub unsafe fn setNumberOfTopKValues(&self, number_of_top_k_values: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setNumberOfTopKValues(&self, number_of_top_k_values: NSUInteger)
MPSMatrix only.Setter for numberOfTopKValues.
Sourcepub unsafe fn initWithDevice_numberOfTopKValues(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
number_of_top_k_values: NSUInteger,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithDevice_numberOfTopKValues( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, number_of_top_k_values: NSUInteger, ) -> Retained<Self>
MPSMatrix only.Initialize an MPSMatrixFindTopK object on a device for a given size.
Parameter device: The device on which the kernel will execute.
Parameter numberOfTopKValues: The number of largest values to find from each row,
must be less or equal to 16.
Returns: A valid MPSMatrixFindTopK object or nil, if failure.
Sourcepub unsafe fn initWithDevice(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSMatrix only.Use the above initialization method instead.
Sourcepub unsafe fn encodeToCommandBuffer_inputMatrix_resultIndexMatrix_resultValueMatrix(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
input_matrix: &MPSMatrix,
result_index_matrix: &MPSMatrix,
result_value_matrix: &MPSMatrix,
)
Available on crate feature MPSMatrix only.
pub unsafe fn encodeToCommandBuffer_inputMatrix_resultIndexMatrix_resultValueMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, input_matrix: &MPSMatrix, result_index_matrix: &MPSMatrix, result_value_matrix: &MPSMatrix, )
MPSMatrix only.Encode a MPSMatrixFindTopK object to a command buffer.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.
Parameter inputMatrix: A valid MPSMatrix object which specifies the input matrix.
Parameter resultIndexMatrix: A valid MPSMatrix object which specifies the matrix which will
be overwritten by the result indices.
This matrix must have datatype MPSDataTypeUInt32.
Parameter resultValueMatrix: A valid MPSMatrix object which specifies the matrix which will
be overwritten by the result values.
Certain constraints apply to the sizes of the matrices depending on the sizes requested at initialization time as well as the origins at the time this routine is called:
Both result matrices must be large enough to hold a two dimensional array of ‘sourceRows’ rows and ‘numberOfTopKValues’ columns beginning at resultMatrixOrigin.
The source matrix must be large enough to contain at least ‘numberOfTopKValues’ values starting from sourceMatrixOrigin.y.
Each matrix within the range specified by batchStart and batchSize, which also specifies a valid set of matrices within inputMatrix, resultIndexMatrix and resultValueMatrix, will be processed.
The datatypes of the matrices inputMatrix and resultValueMatrix must match and be either MPSDataTypeFloat32 or MPSDataTypeFloat16.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSMatrix only.NSSecureCoding compatability
See
MPSKernel#initWithCoder.
Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixFindTopK
Parameter device: The MTLDevice on which to make the MPSMatrixFindTopK
Returns: A new MPSMatrixFindTopK object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
MPSMatrix only.Make a copy of this kernel for a new device -
See: MPSKernel
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Sourceimpl MPSMatrixFindTopK
Methods declared on superclass MPSKernel.
impl MPSMatrixFindTopK
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSMatrix only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSMatrixUnaryKernel>
Sourcepub unsafe fn sourceMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn sourceMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setSourceMatrixOrigin(&self, source_matrix_origin: MTLOrigin)
Available on crate feature MPSMatrix only.
pub unsafe fn setSourceMatrixOrigin(&self, source_matrix_origin: MTLOrigin)
MPSMatrix only.Setter for sourceMatrixOrigin.
Sourcepub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
Available on crate feature MPSMatrix only.
pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
MPSMatrix only.Setter for resultMatrixOrigin.
Sourcepub unsafe fn batchStart(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn batchStart(&self) -> NSUInteger
MPSMatrix only.The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.
Sourcepub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
MPSMatrix only.Setter for batchStart.
Sourcepub unsafe fn batchSize(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn batchSize(&self) -> NSUInteger
MPSMatrix only.The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.
Sourcepub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
MPSMatrix only.Setter for batchSize.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl AsRef<AnyObject> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl AsRef<MPSKernel> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl AsRef<MPSKernel> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl AsRef<MPSMatrixFindTopK> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixFindTopK> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl AsRef<MPSMatrixUnaryKernel> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixUnaryKernel> for MPSMatrixFindTopK
MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixUnaryKernel
fn as_ref(&self) -> &MPSMatrixUnaryKernel
Sourceimpl AsRef<NSObject> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl AsRef<NSObject> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl Borrow<AnyObject> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Borrow<AnyObject> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl Borrow<MPSKernel> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Borrow<MPSKernel> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl Borrow<MPSMatrixUnaryKernel> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Borrow<MPSMatrixUnaryKernel> for MPSMatrixFindTopK
MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixUnaryKernel
fn borrow(&self) -> &MPSMatrixUnaryKernel
Sourceimpl Borrow<NSObject> for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Borrow<NSObject> for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl ClassType for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl ClassType for MPSMatrixFindTopK
MPSMatrix only.Sourceconst NAME: &'static str = "MPSMatrixFindTopK"
const NAME: &'static str = "MPSMatrixFindTopK"
Sourcetype Super = MPSMatrixUnaryKernel
type Super = MPSMatrixUnaryKernel
Sourcetype ThreadKind = <<MPSMatrixFindTopK as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixFindTopK as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl CopyingHelper for MPSMatrixFindTopK
MPSMatrix only.Sourcetype Result = MPSMatrixFindTopK
type Result = MPSMatrixFindTopK
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Debug for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl Deref for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Deref for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl Hash for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Hash for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl Message for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl Message for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl NSCoding for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl NSCoding for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl NSCopying for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl NSCopying for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl NSObjectProtocol for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl NSObjectProtocol for MPSMatrixFindTopK
MPSMatrix only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl NSSecureCoding for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl PartialEq for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl PartialEq for MPSMatrixFindTopK
MPSMatrix only.Sourceimpl RefEncode for MPSMatrixFindTopK
Available on crate feature MPSMatrix only.
impl RefEncode for MPSMatrixFindTopK
MPSMatrix only.Sourceconst ENCODING_REF: Encoding = <MPSMatrixUnaryKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSMatrixUnaryKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixFindTopK
MPSMatrix only.impl Eq for MPSMatrixFindTopK
MPSMatrix only.