MPSMatrixFullyConnected

Struct MPSMatrixFullyConnected 

pub struct MPSMatrixFullyConnected { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSMatrix and MPSMatrixFullyConnected and MPSMatrixTypes only.
Dependencies: This depends on Metal.framework.

Applies a fully connected neural network layer by performing a a matrix multiplication, adding a bias vector, scaling, and applying a neuron activation function.

A MPSMatrixFullyConnected object computes:

y = neuron(alpha * x * W + bias)

y is the output matrix, x and W are input matrices corresponding to a collection of input vectors and weights respectively, and bias is a vector which is broadcast and accumulated to each row of the product. alpha is a scale factor applied to the product.

neuron() is a pointwise function applied to the intermediate result.

See also Apple’s documentation

Implementations

Source

impl MPSMatrixFullyConnected

pub unsafe fn sourceNumberOfFeatureVectors(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of input vectors which make up the input array. This is equivalent to the number of rows to consider from the primary source matrix. This property is modifiable and defaults to NSUIntegerMax. At encode time the larger of this property or the available number of inputs is used. The value of NSUIntegerMax thus indicates that all available input rows (beginning at primarySourceMatrixOrigin.x) should be considered.

pub unsafe fn setSourceNumberOfFeatureVectors( &self, source_number_of_feature_vectors: NSUInteger, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceInputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The input size to to use in the operation. This is equivalent to the number of columns and the number of rows in the primary (input array) and secondary (weight array) source matrices respectively. This property is modifiable and defaults to NSUIntegerMax. At encode time the larger of this property or the available input size is used. The value of NSUIntegerMax thus indicates that all available columns in the input array (beginning at primarySourceMatrixOrigin.y) and all available rows in the weight array (beginning at secondarySourceMatrixOrigin.x) should be considered. Note: The value used in the operation will be MIN(MIN(inputMatrix.columns - primarySourceMatrixOrigin.y, weightMatrix.rows - secondarySourceMatrixOrigin.x), sourceInputFeatureChannels)

pub unsafe fn setSourceInputFeatureChannels( &self, source_input_feature_channels: NSUInteger, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceOutputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The output size to to use in the operation. This is equivalent to the number of columns to consider in the weight array, or the secondary source matrix. This property is modifiable and defaults to NSUIntegerMax. At encode time the larger of this property or the available output size is used. The value of NSUIntegerMax thus indicates that all available columns in the weight array (beginning at secondarySourceMatrixOrigin.y) should be considered.

pub unsafe fn setSourceOutputFeatureChannels( &self, source_output_feature_channels: NSUInteger, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn alpha(&self) -> c_double

Available on crate feature MPSNeuralNetwork only.

The scale factor to apply to the product. Specified in double precision. Will be converted to the appropriate precision in the implementation subject to rounding and/or clamping as necessary. Defaults to 1.0 at initialization time.

pub unsafe fn setAlpha(&self, alpha: c_double)

Available on crate feature MPSNeuralNetwork only.

Setter for alpha.

pub unsafe fn setNeuronType_parameterA_parameterB_parameterC( &self, neuron_type: MPSCNNNeuronType, parameter_a: c_float, parameter_b: c_float, parameter_c: c_float, )

Available on crate features MPSNeuralNetwork and MPSCNNNeuronType only.

Specifies a neuron activation function to be used.

This method can be used to add a neuron activation funtion of given type with associated scalar parameters A, B, and C that are shared across all output values. Note that this method can only be used to specify neurons which are specified by three (or fewer) parameters shared across all output values (or channels, in CNN nomenclature). It is an error to call this method for neuron activation functions like MPSCNNNeuronTypePReLU, which require per-channel parameter values. For those kind of neuron activation functions, use appropriate setter functions. An MPSMatrixFullyConnected kernel is initialized with a default neuron function of MPSCNNNeuronTypeNone.

Parameter neuronType: Type of neuron activation function. For full list see MPSCNNNeuronType.h

Parameter parameterA: parameterA of neuron activation that is shared across all output values.

Parameter parameterB: parameterB of neuron activation that is shared across all output values.

Parameter parameterC: parameterC of neuron activation that is shared across all output values.

pub unsafe fn neuronType(&self) -> MPSCNNNeuronType

Available on crate features MPSNeuralNetwork and MPSCNNNeuronType only.

Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method

pub unsafe fn neuronParameterA(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method

pub unsafe fn neuronParameterB(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method

pub unsafe fn neuronParameterC(&self) -> c_float

Available on crate feature MPSNeuralNetwork only.

Getter funtion for neuronType set using setNeuronType:parameterA:parameterB:parameterC method

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeToCommandBuffer_inputMatrix_weightMatrix_biasVector_resultMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, input_matrix: &MPSMatrix, weight_matrix: &MPSMatrix, bias_vector: Option<&MPSVector>, result_matrix: &MPSMatrix, )

Available on crate feature MPSNeuralNetwork only.

Encode a MPSMatrixFullyConnected object to a command buffer.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.

Parameter inputMatrix: A valid MPSMatrix object which specifies the input array.

Parameter weightMatrix: A valid MPSMatrix object which specifies the weight array.

Parameter biasVector: A valid MPSVector object which specifies the bias values, or a null object to indicate that no bias is to be applied.

Parameter resultMatrix: A valid MPSMatrix object which specifies the output array.

Encodes the operation to the specified command buffer. resultMatrix must be large enough to hold a MIN(sourceNumberOfInputs, inputMatrix.rows - primarySourceMatrixOrigin.x) x MIN(sourceOutputFeatureChannels, weightMatrix.columns - secondarySourceMatrixOrigin.y) array.

The bias vector must contain at least MIN(sourceOutputFeatureChannels, weightMatrix.columns - secondarySourceMatrixOrigin.y) elements.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

NSSecureCoding compatability

See MPSKernel#initWithCoder. Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixFullyConnected

Parameter device: The MTLDevice on which to make the MPSMatrixFullyConnected object.

Returns: A new MPSMatrixFullyConnected object, or nil if failure.

Safety

a_decoder possibly has further requirements.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Make a copy of this kernel for a new device -

See: MPSKernel

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: A pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Source

impl MPSMatrixFullyConnected

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSMatrixFullyConnected

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = MPSMatrixBinaryKernel>

pub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin

The origin, relative to [0, 0] in the primary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setPrimarySourceMatrixOrigin( &self, primary_source_matrix_origin: MTLOrigin, )

pub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin

The origin, relative to [0, 0] in the secondary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setSecondarySourceMatrixOrigin( &self, secondary_source_matrix_origin: MTLOrigin, )

pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin

The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)

Setter for resultMatrixOrigin.

pub unsafe fn batchStart(&self) -> NSUInteger

The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.

pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)

Setter for batchStart.

pub unsafe fn batchSize(&self) -> NSUInteger

The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.

pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)

Setter for batchSize.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}