MPSMatrixFullyConnectedGradient

Struct MPSMatrixFullyConnectedGradient 

pub struct MPSMatrixFullyConnectedGradient { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSMatrix and MPSMatrixFullyConnected and MPSMatrixTypes only.
Dependencies: This depends on Metal.framework.

Computes the gradient of the fully connected layer with respect to either the weights and bias terms or the input feature vectors.

An MPSMatrixFullyConnectedGradient kernel may be used to compute the gradients corresponding to a MPSMatrixFullyConnected kernel. The properties, input, and weight data must match those values used in the forward computation. This kernel does not compute the gradient of any non-identity activation function which may have been applied in the forward kernel. Such a kernel must be expressed using both MPSMatrixFullyConnected and MPSMatrixNeuron if a gradient is to be computed.

See also Apple’s documentation

Implementations

Source

impl MPSMatrixFullyConnectedGradient

pub unsafe fn sourceNumberOfFeatureVectors(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of input vectors which make up the input array. This is equivalent to the number of rows in both the input matrix and the source gradient matrix.

This value should be equal to the corresponding value in the forward fully connected kernel.

pub unsafe fn setSourceNumberOfFeatureVectors( &self, source_number_of_feature_vectors: NSUInteger, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceOutputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of feature channels in the output of the forward fully connected layer. This is equivalent to the number of columns in both the weight matrix and the source gradient matrix.

This value should be equal to the corresponding value in the forward fully connected kernel.

pub unsafe fn setSourceOutputFeatureChannels( &self, source_output_feature_channels: NSUInteger, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceInputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of feature channels in the input to the forward fully connected layer. This is equivalent to the number of columns in the input matrix.

This value should be equal to the corresponding value in the forward fully connected kernel.

pub unsafe fn setSourceInputFeatureChannels( &self, source_input_feature_channels: NSUInteger, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn alpha(&self) -> c_double

Available on crate feature MPSNeuralNetwork only.

Scale factor to apply to the product. This value should be equal to the corresponding value in the forward fully connected kernel.

pub unsafe fn setAlpha(&self, alpha: c_double)

Available on crate feature MPSNeuralNetwork only.

Setter for alpha.

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeGradientForDataToCommandBuffer_gradientMatrix_weightMatrix_resultGradientForDataMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, gradient_matrix: &MPSMatrix, weight_matrix: &MPSMatrix, result_gradient_for_data_matrix: &MPSMatrix, )

Available on crate feature MPSNeuralNetwork only.

Encode a MPSMatrixFullyConnectedGradient object to a command buffer and produce the gradient of the loss function with respect to the input data.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.

Parameter gradientMatrix: A valid MPSMatrix object which specifies the input gradient.

Parameter weightMatrix: A valid MPSMatrix object which specifies the weight array.

Parameter resultGradientForDataMatrix: A valid MPSMatrix object which specifies the result gradient.

This operation computes the resulting gradient of the loss function with respect to the forward kernel’s input data. weightMatrix should contain the same values used to compute the result of the forward kernel.

pub unsafe fn encodeGradientForWeightsAndBiasToCommandBuffer_gradientMatrix_inputMatrix_resultGradientForWeightMatrix_resultGradientForBiasVector( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, gradient_matrix: &MPSMatrix, input_matrix: &MPSMatrix, result_gradient_for_weight_matrix: &MPSMatrix, result_gradient_for_bias_vector: Option<&MPSVector>, )

Available on crate feature MPSNeuralNetwork only.

Encode a MPSMatrixFullyConnectedGradient object to a command buffer and produce the gradient of the loss function with respect to the weight matrix and bias vector.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.

Parameter gradientMatrix: A valid MPSMatrix object which specifies the input gradient.

Parameter inputMatrix: A valid MPSMatrix object which specifies the input array.

Parameter resultGradientForWeightMatrix: A valid MPSMatrix object which specifies the resulting gradients with respect to the weights.

Parameter resultGradientForBiasVector: A valid MPSVector object which specifies the resulting gradients with respect to the bias terms. If NULL these values will not be returned.

This operation computes the resulting gradient of the loss function with respect to the forward kernel’s weight data. inputMatrix should contain the same values used to compute the result of the forward kernel.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

NSSecureCoding compatability

See MPSKernel#initWithCoder. Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixFullyConnectedGradient

Parameter device: The MTLDevice on which to make the MPSMatrixFullyConnectedGradient object.

Returns: A new MPSMatrixFullyConnected object, or nil if failure.

Safety

a_decoder possibly has further requirements.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Make a copy of this kernel for a new device -

See: MPSKernel

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: A pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Source

impl MPSMatrixFullyConnectedGradient

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSMatrixFullyConnectedGradient

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = MPSMatrixBinaryKernel>

pub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin

The origin, relative to [0, 0] in the primary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setPrimarySourceMatrixOrigin( &self, primary_source_matrix_origin: MTLOrigin, )

pub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin

The origin, relative to [0, 0] in the secondary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setSecondarySourceMatrixOrigin( &self, secondary_source_matrix_origin: MTLOrigin, )

pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin

The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)

Setter for resultMatrixOrigin.

pub unsafe fn batchStart(&self) -> NSUInteger

The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.

pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)

Setter for batchStart.

pub unsafe fn batchSize(&self) -> NSUInteger

The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.

pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)

Setter for batchSize.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}