MPSMatrixMultiplication

Struct MPSMatrixMultiplication 

pub struct MPSMatrixMultiplication { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSMatrixMultiplication only.
Dependencies: This depends on Metal.framework.

A matrix multiplication kernel.

A MPSMatrixMultiplication object computes:

C = alpha * op(A) * op(B) + beta * C

A, B, and C are matrices which are represented by MPSMatrix objects. alpha and beta are scalar values (of the same data type as values of C) which are applied as shown above. A and B may each have an optional transposition operation applied.

A, B, and C (also referred to in later discussions as the left input matrix, the right input matrix, and the result matrix respectively).

A MPSMatrixMultiplication object is initialized with the transpose operators to apply to A and B, sizes for the operation to perform, and the scalar values alpha and beta.

See also Apple’s documentation

Implementations

Source

impl MPSMatrixMultiplication

pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin

Available on crate feature MPSMatrix only.

The origin, relative to [0, 0] in the result matrix, at which to start writing (and reading if necessary) results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)

Available on crate feature MPSMatrix only.

Setter for resultMatrixOrigin.

pub unsafe fn leftMatrixOrigin(&self) -> MTLOrigin

Available on crate feature MPSMatrix only.

The origin, relative to [0, 0] in the left input matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setLeftMatrixOrigin(&self, left_matrix_origin: MTLOrigin)

Available on crate feature MPSMatrix only.

Setter for leftMatrixOrigin.

pub unsafe fn rightMatrixOrigin(&self) -> MTLOrigin

Available on crate feature MPSMatrix only.

The origin, relative to [0, 0] in the right input matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.

pub unsafe fn setRightMatrixOrigin(&self, right_matrix_origin: MTLOrigin)

Available on crate feature MPSMatrix only.

Setter for rightMatrixOrigin.

pub unsafe fn batchStart(&self) -> NSUInteger

Available on crate feature MPSMatrix only.

The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.

pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)

Available on crate feature MPSMatrix only.

Setter for batchStart.

pub unsafe fn batchSize(&self) -> NSUInteger

Available on crate feature MPSMatrix only.

The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed.

pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)

Available on crate feature MPSMatrix only.

Setter for batchSize.

pub unsafe fn initWithDevice_transposeLeft_transposeRight_resultRows_resultColumns_interiorColumns_alpha_beta( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, transpose_left: bool, transpose_right: bool, result_rows: NSUInteger, result_columns: NSUInteger, interior_columns: NSUInteger, alpha: c_double, beta: c_double, ) -> Retained<Self>

Available on crate feature MPSMatrix only.

Initialize an MPSMatrixMultiplication object on a device for a given size and desired transpose and scale values.

Parameter device: The device on which the kernel will execute.

Parameter transposeLeft: A boolean value which indicates if the left input matrix should be used in transposed form. If ‘YES’ then op(A) = A**T, otherwise op(A) = A.

Parameter transposeRight: A boolean value which indicates if the right input matrix should be used in transposed form. If ‘YES’ then op(B) = B**T, otherwise op(B) = B.

Parameter resultRows: The number of rows in the result matrix, M in BLAS GEMM description.

Parameter resultColumns: The number of columns in the result matrix, N in BLAS GEMM description.

Parameter interiorColumns: The number of columns of the left input matrix after the appropriate transpose operation has been applied. K in BLAS GEMM description.

Parameter alpha: The scale factor to apply to the product. Specified in double precision. Will be converted to the appropriate precision in the implementation subject to rounding and/or clamping as necessary.

Parameter beta: The scale factor to apply to the initial values of C. Specified in double precision. Will be converted to the appropriate precision in the implementation subject to rounding and/or clamping as necessary.

Returns: A valid MPSMatrixMultiplication object or nil, if failure.

pub unsafe fn initWithDevice_resultRows_resultColumns_interiorColumns( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, result_rows: NSUInteger, result_columns: NSUInteger, interior_columns: NSUInteger, ) -> Retained<Self>

Available on crate feature MPSMatrix only.

Convenience initialization for a matrix-matrix multiplication with no transpositions, unit scaling of the product, and no accumulation of the result. The scaling factors alpha and beta are taken to be 1.0 and 0.0 respectively.

Parameter device: The device on which the kernel will execute.

Parameter resultRows: The number of rows in the result matrix, M in BLAS GEMM description.

Parameter resultColumns: The number of columns in the result matrix, N in BLAS GEMM description.

Parameter interiorColumns: The number of columns of the left input matrix. K in BLAS GEMM description.

Returns: A valid MPSMatrixMultiplication object or nil, if failure.

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSMatrix only.

Use the above initialization method instead.

pub unsafe fn encodeToCommandBuffer_leftMatrix_rightMatrix_resultMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, left_matrix: &MPSMatrix, right_matrix: &MPSMatrix, result_matrix: &MPSMatrix, )

Available on crate feature MPSMatrix only.

Encode a MPSMatrixMultiplication object to a command buffer.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.

Parameter leftMatrix: A valid MPSMatrix object which specifies the left input matrix.

Parameter rightMatrix: A valid MPSMatrix object which specifies the right input matrix.

Parameter resultMatrix: A valid MPSMatrix object which specifies the addend matrix which will also be overwritten by the result.

Certain constraints apply to the sizes of the matrices depending on the transposition operations and sizes requested at initialization time as well as the origins at the time this routine is called:

The left input matrix must be large enough to hold an array of size resultRows x interiorColumns elements beginning at leftMatrixOrigin.

The right input matrix must be large enough to hold an array of size interiorColumns x resultColumns elements beginning at rightMatrixOrigin.

The result matrix must be large enough to hold an array of size resultRows x resultColumns elements beginning at resultMatrixOrigin.

Each matrix within the range specified by batchStart and batchSize, which also specifies a valid set of matrices within leftMatrix, rightMatrix, and resultMatrix, will be processed.

Source

impl MPSMatrixMultiplication

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSMatrix only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSMatrix only.

NSSecureCoding compatability

While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.

Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel

Parameter device: The MTLDevice on which to make the MPSKernel

Returns: A new MPSKernel object, or nil if failure.

Safety

a_decoder possibly has further requirements.

Source

impl MPSMatrixMultiplication

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSMatrix only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSMatrix only.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}