pub struct MPSMatrixRandom { /* private fields */ }MPSCore and MPSKernel and MPSMatrix and MPSMatrixRandom only.Expand description
Kernels that implement random number generation.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixRandom
impl MPSMatrixRandom
Sourcepub unsafe fn destinationDataType(&self) -> MPSDataType
Available on crate feature MPSCoreTypes only.
pub unsafe fn destinationDataType(&self) -> MPSDataType
MPSCoreTypes only.The type of the data which makes up the values of the result. Supported values are: MPSDataTypeUInt32 MPSDataTypeFloat32
Default is MPSDataTypeUInt32
Sourcepub unsafe fn distributionType(&self) -> MPSMatrixRandomDistribution
pub unsafe fn distributionType(&self) -> MPSMatrixRandomDistribution
The distribution from which to generate random values.
Default is MPSMatrixRandomDistributionDefault
Sourcepub unsafe fn batchStart(&self) -> NSUInteger
pub unsafe fn batchStart(&self) -> NSUInteger
The starting index in the destination batch.
Sourcepub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
Setter for batchStart.
Sourcepub unsafe fn batchSize(&self) -> NSUInteger
pub unsafe fn batchSize(&self) -> NSUInteger
The size of the batch to process.
Sourcepub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
Setter for batchSize.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
Sourcepub unsafe fn encodeToCommandBuffer_destinationVector(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
destination_vector: &MPSVector,
)
pub unsafe fn encodeToCommandBuffer_destinationVector( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, destination_vector: &MPSVector, )
Encode a MPSMatrixRandom kernel into a command Buffer.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter
Parameter destinationVector: A valid MPSVector to contain the result.
Sourcepub unsafe fn encodeToCommandBuffer_destinationMatrix(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
destination_matrix: &MPSMatrix,
)
pub unsafe fn encodeToCommandBuffer_destinationMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, destination_matrix: &MPSMatrix, )
Encode a MPSMatrixRandom kernel into a command Buffer.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter
Parameter destinationMatrix: A valid MPSMatrix to contain the result.
Sourceimpl MPSMatrixRandom
Methods declared on superclass MPSKernel.
impl MPSMatrixRandom
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
NSSecureCoding compatability
While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.
Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel
Parameter device: The MTLDevice on which to make the MPSKernel
Returns: A new MPSKernel object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
this is difficult to use correctly, use Ivar::load instead.
Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixRandom
impl AsRef<AnyObject> for MPSMatrixRandom
Sourceimpl AsRef<MPSKernel> for MPSMatrixRandom
impl AsRef<MPSKernel> for MPSMatrixRandom
Sourceimpl AsRef<MPSMatrixRandom> for MPSMatrixRandom
impl AsRef<MPSMatrixRandom> for MPSMatrixRandom
Sourceimpl AsRef<MPSMatrixRandom> for MPSMatrixRandomMTGP32
impl AsRef<MPSMatrixRandom> for MPSMatrixRandomMTGP32
Sourcefn as_ref(&self) -> &MPSMatrixRandom
fn as_ref(&self) -> &MPSMatrixRandom
Sourceimpl AsRef<MPSMatrixRandom> for MPSMatrixRandomPhilox
impl AsRef<MPSMatrixRandom> for MPSMatrixRandomPhilox
Sourcefn as_ref(&self) -> &MPSMatrixRandom
fn as_ref(&self) -> &MPSMatrixRandom
Sourceimpl AsRef<NSObject> for MPSMatrixRandom
impl AsRef<NSObject> for MPSMatrixRandom
Sourceimpl Borrow<AnyObject> for MPSMatrixRandom
impl Borrow<AnyObject> for MPSMatrixRandom
Sourceimpl Borrow<MPSKernel> for MPSMatrixRandom
impl Borrow<MPSKernel> for MPSMatrixRandom
Sourceimpl Borrow<MPSMatrixRandom> for MPSMatrixRandomMTGP32
impl Borrow<MPSMatrixRandom> for MPSMatrixRandomMTGP32
Sourcefn borrow(&self) -> &MPSMatrixRandom
fn borrow(&self) -> &MPSMatrixRandom
Sourceimpl Borrow<MPSMatrixRandom> for MPSMatrixRandomPhilox
impl Borrow<MPSMatrixRandom> for MPSMatrixRandomPhilox
Sourcefn borrow(&self) -> &MPSMatrixRandom
fn borrow(&self) -> &MPSMatrixRandom
Sourceimpl Borrow<NSObject> for MPSMatrixRandom
impl Borrow<NSObject> for MPSMatrixRandom
Sourceimpl ClassType for MPSMatrixRandom
impl ClassType for MPSMatrixRandom
Sourceconst NAME: &'static str = "MPSMatrixRandom"
const NAME: &'static str = "MPSMatrixRandom"
Sourcetype ThreadKind = <<MPSMatrixRandom as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixRandom as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixRandom
impl CopyingHelper for MPSMatrixRandom
Sourcetype Result = MPSMatrixRandom
type Result = MPSMatrixRandom
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixRandom
impl Debug for MPSMatrixRandom
Sourceimpl Deref for MPSMatrixRandom
impl Deref for MPSMatrixRandom
Sourceimpl Hash for MPSMatrixRandom
impl Hash for MPSMatrixRandom
Sourceimpl Message for MPSMatrixRandom
impl Message for MPSMatrixRandom
Sourceimpl NSCoding for MPSMatrixRandom
impl NSCoding for MPSMatrixRandom
Sourceunsafe fn encodeWithCoder(&self, coder: &NSCoder)
unsafe fn encodeWithCoder(&self, coder: &NSCoder)
NSCoder only.Sourceimpl NSCopying for MPSMatrixRandom
impl NSCopying for MPSMatrixRandom
Sourceimpl NSObjectProtocol for MPSMatrixRandom
impl NSObjectProtocol for MPSMatrixRandom
Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
use isKindOfClass directly, or cast your objects with AnyObject::downcast_ref