pub struct MPSMatrixRandomDistributionDescriptor { /* private fields */ }MPSMatrixRandom only.Expand description
Dependencies: This depends on Metal.framework
Decribes properties of a distribution of random values.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixRandomDistributionDescriptor
impl MPSMatrixRandomDistributionDescriptor
Sourcepub unsafe fn distributionType(&self) -> MPSMatrixRandomDistribution
Available on crate feature MPSMatrix only.
pub unsafe fn distributionType(&self) -> MPSMatrixRandomDistribution
MPSMatrix only.The type of distribution.
Sourcepub unsafe fn setDistributionType(
&self,
distribution_type: MPSMatrixRandomDistribution,
)
Available on crate feature MPSMatrix only.
pub unsafe fn setDistributionType( &self, distribution_type: MPSMatrixRandomDistribution, )
MPSMatrix only.Setter for distributionType.
Sourcepub unsafe fn minimum(&self) -> c_float
Available on crate feature MPSMatrix only.
pub unsafe fn minimum(&self) -> c_float
MPSMatrix only.For distributions of values bounded below, this value describes the minimum.
Sourcepub unsafe fn setMinimum(&self, minimum: c_float)
Available on crate feature MPSMatrix only.
pub unsafe fn setMinimum(&self, minimum: c_float)
MPSMatrix only.Setter for minimum.
Sourcepub unsafe fn maximum(&self) -> c_float
Available on crate feature MPSMatrix only.
pub unsafe fn maximum(&self) -> c_float
MPSMatrix only.For distributions of values bounded above, this value describes the maximum.
Sourcepub unsafe fn setMaximum(&self, maximum: c_float)
Available on crate feature MPSMatrix only.
pub unsafe fn setMaximum(&self, maximum: c_float)
MPSMatrix only.Setter for maximum.
Sourcepub unsafe fn mean(&self) -> c_float
Available on crate feature MPSMatrix only.
pub unsafe fn mean(&self) -> c_float
MPSMatrix only.The value to use for distributions described by their mean.
Sourcepub unsafe fn setMean(&self, mean: c_float)
Available on crate feature MPSMatrix only.
pub unsafe fn setMean(&self, mean: c_float)
MPSMatrix only.Setter for mean.
Sourcepub unsafe fn standardDeviation(&self) -> c_float
Available on crate feature MPSMatrix only.
pub unsafe fn standardDeviation(&self) -> c_float
MPSMatrix only.The value to use for distributions described by their standardDeviation.
Sourcepub unsafe fn setStandardDeviation(&self, standard_deviation: c_float)
Available on crate feature MPSMatrix only.
pub unsafe fn setStandardDeviation(&self, standard_deviation: c_float)
MPSMatrix only.Setter for standardDeviation.
Sourcepub unsafe fn uniformDistributionDescriptorWithMinimum_maximum(
minimum: c_float,
maximum: c_float,
) -> Retained<MPSMatrixRandomDistributionDescriptor>
Available on crate feature MPSMatrix only.
pub unsafe fn uniformDistributionDescriptorWithMinimum_maximum( minimum: c_float, maximum: c_float, ) -> Retained<MPSMatrixRandomDistributionDescriptor>
MPSMatrix only.Make a descriptor for a uniform distribution of floating point values in the range [minimum, maximum).
Parameter minimum: The lower bound of the range.
Parameter maximum: The upper bound of the range.
Returns: A valid MPSMatrixRandomDistribution object or nil, if failure.
Sourcepub unsafe fn normalDistributionDescriptorWithMean_standardDeviation(
mean: c_float,
standard_deviation: c_float,
) -> Retained<MPSMatrixRandomDistributionDescriptor>
Available on crate feature MPSMatrix only.
pub unsafe fn normalDistributionDescriptorWithMean_standardDeviation( mean: c_float, standard_deviation: c_float, ) -> Retained<MPSMatrixRandomDistributionDescriptor>
MPSMatrix only.Make a descriptor for a normal distribution of floating point values.
Parameter mean: The mean of the distribution
Parameter standardDeviation: The standard deviation of the distribution.
Returns: A valid MPSMatrixRandomDistribution object or nil if failure.
Sourcepub unsafe fn normalDistributionDescriptorWithMean_standardDeviation_minimum_maximum(
mean: c_float,
standard_deviation: c_float,
minimum: c_float,
maximum: c_float,
) -> Retained<MPSMatrixRandomDistributionDescriptor>
Available on crate feature MPSMatrix only.
pub unsafe fn normalDistributionDescriptorWithMean_standardDeviation_minimum_maximum( mean: c_float, standard_deviation: c_float, minimum: c_float, maximum: c_float, ) -> Retained<MPSMatrixRandomDistributionDescriptor>
MPSMatrix only.Make a descriptor for a truncated normal distribution of floating point values.
Parameter mean: The mean of the distribution
Parameter standardDeviation: The standard deviation of the distribution.
Parameter minimum: The lower bound of the distribution
Parameter maximum: The upper bound of the distribution
Returns: A valid MPSMatrixRandomDistribution object or nil if failure.
Sourcepub unsafe fn defaultDistributionDescriptor() -> Retained<MPSMatrixRandomDistributionDescriptor>
Available on crate feature MPSMatrix only.
pub unsafe fn defaultDistributionDescriptor() -> Retained<MPSMatrixRandomDistributionDescriptor>
MPSMatrix only.Make a descriptor for a default distribution.
Returns: A valid MPSMatrixRandomDistribution object or nil, if failure.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl AsRef<AnyObject> for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl AsRef<MPSMatrixRandomDistributionDescriptor> for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixRandomDistributionDescriptor> for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl AsRef<NSObject> for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl AsRef<NSObject> for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl Borrow<AnyObject> for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl Borrow<AnyObject> for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl Borrow<NSObject> for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl Borrow<NSObject> for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl ClassType for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl ClassType for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceconst NAME: &'static str = "MPSMatrixRandomDistributionDescriptor"
const NAME: &'static str = "MPSMatrixRandomDistributionDescriptor"
Sourcetype ThreadKind = <<MPSMatrixRandomDistributionDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixRandomDistributionDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl CopyingHelper for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourcetype Result = MPSMatrixRandomDistributionDescriptor
type Result = MPSMatrixRandomDistributionDescriptor
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl Debug for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl Deref for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl Deref for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl Hash for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl Hash for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl Message for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl Message for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl NSCopying for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl NSCopying for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl NSObjectProtocol for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl NSObjectProtocol for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl PartialEq for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceimpl RefEncode for MPSMatrixRandomDistributionDescriptor
Available on crate feature MPSMatrix only.
impl RefEncode for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.impl Eq for MPSMatrixRandomDistributionDescriptor
MPSMatrix only.