pub struct MPSMatrixSoftMaxGradient { /* private fields */ }MPSCore and MPSKernel and MPSMatrixSoftMax and MPSMatrixTypes only.Expand description
Dependencies: This depends on Metal.framework.
Computes the gradient corresponding to a forward MPSMatrixSoftMax object.
A MPSMatrixSoftMaxGradient object computes:
dL_dX_ij = Y_ij * (dL_dY_ij - sum_k(dL_dY_ik * Y_ik)
Where dL_dX is the resulting gradient of the loss function with respect to the original input to the forward MPSMatrixSoftMax operation, Y is the output of the forward MPSMatrixSoftMax operation, and dL_dY is the gradient of the loss function with respect to Y.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixSoftMaxGradient
impl MPSMatrixSoftMaxGradient
Sourcepub unsafe fn sourceRows(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn sourceRows(&self) -> NSUInteger
MPSMatrix only.The number of rows to consider from the sources in the operation. This property is modifiable and defaults to NSUIntegerMax and the number is adjusted dynamically at kernel encode time (see encodeToCommandBuffer) to fit into the source matrices available starting from [primary/secondary]SourceMatrixOrigin.x, indicating that by default the whole source matrix is used. If a different size is desired then this should be modified prior to encoding the kernel. It is the user’s responsibility to ensure that the resultMatrix parameter in encodeToCommandBuffer is large enough to accommodate the results of this operation, otherwise the results of the encode call are undefined. NOTE: primarySourceMatrixOrigin, secondarySourceMatrixOrigin and resultMatrixOrigin from MPSMatrixBinaryKernel can be used to control the starting points in the primary source, secondary source, and result matrices respectively.
Sourcepub unsafe fn setSourceRows(&self, source_rows: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setSourceRows(&self, source_rows: NSUInteger)
MPSMatrix only.Setter for sourceRows.
Sourcepub unsafe fn sourceColumns(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn sourceColumns(&self) -> NSUInteger
MPSMatrix only.The number of columns to consider from the sources in the operation. This property is modifiable and defaults to NSUIntegerMax and the number is adjusted dynamically at kernel encode time (see encodeToCommandBuffer) to fit into the source matrices available starting from [primary/secondary]SourceMatrixOrigin.y, indicating that by default the whole source matrix is used. If a different size is desired then this should be modified prior to encoding the kernel. It is the user’s responsibility to ensure that the resultMatrix parameter in encodeToCommandBuffer is large enough to accommodate the results of this operation, otherwise the results of the encode call are undefined. NOTE: primarySourceMatrixOrigin, secondarySourceMatrixOrigin and resultMatrixOrigin from MPSMatrixBinaryKernel can be used to control the starting points in the primary source, secondary source, and result matrices respectively.
Sourcepub unsafe fn setSourceColumns(&self, source_columns: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setSourceColumns(&self, source_columns: NSUInteger)
MPSMatrix only.Setter for sourceColumns.
Sourcepub unsafe fn initWithDevice(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSMatrix only.Initialize an MPSMatrixSoftMaxGradient object on a device.
Parameter device: The device on which the kernel will execute.
Returns: A valid MPSMatrixSoftMaxGradient object or nil, if failure.
Sourcepub unsafe fn encodeToCommandBuffer_gradientMatrix_forwardOutputMatrix_resultMatrix(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
gradient_matrix: &MPSMatrix,
forward_output_matrix: &MPSMatrix,
result_matrix: &MPSMatrix,
)
Available on crate feature MPSMatrix only.
pub unsafe fn encodeToCommandBuffer_gradientMatrix_forwardOutputMatrix_resultMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, gradient_matrix: &MPSMatrix, forward_output_matrix: &MPSMatrix, result_matrix: &MPSMatrix, )
MPSMatrix only.Encode a MPSMatrixSoftMaxGradient object to a command buffer.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.
Parameter gradientMatrix: A MPSMatrix object containing gradient values with respect
to the forward operation’s output. dL_dY in the class
description.
Parameter forwardOutputMatrix: A MPSMatrix object containing the output values from the
forward operation. Y in the class description.
Parameter resultMatrix: The MPSMatrix object to hold the resulting gradient values
with respect to the forward operation’s input. dL_dX in the
class description.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSMatrix only.NSSecureCoding compatability
See
MPSKernel#initWithCoder.
Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixSoftMaxGradient
Parameter device: The MTLDevice on which to make the MPSMatrixSoftMaxGradient
Returns: A new MPSMatrixSoftMaxGradient object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
Available on crate feature MPSMatrix only.
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
MPSMatrix only.Make a copy of this kernel for a new device -
See: MPSKernel
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Sourceimpl MPSMatrixSoftMaxGradient
Methods declared on superclass MPSKernel.
impl MPSMatrixSoftMaxGradient
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSMatrix only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSMatrix only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSMatrixBinaryKernel>
Sourcepub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the primary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setPrimarySourceMatrixOrigin(
&self,
primary_source_matrix_origin: MTLOrigin,
)
Available on crate feature MPSMatrix only.
pub unsafe fn setPrimarySourceMatrixOrigin( &self, primary_source_matrix_origin: MTLOrigin, )
MPSMatrix only.Setter for primarySourceMatrixOrigin.
Sourcepub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the secondary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setSecondarySourceMatrixOrigin(
&self,
secondary_source_matrix_origin: MTLOrigin,
)
Available on crate feature MPSMatrix only.
pub unsafe fn setSecondarySourceMatrixOrigin( &self, secondary_source_matrix_origin: MTLOrigin, )
MPSMatrix only.Setter for secondarySourceMatrixOrigin.
Sourcepub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
Available on crate feature MPSMatrix only.
pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
MPSMatrix only.The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
Available on crate feature MPSMatrix only.
pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
MPSMatrix only.Setter for resultMatrixOrigin.
Sourcepub unsafe fn batchStart(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn batchStart(&self) -> NSUInteger
MPSMatrix only.The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.
Sourcepub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
MPSMatrix only.Setter for batchStart.
Sourcepub unsafe fn batchSize(&self) -> NSUInteger
Available on crate feature MPSMatrix only.
pub unsafe fn batchSize(&self) -> NSUInteger
MPSMatrix only.The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.
Sourcepub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
Available on crate feature MPSMatrix only.
pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
MPSMatrix only.Setter for batchSize.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl AsRef<AnyObject> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl AsRef<MPSKernel> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl AsRef<MPSKernel> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixSoftMaxGradient> for MPSMatrixLogSoftMaxGradient
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixSoftMaxGradient> for MPSMatrixLogSoftMaxGradient
MPSMatrix only.Sourcefn as_ref(&self) -> &MPSMatrixSoftMaxGradient
fn as_ref(&self) -> &MPSMatrixSoftMaxGradient
Sourceimpl AsRef<MPSMatrixSoftMaxGradient> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl AsRef<MPSMatrixSoftMaxGradient> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl AsRef<NSObject> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl AsRef<NSObject> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl Borrow<AnyObject> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Borrow<AnyObject> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl Borrow<MPSKernel> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Borrow<MPSKernel> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<MPSMatrixSoftMaxGradient> for MPSMatrixLogSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Borrow<MPSMatrixSoftMaxGradient> for MPSMatrixLogSoftMaxGradient
MPSMatrix only.Sourcefn borrow(&self) -> &MPSMatrixSoftMaxGradient
fn borrow(&self) -> &MPSMatrixSoftMaxGradient
Sourceimpl Borrow<NSObject> for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Borrow<NSObject> for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl ClassType for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl ClassType for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceconst NAME: &'static str = "MPSMatrixSoftMaxGradient"
const NAME: &'static str = "MPSMatrixSoftMaxGradient"
Sourcetype Super = MPSMatrixBinaryKernel
type Super = MPSMatrixBinaryKernel
Sourcetype ThreadKind = <<MPSMatrixSoftMaxGradient as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixSoftMaxGradient as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl CopyingHelper for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourcetype Result = MPSMatrixSoftMaxGradient
type Result = MPSMatrixSoftMaxGradient
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Debug for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl Deref for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Deref for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl Hash for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Hash for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl Message for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl Message for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl NSCoding for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl NSCoding for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl NSCopying for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl NSCopying for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl NSObjectProtocol for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl NSObjectProtocol for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl NSSecureCoding for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl PartialEq for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl PartialEq for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceimpl RefEncode for MPSMatrixSoftMaxGradient
Available on crate feature MPSMatrix only.
impl RefEncode for MPSMatrixSoftMaxGradient
MPSMatrix only.Sourceconst ENCODING_REF: Encoding = <MPSMatrixBinaryKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSMatrixBinaryKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixSoftMaxGradient
MPSMatrix only.impl Eq for MPSMatrixSoftMaxGradient
MPSMatrix only.