pub struct MPSNDArrayBinaryKernel { /* private fields */ }MPSCore and MPSKernel and MPSNDArray and MPSNDArrayKernel only.Expand description
Implementations
Sourceimpl MPSNDArrayBinaryKernel
impl MPSNDArrayBinaryKernel
Sourcepub unsafe fn primaryOffsets(&self) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn primaryOffsets(&self) -> MPSNDArrayOffsets
MPSNDArrayTypes only.The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension. Default: 0,0,0…
Sourcepub unsafe fn primaryEdgeMode(&self) -> MPSImageEdgeMode
👎DeprecatedAvailable on crate feature MPSCoreTypes only.
pub unsafe fn primaryEdgeMode(&self) -> MPSImageEdgeMode
MPSCoreTypes only.The edge mode used for a source NDArray Default: MPSImageEdgeModeZero
Sourcepub unsafe fn primaryKernelSizes(&self) -> MPSNDArraySizes
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn primaryKernelSizes(&self) -> MPSNDArraySizes
MPSNDArrayTypes only.The diameters of the point spread function in each dimension for a source NDArray Default: 1
Sourcepub unsafe fn primaryStrides(&self) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn primaryStrides(&self) -> MPSNDArrayOffsets
MPSNDArrayTypes only.If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result. Default: 1
Sourcepub unsafe fn primaryDilationRates(&self) -> MPSNDArraySizes
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn primaryDilationRates(&self) -> MPSNDArraySizes
MPSNDArrayTypes only.The stride in each dimension from one PSF tap to an adjacent PSF tap. Default: 1
Sourcepub unsafe fn secondaryOffsets(&self) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn secondaryOffsets(&self) -> MPSNDArrayOffsets
MPSNDArrayTypes only.The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension. Default: 0,0,0…
Sourcepub unsafe fn secondaryEdgeMode(&self) -> MPSImageEdgeMode
👎DeprecatedAvailable on crate feature MPSCoreTypes only.
pub unsafe fn secondaryEdgeMode(&self) -> MPSImageEdgeMode
MPSCoreTypes only.The edge mode used for a source NDArray Default: MPSImageEdgeModeZero
Sourcepub unsafe fn secondaryKernelSizes(&self) -> MPSNDArraySizes
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn secondaryKernelSizes(&self) -> MPSNDArraySizes
MPSNDArrayTypes only.The diameters of the point spread function in each dimension for a source NDArray Default: 1
Sourcepub unsafe fn secondaryStrides(&self) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn secondaryStrides(&self) -> MPSNDArrayOffsets
MPSNDArrayTypes only.If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result. Default: 1
Sourcepub unsafe fn secondaryDilationRates(&self) -> MPSNDArraySizes
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn secondaryDilationRates(&self) -> MPSNDArraySizes
MPSNDArrayTypes only.The stride in each dimension from one PSF tap to an adjacent PSF tap. Default: 1
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
pub unsafe fn initWithDevice_sourceCount( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, count: NSUInteger, ) -> Retained<Self>
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
coder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
pub unsafe fn initWithCoder_device( this: Allocated<Self>, coder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
Safety
coder possibly has further requirements.
Sourcepub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
primary_source_array: &MPSNDArray,
secondary_source_array: &MPSNDArray,
) -> Retained<MPSNDArray>
pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, ) -> Retained<MPSNDArray>
Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter primarySourceArray: The primary source for the filter in an NSArray.
Parameter secondarySourceArray: The secondary source for the filter in an NSArray.
Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_destinationArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
primary_source_array: &MPSNDArray,
secondary_source_array: &MPSNDArray,
destination: &MPSNDArray,
)
pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, destination: &MPSNDArray, )
Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter primarySourceArray: The primary source for the filter in an NSArray.
Parameter secondarySourceArray: The secondary source for the filter in an NSArray.
Parameter destination: The NDArray to receive the result
Sourcepub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_resultState_outputStateIsTemporary(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
primary_source_array: &MPSNDArray,
secondary_source_array: &MPSNDArray,
out_gradient_state: Option<&mut Option<Retained<MPSState>>>,
output_state_is_temporary: bool,
) -> Retained<MPSNDArray>
Available on crate feature MPSState only.
pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_resultState_outputStateIsTemporary( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, out_gradient_state: Option<&mut Option<Retained<MPSState>>>, output_state_is_temporary: bool, ) -> Retained<MPSNDArray>
MPSState only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter primarySourceArray: The primary source for the filter in an NSArray.
Parameter secondarySourceArray: The secondary source for the filter in an NSArray.
Parameter outGradientState: If non-nil, the address output gradient state is written to this address
Parameter outputStateIsTemporary: If YES, the state if any will be allocated to contain temporary textures and buffers as needed
Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_resultState_destinationArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
primary_source_array: &MPSNDArray,
secondary_source_array: &MPSNDArray,
out_gradient_state: Option<&MPSState>,
destination: &MPSNDArray,
)
Available on crate feature MPSState only.
pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_resultState_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, out_gradient_state: Option<&MPSState>, destination: &MPSNDArray, )
MPSState only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter primarySourceArray: The primary source for the filter in an NSArray.
Parameter secondarySourceArray: The secondary source for the filter in an NSArray.
Parameter outGradientState: The output gradient state to record the operation for later use by gradient
Parameter destination: A destination array to contain the result of the calculation
when the command buffer completes successfully.
Sourceimpl MPSNDArrayBinaryKernel
Methods declared on superclass MPSKernel.
impl MPSNDArrayBinaryKernel
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSNDArrayMultiaryKernel>
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
) -> Retained<MPSNDArray>
pub unsafe fn encodeToCommandBuffer_sourceArrays( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, ) -> Retained<MPSNDArray>
Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays_destinationArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
destination: &MPSNDArray,
)
pub unsafe fn encodeToCommandBuffer_sourceArrays_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, destination: &MPSNDArray, )
Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter destination: The NDArray to receive the result
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_outputStateIsTemporary(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
out_gradient_state: Option<&mut Option<Retained<MPSState>>>,
output_state_is_temporary: bool,
) -> Retained<MPSNDArray>
Available on crate feature MPSState only.
pub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_outputStateIsTemporary( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, out_gradient_state: Option<&mut Option<Retained<MPSState>>>, output_state_is_temporary: bool, ) -> Retained<MPSNDArray>
MPSState only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter outGradientState: If non-nil, the address output gradient state is written to this address
Parameter outputStateIsTemporary: If YES, the state if any will be allocated to contain temporary textures and buffers as needed
Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_destinationArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
out_gradient_state: Option<&MPSState>,
destination: &MPSNDArray,
)
Available on crate feature MPSState only.
pub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, out_gradient_state: Option<&MPSState>, destination: &MPSNDArray, )
MPSState only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter outGradientState: The output gradient state to record the operation for later use by gradient
Parameter destination: A destination array to contain the result of the calculation
when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandEncoder_commandBuffer_sourceArrays_destinationArray(
&self,
encoder: Option<&ProtocolObject<dyn MTLComputeCommandEncoder>>,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
destination: &MPSNDArray,
)
pub unsafe fn encodeToCommandEncoder_commandBuffer_sourceArrays_destinationArray( &self, encoder: Option<&ProtocolObject<dyn MTLComputeCommandEncoder>>, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, destination: &MPSNDArray, )
Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter encoder: The MTLComputeCommandEncoder that the kernel will be encoded on
Parameter commandBuffer: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter destination: A destination array to contain the result of the calculation
when the command buffer completes successfully.
Methods from Deref<Target = MPSNDArrayMultiaryBase>
Sourcepub unsafe fn offsetsAtSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn offsetsAtSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArrayOffsets
MPSNDArrayTypes only.Read offsets to use when addressing a source NDArray
The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension.
Parameter sourceIndex: The index of the source MPSNDArray to which the list of offsets is applied
Sourcepub unsafe fn edgeModeAtSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSImageEdgeMode
👎DeprecatedAvailable on crate feature MPSCoreTypes only.
pub unsafe fn edgeModeAtSourceIndex( &self, source_index: NSUInteger, ) -> MPSImageEdgeMode
MPSCoreTypes only.The edge mode used for each source NDArray
Parameter sourceIndex: The index of the source image
Returns: The MPSImageEdgeMode for that image
Sourcepub unsafe fn kernelSizesForSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArraySizes
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn kernelSizesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArraySizes
MPSNDArrayTypes only.Get the diameters of the point spread function (PSF) in each dimension
Parameter sourceIndex: The MPSNDArrayMultiaryKernel source NDArray to which the kernel will be applied
Returns: A list of kernel diameters in each dimension
Sourcepub unsafe fn stridesForSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn stridesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArrayOffsets
MPSNDArrayTypes only.Return the downsampling ratio for the kernel in each dimension
If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result.
Parameter sourceIndex: The index of the source for which the strides apply
Returns: The strides from one destination sample to the next in each dimension of the corresponding source NDArray
Sourcepub unsafe fn dilationRatesForSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArraySizes
👎DeprecatedAvailable on crate feature MPSNDArrayTypes only.
pub unsafe fn dilationRatesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArraySizes
MPSNDArrayTypes only.Get the kernel dilation rate for each dimension
Parameter sourceIndex: The index of the source image for which this applies
Returns: The kernel dilation rate for each dimension.
Sourcepub unsafe fn destinationArrayAllocator(
&self,
) -> Retained<ProtocolObject<dyn MPSNDArrayAllocator>>
pub unsafe fn destinationArrayAllocator( &self, ) -> Retained<ProtocolObject<dyn MPSNDArrayAllocator>>
Method to allocate the result image for -encodeToCommandBuffer:sourceImage:
Default: MPSTemporaryImage.defaultAllocator
Sourcepub unsafe fn setDestinationArrayAllocator(
&self,
destination_array_allocator: &ProtocolObject<dyn MPSNDArrayAllocator>,
)
pub unsafe fn setDestinationArrayAllocator( &self, destination_array_allocator: &ProtocolObject<dyn MPSNDArrayAllocator>, )
Setter for destinationArrayAllocator.
Sourcepub unsafe fn encodeWithCoder(&self, coder: &NSCoder)
pub unsafe fn encodeWithCoder(&self, coder: &NSCoder)
Initialize a MPSNDArrayMultiaryKernel from a NSCoder
Parameter coder: The NSCoder that contains the serialized object
Safety
coder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Create a copy with
Parameter zone: The NSZone in which to allocate the MPSNDArrayMultiaryKernel object
Parameter device: The device on which the new kernel will run. Pass nil for same device.
Returns: A valid MPSNDArrayMultiaryKernel, or nil if allocation failure.
Safety
zone must be a valid pointer or null.
pub unsafe fn resultStateForSourceArrays_sourceStates_destinationArray( &self, source_arrays: &NSArray<MPSNDArray>, source_states: Option<&NSArray<MPSState>>, destination_array: &MPSNDArray, ) -> Option<Retained<MPSState>>
MPSState only.Sourcepub unsafe fn destinationArrayDescriptorForSourceArrays_sourceState(
&self,
sources: &NSArray<MPSNDArray>,
state: Option<&MPSState>,
) -> Retained<MPSNDArrayDescriptor>
Available on crate feature MPSState only.
pub unsafe fn destinationArrayDescriptorForSourceArrays_sourceState( &self, sources: &NSArray<MPSNDArray>, state: Option<&MPSState>, ) -> Retained<MPSNDArrayDescriptor>
MPSState only.Return a descriptor suitable for allocating a NSArray to receive the result
The object properties (kernelSize, offsets, edgeMode, etc.) should be properly configured as if the -encode call was about to be made, before this method is called. Those properties may affect the results.
Parameter sources: The list of sources passed into the -encode call
Parameter state: The source state object, if any passed to the -encode call
Returns: a valid MPSNDArrayDescriptor that may be used to create a MPSNDArray to used to hold the results of this kernel.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
this is difficult to use correctly, use Ivar::load instead.
Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSNDArrayBinaryKernel
impl AsRef<AnyObject> for MPSNDArrayBinaryKernel
Sourceimpl AsRef<MPSKernel> for MPSNDArrayBinaryKernel
impl AsRef<MPSKernel> for MPSNDArrayBinaryKernel
Sourceimpl AsRef<MPSNDArrayBinaryKernel> for MPSNDArrayGather
Available on crate feature MPSNDArrayGather only.
impl AsRef<MPSNDArrayBinaryKernel> for MPSNDArrayGather
MPSNDArrayGather only.Sourcefn as_ref(&self) -> &MPSNDArrayBinaryKernel
fn as_ref(&self) -> &MPSNDArrayBinaryKernel
Sourceimpl AsRef<MPSNDArrayMultiaryBase> for MPSNDArrayBinaryKernel
impl AsRef<MPSNDArrayMultiaryBase> for MPSNDArrayBinaryKernel
Sourcefn as_ref(&self) -> &MPSNDArrayMultiaryBase
fn as_ref(&self) -> &MPSNDArrayMultiaryBase
Sourceimpl AsRef<MPSNDArrayMultiaryKernel> for MPSNDArrayBinaryKernel
impl AsRef<MPSNDArrayMultiaryKernel> for MPSNDArrayBinaryKernel
Sourcefn as_ref(&self) -> &MPSNDArrayMultiaryKernel
fn as_ref(&self) -> &MPSNDArrayMultiaryKernel
Sourceimpl AsRef<NSObject> for MPSNDArrayBinaryKernel
impl AsRef<NSObject> for MPSNDArrayBinaryKernel
Sourceimpl Borrow<AnyObject> for MPSNDArrayBinaryKernel
impl Borrow<AnyObject> for MPSNDArrayBinaryKernel
Sourceimpl Borrow<MPSKernel> for MPSNDArrayBinaryKernel
impl Borrow<MPSKernel> for MPSNDArrayBinaryKernel
Sourceimpl Borrow<MPSNDArrayBinaryKernel> for MPSNDArrayGather
Available on crate feature MPSNDArrayGather only.
impl Borrow<MPSNDArrayBinaryKernel> for MPSNDArrayGather
MPSNDArrayGather only.Sourcefn borrow(&self) -> &MPSNDArrayBinaryKernel
fn borrow(&self) -> &MPSNDArrayBinaryKernel
Sourceimpl Borrow<MPSNDArrayMultiaryBase> for MPSNDArrayBinaryKernel
impl Borrow<MPSNDArrayMultiaryBase> for MPSNDArrayBinaryKernel
Sourcefn borrow(&self) -> &MPSNDArrayMultiaryBase
fn borrow(&self) -> &MPSNDArrayMultiaryBase
Sourceimpl Borrow<MPSNDArrayMultiaryKernel> for MPSNDArrayBinaryKernel
impl Borrow<MPSNDArrayMultiaryKernel> for MPSNDArrayBinaryKernel
Sourcefn borrow(&self) -> &MPSNDArrayMultiaryKernel
fn borrow(&self) -> &MPSNDArrayMultiaryKernel
Sourceimpl Borrow<NSObject> for MPSNDArrayBinaryKernel
impl Borrow<NSObject> for MPSNDArrayBinaryKernel
Sourceimpl ClassType for MPSNDArrayBinaryKernel
impl ClassType for MPSNDArrayBinaryKernel
Sourceconst NAME: &'static str = "MPSNDArrayBinaryKernel"
const NAME: &'static str = "MPSNDArrayBinaryKernel"
Sourcetype Super = MPSNDArrayMultiaryKernel
type Super = MPSNDArrayMultiaryKernel
Sourcetype ThreadKind = <<MPSNDArrayBinaryKernel as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSNDArrayBinaryKernel as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSNDArrayBinaryKernel
impl CopyingHelper for MPSNDArrayBinaryKernel
Sourcetype Result = MPSNDArrayBinaryKernel
type Result = MPSNDArrayBinaryKernel
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSNDArrayBinaryKernel
impl Debug for MPSNDArrayBinaryKernel
Sourceimpl Deref for MPSNDArrayBinaryKernel
impl Deref for MPSNDArrayBinaryKernel
Sourceimpl Hash for MPSNDArrayBinaryKernel
impl Hash for MPSNDArrayBinaryKernel
Sourceimpl Message for MPSNDArrayBinaryKernel
impl Message for MPSNDArrayBinaryKernel
Sourceimpl NSCoding for MPSNDArrayBinaryKernel
impl NSCoding for MPSNDArrayBinaryKernel
Sourceunsafe fn encodeWithCoder(&self, coder: &NSCoder)
unsafe fn encodeWithCoder(&self, coder: &NSCoder)
NSCoder only.Sourceimpl NSCopying for MPSNDArrayBinaryKernel
impl NSCopying for MPSNDArrayBinaryKernel
Sourceimpl NSObjectProtocol for MPSNDArrayBinaryKernel
impl NSObjectProtocol for MPSNDArrayBinaryKernel
Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
use isKindOfClass directly, or cast your objects with AnyObject::downcast_ref