MPSNDArrayGather

Struct MPSNDArrayGather 

pub struct MPSNDArrayGather { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSNDArrayGather and MPSNDArrayKernel only.
Dependencies: This depends on Metal.framework.

Applies a gather operation along a given axis. The encoded primary source array contains the data and the secondary array is a 1-D MPSNDArray containing the indices.

For each dimension other than axis result[i] = source[i]; 0 < = i < array slice length along dimension Along the specified axis result[i] = source[indices[i]]; 0 < = i < number of indices

See also Apple’s documentation

Implementations

Source

impl MPSNDArrayGather

pub unsafe fn axis(&self) -> NSUInteger

Available on crate feature MPSNDArray only.

The axis along which to apply the gather operation. Defaults to zero.

pub unsafe fn setAxis(&self, axis: NSUInteger)

Available on crate feature MPSNDArray only.

Setter for axis.

Source

impl MPSNDArrayGather

Methods declared on superclass MPSNDArrayBinaryKernel.

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSNDArray only.
pub unsafe fn initWithDevice_sourceCount( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, count: NSUInteger, ) -> Retained<Self>

Available on crate feature MPSNDArray only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, coder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSNDArray only.
Safety

coder possibly has further requirements.

Source

impl MPSNDArrayGather

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSNDArray only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSNDArrayGather

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNDArray only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNDArray only.

Methods from Deref<Target = MPSNDArrayBinaryKernel>

pub unsafe fn primaryOffsets(&self) -> MPSNDArrayOffsets

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension. Default: 0,0,0…

pub unsafe fn primaryEdgeMode(&self) -> MPSImageEdgeMode

👎Deprecated
Available on crate features MPSNDArray and MPSCoreTypes only.

The edge mode used for a source NDArray Default: MPSImageEdgeModeZero

pub unsafe fn primaryKernelSizes(&self) -> MPSNDArraySizes

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

The diameters of the point spread function in each dimension for a source NDArray Default: 1

pub unsafe fn primaryStrides(&self) -> MPSNDArrayOffsets

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result. Default: 1

pub unsafe fn primaryDilationRates(&self) -> MPSNDArraySizes

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

The stride in each dimension from one PSF tap to an adjacent PSF tap. Default: 1

pub unsafe fn secondaryOffsets(&self) -> MPSNDArrayOffsets

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension. Default: 0,0,0…

pub unsafe fn secondaryEdgeMode(&self) -> MPSImageEdgeMode

👎Deprecated
Available on crate features MPSNDArray and MPSCoreTypes only.

The edge mode used for a source NDArray Default: MPSImageEdgeModeZero

pub unsafe fn secondaryKernelSizes(&self) -> MPSNDArraySizes

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

The diameters of the point spread function in each dimension for a source NDArray Default: 1

pub unsafe fn secondaryStrides(&self) -> MPSNDArrayOffsets

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result. Default: 1

pub unsafe fn secondaryDilationRates(&self) -> MPSNDArraySizes

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

The stride in each dimension from one PSF tap to an adjacent PSF tap. Default: 1

pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, ) -> Retained<MPSNDArray>

Available on crate feature MPSNDArray only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter primarySourceArray: The primary source for the filter in an NSArray.

Parameter secondarySourceArray: The secondary source for the filter in an NSArray.

Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.

pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, destination: &MPSNDArray, )

Available on crate feature MPSNDArray only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter primarySourceArray: The primary source for the filter in an NSArray.

Parameter secondarySourceArray: The secondary source for the filter in an NSArray.

Parameter destination: The NDArray to receive the result

pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_resultState_outputStateIsTemporary( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, out_gradient_state: Option<&mut Option<Retained<MPSState>>>, output_state_is_temporary: bool, ) -> Retained<MPSNDArray>

Available on crate features MPSNDArray and MPSState only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter primarySourceArray: The primary source for the filter in an NSArray.

Parameter secondarySourceArray: The secondary source for the filter in an NSArray.

Parameter outGradientState: If non-nil, the address output gradient state is written to this address

Parameter outputStateIsTemporary: If YES, the state if any will be allocated to contain temporary textures and buffers as needed

Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.

pub unsafe fn encodeToCommandBuffer_primarySourceArray_secondarySourceArray_resultState_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, primary_source_array: &MPSNDArray, secondary_source_array: &MPSNDArray, out_gradient_state: Option<&MPSState>, destination: &MPSNDArray, )

Available on crate features MPSNDArray and MPSState only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter primarySourceArray: The primary source for the filter in an NSArray.

Parameter secondarySourceArray: The secondary source for the filter in an NSArray.

Parameter outGradientState: The output gradient state to record the operation for later use by gradient

Parameter destination: A destination array to contain the result of the calculation when the command buffer completes successfully.

Methods from Deref<Target = MPSNDArrayMultiaryKernel>

pub unsafe fn encodeToCommandBuffer_sourceArrays( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, ) -> Retained<MPSNDArray>

Available on crate feature MPSNDArray only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter sourceArrays: The list of sources for the filter in a NSArray. Ordering to be defined by subclass

Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.

pub unsafe fn encodeToCommandBuffer_sourceArrays_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, destination: &MPSNDArray, )

Available on crate feature MPSNDArray only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter sourceArrays: The list of sources for the filter in a NSArray. Ordering to be defined by subclass

Parameter destination: The NDArray to receive the result

pub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_outputStateIsTemporary( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, out_gradient_state: Option<&mut Option<Retained<MPSState>>>, output_state_is_temporary: bool, ) -> Retained<MPSNDArray>

Available on crate features MPSNDArray and MPSState only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter sourceArrays: The list of sources for the filter in a NSArray. Ordering to be defined by subclass

Parameter outGradientState: If non-nil, the address output gradient state is written to this address

Parameter outputStateIsTemporary: If YES, the state if any will be allocated to contain temporary textures and buffers as needed

Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.

pub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, out_gradient_state: Option<&MPSState>, destination: &MPSNDArray, )

Available on crate features MPSNDArray and MPSState only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter cmdBuf: The command buffer into which to encode the kernel

Parameter sourceArrays: The list of sources for the filter in a NSArray. Ordering to be defined by subclass

Parameter outGradientState: The output gradient state to record the operation for later use by gradient

Parameter destination: A destination array to contain the result of the calculation when the command buffer completes successfully.

pub unsafe fn encodeToCommandEncoder_commandBuffer_sourceArrays_destinationArray( &self, encoder: Option<&ProtocolObject<dyn MTLComputeCommandEncoder>>, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, destination: &MPSNDArray, )

Available on crate feature MPSNDArray only.

Encode a simple inference NDArray kernel and return a NDArray to hold the result

Parameter encoder: The MTLComputeCommandEncoder that the kernel will be encoded on

Parameter commandBuffer: The command buffer into which to encode the kernel

Parameter sourceArrays: The list of sources for the filter in a NSArray. Ordering to be defined by subclass

Parameter destination: A destination array to contain the result of the calculation when the command buffer completes successfully.

Methods from Deref<Target = MPSNDArrayMultiaryBase>

pub unsafe fn offsetsAtSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArrayOffsets

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

Read offsets to use when addressing a source NDArray

The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension.

Parameter sourceIndex: The index of the source MPSNDArray to which the list of offsets is applied

pub unsafe fn edgeModeAtSourceIndex( &self, source_index: NSUInteger, ) -> MPSImageEdgeMode

👎Deprecated
Available on crate features MPSNDArray and MPSCoreTypes only.

The edge mode used for each source NDArray

Parameter sourceIndex: The index of the source image

Returns: The MPSImageEdgeMode for that image

pub unsafe fn kernelSizesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArraySizes

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

Get the diameters of the point spread function (PSF) in each dimension

Parameter sourceIndex: The MPSNDArrayMultiaryKernel source NDArray to which the kernel will be applied

Returns: A list of kernel diameters in each dimension

pub unsafe fn stridesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArrayOffsets

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

Return the downsampling ratio for the kernel in each dimension

If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result.

Parameter sourceIndex: The index of the source for which the strides apply

Returns: The strides from one destination sample to the next in each dimension of the corresponding source NDArray

pub unsafe fn dilationRatesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArraySizes

👎Deprecated
Available on crate features MPSNDArray and MPSNDArrayTypes only.

Get the kernel dilation rate for each dimension

Parameter sourceIndex: The index of the source image for which this applies

Returns: The kernel dilation rate for each dimension.

pub unsafe fn destinationArrayAllocator( &self, ) -> Retained<ProtocolObject<dyn MPSNDArrayAllocator>>

Available on crate feature MPSNDArray only.

Method to allocate the result image for -encodeToCommandBuffer:sourceImage:

Default: MPSTemporaryImage.defaultAllocator

pub unsafe fn setDestinationArrayAllocator( &self, destination_array_allocator: &ProtocolObject<dyn MPSNDArrayAllocator>, )

Available on crate feature MPSNDArray only.
pub unsafe fn encodeWithCoder(&self, coder: &NSCoder)

Available on crate feature MPSNDArray only.

Initialize a MPSNDArrayMultiaryKernel from a NSCoder

Parameter coder: The NSCoder that contains the serialized object

Safety

coder possibly has further requirements.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Available on crate feature MPSNDArray only.

Create a copy with

Parameter zone: The NSZone in which to allocate the MPSNDArrayMultiaryKernel object

Parameter device: The device on which the new kernel will run. Pass nil for same device.

Returns: A valid MPSNDArrayMultiaryKernel, or nil if allocation failure.

Safety

zone must be a valid pointer or null.

pub unsafe fn resultStateForSourceArrays_sourceStates_destinationArray( &self, source_arrays: &NSArray<MPSNDArray>, source_states: Option<&NSArray<MPSState>>, destination_array: &MPSNDArray, ) -> Option<Retained<MPSState>>

Available on crate features MPSNDArray and MPSState only.
pub unsafe fn destinationArrayDescriptorForSourceArrays_sourceState( &self, sources: &NSArray<MPSNDArray>, state: Option<&MPSState>, ) -> Retained<MPSNDArrayDescriptor>

Available on crate features MPSNDArray and MPSState only.

Return a descriptor suitable for allocating a NSArray to receive the result

The object properties (kernelSize, offsets, edgeMode, etc.) should be properly configured as if the -encode call was about to be made, before this method is called. Those properties may affect the results.

Parameter sources: The list of sources passed into the -encode call

Parameter state: The source state object, if any passed to the -encode call

Returns: a valid MPSNDArrayDescriptor that may be used to create a MPSNDArray to used to hold the results of this kernel.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}