pub struct MPSNDArrayLUTQuantizationDescriptor { /* private fields */ }MPSNDArrayQuantization only.Expand description
Dependencies: This depends on Metal.framework.
Describes a lookup-table based quantization scheme
See also Apple’s documentation
Implementations
Sourceimpl MPSNDArrayLUTQuantizationDescriptor
impl MPSNDArrayLUTQuantizationDescriptor
Sourcepub unsafe fn initWithDataType(
this: Allocated<Self>,
quantization_data_type: MPSDataType,
) -> Retained<Self>
Available on crate features MPSNDArray and MPSCore and MPSCoreTypes only.
pub unsafe fn initWithDataType( this: Allocated<Self>, quantization_data_type: MPSDataType, ) -> Retained<Self>
MPSNDArray and MPSCore and MPSCoreTypes only.Initializes a scalar lookup-table quantization descriptor.
Parameter quantizationDataType: Which quantized datatype is used.
Returns: A new quantization descriptor.
Sourcepub unsafe fn initWithDataType_vectorAxis(
this: Allocated<Self>,
quantization_data_type: MPSDataType,
vector_axis: NSUInteger,
) -> Retained<Self>
Available on crate features MPSNDArray and MPSCore and MPSCoreTypes only.
pub unsafe fn initWithDataType_vectorAxis( this: Allocated<Self>, quantization_data_type: MPSDataType, vector_axis: NSUInteger, ) -> Retained<Self>
MPSNDArray and MPSCore and MPSCoreTypes only.Initializes a vector lookup-table quantization descriptor.
Parameter quantizationDataType: Which quantized datatype is used.
Parameter vectorAxis: The quantization vector axis - this axis will receive the vector component in the destination.
Returns: A new quantization descriptor.
Methods from Deref<Target = MPSNDArrayQuantizationDescriptor>
Sourcepub unsafe fn quantizationDataType(&self) -> MPSDataType
Available on crate features MPSNDArray and MPSCore and MPSCoreTypes only.
pub unsafe fn quantizationDataType(&self) -> MPSDataType
MPSNDArray and MPSCore and MPSCoreTypes only.The datatype to use with quantization - the default is MPSDataTypeUint8
Sourcepub unsafe fn quantizationScheme(&self) -> MPSNDArrayQuantizationScheme
Available on crate feature MPSNDArray only.
pub unsafe fn quantizationScheme(&self) -> MPSNDArrayQuantizationScheme
MPSNDArray only.The quantization scheme for this descriptor. The default is MPSNDArrayQuantizationTypeNone.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl AsRef<AnyObject> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl AsRef<MPSNDArrayLUTQuantizationDescriptor> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl AsRef<MPSNDArrayLUTQuantizationDescriptor> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl AsRef<MPSNDArrayQuantizationDescriptor> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl AsRef<MPSNDArrayQuantizationDescriptor> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourcefn as_ref(&self) -> &MPSNDArrayQuantizationDescriptor
fn as_ref(&self) -> &MPSNDArrayQuantizationDescriptor
Sourceimpl AsRef<NSObject> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl AsRef<NSObject> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl Borrow<AnyObject> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Borrow<AnyObject> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl Borrow<MPSNDArrayQuantizationDescriptor> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Borrow<MPSNDArrayQuantizationDescriptor> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourcefn borrow(&self) -> &MPSNDArrayQuantizationDescriptor
fn borrow(&self) -> &MPSNDArrayQuantizationDescriptor
Sourceimpl Borrow<NSObject> for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Borrow<NSObject> for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl ClassType for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl ClassType for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceconst NAME: &'static str = "MPSNDArrayLUTQuantizationDescriptor"
const NAME: &'static str = "MPSNDArrayLUTQuantizationDescriptor"
Sourcetype Super = MPSNDArrayQuantizationDescriptor
type Super = MPSNDArrayQuantizationDescriptor
Sourcetype ThreadKind = <<MPSNDArrayLUTQuantizationDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSNDArrayLUTQuantizationDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl CopyingHelper for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourcetype Result = MPSNDArrayLUTQuantizationDescriptor
type Result = MPSNDArrayLUTQuantizationDescriptor
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Debug for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl Deref for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Deref for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl Hash for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Hash for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl Message for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl Message for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl NSCopying for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl NSCopying for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl NSObjectProtocol for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl NSObjectProtocol for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl PartialEq for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceimpl RefEncode for MPSNDArrayLUTQuantizationDescriptor
Available on crate feature MPSNDArray only.
impl RefEncode for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.Sourceconst ENCODING_REF: Encoding = <MPSNDArrayQuantizationDescriptor as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSNDArrayQuantizationDescriptor as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.impl Eq for MPSNDArrayLUTQuantizationDescriptor
MPSNDArray only.