pub struct MPSNDArrayQuantizedMatrixMultiplication { /* private fields */ }MPSCore and MPSKernel and MPSNDArrayKernel and MPSNDArrayMatrixMultiplication and MPSNDArrayQuantizedMatrixMultiplication only.Expand description
Dependencies: This depends on Metal.framework.
A quantized matrix multiplication kernel: C = AB, where each input A and B can be quantized.
The kernel works with 2-8 inputs, order of inputs: First all LHS inputs, then all RHS inputs.
The order of inputs for LUT based LHS or RHS: 1) quantized input 2) Lookup Table.
The order of inputs for affine LHS or RHS: 1) quantized input 2) scale 3) zeropoint 4) minValue.
The full order of inputs for the encode methods is:
[LHS, RHS, <LHS quantization inputs>, <RHS quantization inputs>],
where LHS is the left input (quantized or float) RHS is the right input (quantized or float) and
<LHS quantization inputs> are the auxiliary quantization inputs for the LHS array (scales, zeropoints etc).
and <RHS quantization inputs> are the auxiliary quantization input for the RHS array.
The inputs are provided as a compacted NSArray <MPSNDArray *>, for example for computing
C = A * B^T where A is quantized with a LUT and B is quantized with affine quantization that
uses scale and minValue the array of inputs is:
[ Aq, Bq^T, ALUT, BScale^T, BMin^T ].
NOTE: For affine scale, zeropoint and minValue must have same transposes as quantized input.
See also Apple’s documentation
Implementations
Sourceimpl MPSNDArrayQuantizedMatrixMultiplication
impl MPSNDArrayQuantizedMatrixMultiplication
pub unsafe fn initWithDevice_sourceCount( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, source_count: NSUInteger, ) -> Retained<Self>
MPSNDArray only.Sourcepub unsafe fn initWithDevice_leftQuantizationDescriptor_rightQuantizationDescriptor(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
left_quantization_descriptor: Option<&MPSNDArrayQuantizationDescriptor>,
right_quantization_descriptor: Option<&MPSNDArrayQuantizationDescriptor>,
) -> Retained<Self>
Available on crate features MPSNDArray and MPSNDArrayQuantization only.
pub unsafe fn initWithDevice_leftQuantizationDescriptor_rightQuantizationDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, left_quantization_descriptor: Option<&MPSNDArrayQuantizationDescriptor>, right_quantization_descriptor: Option<&MPSNDArrayQuantizationDescriptor>, ) -> Retained<Self>
MPSNDArray and MPSNDArrayQuantization only.Initializes a quantized matrix multiplication kernel.
Parameter leftQuantizationDescriptor: The quantization definition for the LHS input.
Parameter rightQuantizationDescriptor: The quantization definition for the RHS input.
Returns: A new valid quantized matrix multiplication kernel.
Sourceimpl MPSNDArrayQuantizedMatrixMultiplication
Methods declared on superclass MPSNDArrayMultiaryKernel.
impl MPSNDArrayQuantizedMatrixMultiplication
Methods declared on superclass MPSNDArrayMultiaryKernel.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
coder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Retained<Self>
Available on crate feature MPSNDArray only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, coder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSNDArray only.Safety
coder possibly has further requirements.
Sourceimpl MPSNDArrayQuantizedMatrixMultiplication
Methods declared on superclass MPSNDArrayMultiaryBase.
impl MPSNDArrayQuantizedMatrixMultiplication
Methods declared on superclass MPSNDArrayMultiaryBase.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSNDArray only.Sourceimpl MPSNDArrayQuantizedMatrixMultiplication
Methods declared on superclass MPSKernel.
impl MPSNDArrayQuantizedMatrixMultiplication
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSNDArray only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSNDArray only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSNDArrayMatrixMultiplication>
Sourcepub unsafe fn alpha(&self) -> c_double
Available on crate feature MPSNDArray only.
pub unsafe fn alpha(&self) -> c_double
MPSNDArray only.The scale factor to apply to the product. Specified in double precision. Will be converted to the appropriate precision in the implementation subject to rounding and/or clamping as necessary. Defaults to 1.0 at initialization time.
Sourcepub unsafe fn setAlpha(&self, alpha: c_double)
Available on crate feature MPSNDArray only.
pub unsafe fn setAlpha(&self, alpha: c_double)
MPSNDArray only.Setter for alpha.
Sourcepub unsafe fn beta(&self) -> c_double
Available on crate feature MPSNDArray only.
pub unsafe fn beta(&self) -> c_double
MPSNDArray only.The scale factor to apply to the addend if available. Specified in double precision. Will be converted to the appropriate precision in the implementation subject to rounding and/or clamping as necessary. Defaults to 1.0 at initialization time.
Methods from Deref<Target = MPSNDArrayMultiaryKernel>
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
) -> Retained<MPSNDArray>
Available on crate feature MPSNDArray only.
pub unsafe fn encodeToCommandBuffer_sourceArrays( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, ) -> Retained<MPSNDArray>
MPSNDArray only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays_destinationArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
destination: &MPSNDArray,
)
Available on crate feature MPSNDArray only.
pub unsafe fn encodeToCommandBuffer_sourceArrays_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, destination: &MPSNDArray, )
MPSNDArray only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter destination: The NDArray to receive the result
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_outputStateIsTemporary(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
out_gradient_state: Option<&mut Option<Retained<MPSState>>>,
output_state_is_temporary: bool,
) -> Retained<MPSNDArray>
Available on crate features MPSNDArray and MPSState only.
pub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_outputStateIsTemporary( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, out_gradient_state: Option<&mut Option<Retained<MPSState>>>, output_state_is_temporary: bool, ) -> Retained<MPSNDArray>
MPSNDArray and MPSState only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter outGradientState: If non-nil, the address output gradient state is written to this address
Parameter outputStateIsTemporary: If YES, the state if any will be allocated to contain temporary textures and buffers as needed
Returns: A newly allocated MPSNDArray that will contain the result of the calculation when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_destinationArray(
&self,
cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
out_gradient_state: Option<&MPSState>,
destination: &MPSNDArray,
)
Available on crate features MPSNDArray and MPSState only.
pub unsafe fn encodeToCommandBuffer_sourceArrays_resultState_destinationArray( &self, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, out_gradient_state: Option<&MPSState>, destination: &MPSNDArray, )
MPSNDArray and MPSState only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter cmdBuf: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter outGradientState: The output gradient state to record the operation for later use by gradient
Parameter destination: A destination array to contain the result of the calculation
when the command buffer completes successfully.
Sourcepub unsafe fn encodeToCommandEncoder_commandBuffer_sourceArrays_destinationArray(
&self,
encoder: Option<&ProtocolObject<dyn MTLComputeCommandEncoder>>,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source_arrays: &NSArray<MPSNDArray>,
destination: &MPSNDArray,
)
Available on crate feature MPSNDArray only.
pub unsafe fn encodeToCommandEncoder_commandBuffer_sourceArrays_destinationArray( &self, encoder: Option<&ProtocolObject<dyn MTLComputeCommandEncoder>>, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_arrays: &NSArray<MPSNDArray>, destination: &MPSNDArray, )
MPSNDArray only.Encode a simple inference NDArray kernel and return a NDArray to hold the result
Parameter encoder: The MTLComputeCommandEncoder that the kernel will be encoded on
Parameter commandBuffer: The command buffer into which to encode the kernel
Parameter sourceArrays: The list of sources for the filter in a NSArray.
Ordering to be defined by subclass
Parameter destination: A destination array to contain the result of the calculation
when the command buffer completes successfully.
Methods from Deref<Target = MPSNDArrayMultiaryBase>
Sourcepub unsafe fn offsetsAtSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate features MPSNDArray and MPSNDArrayTypes only.
pub unsafe fn offsetsAtSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArrayOffsets
MPSNDArray and MPSNDArrayTypes only.Read offsets to use when addressing a source NDArray
The coordinate of the position read from this source array which is used to calculate the result value at [0,0,0,….] If the position read is actually a contiguous region (e.g. the area covered by a convolution kernel) then this is the center of that region, rounded down, for each dimension.
Parameter sourceIndex: The index of the source MPSNDArray to which the list of offsets is applied
Sourcepub unsafe fn edgeModeAtSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSImageEdgeMode
👎DeprecatedAvailable on crate features MPSNDArray and MPSCoreTypes only.
pub unsafe fn edgeModeAtSourceIndex( &self, source_index: NSUInteger, ) -> MPSImageEdgeMode
MPSNDArray and MPSCoreTypes only.The edge mode used for each source NDArray
Parameter sourceIndex: The index of the source image
Returns: The MPSImageEdgeMode for that image
Sourcepub unsafe fn kernelSizesForSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArraySizes
👎DeprecatedAvailable on crate features MPSNDArray and MPSNDArrayTypes only.
pub unsafe fn kernelSizesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArraySizes
MPSNDArray and MPSNDArrayTypes only.Get the diameters of the point spread function (PSF) in each dimension
Parameter sourceIndex: The MPSNDArrayMultiaryKernel source NDArray to which the kernel will be applied
Returns: A list of kernel diameters in each dimension
Sourcepub unsafe fn stridesForSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArrayOffsets
👎DeprecatedAvailable on crate features MPSNDArray and MPSNDArrayTypes only.
pub unsafe fn stridesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArrayOffsets
MPSNDArray and MPSNDArrayTypes only.Return the downsampling ratio for the kernel in each dimension
If the filter is a “backwards” filter such as a gradient filter or convolution transpose, then this is the upsampling ratio and zeros are inserted in the result.
Parameter sourceIndex: The index of the source for which the strides apply
Returns: The strides from one destination sample to the next in each dimension of the corresponding source NDArray
Sourcepub unsafe fn dilationRatesForSourceIndex(
&self,
source_index: NSUInteger,
) -> MPSNDArraySizes
👎DeprecatedAvailable on crate features MPSNDArray and MPSNDArrayTypes only.
pub unsafe fn dilationRatesForSourceIndex( &self, source_index: NSUInteger, ) -> MPSNDArraySizes
MPSNDArray and MPSNDArrayTypes only.Get the kernel dilation rate for each dimension
Parameter sourceIndex: The index of the source image for which this applies
Returns: The kernel dilation rate for each dimension.
Sourcepub unsafe fn destinationArrayAllocator(
&self,
) -> Retained<ProtocolObject<dyn MPSNDArrayAllocator>>
Available on crate feature MPSNDArray only.
pub unsafe fn destinationArrayAllocator( &self, ) -> Retained<ProtocolObject<dyn MPSNDArrayAllocator>>
MPSNDArray only.Method to allocate the result image for -encodeToCommandBuffer:sourceImage:
Default: MPSTemporaryImage.defaultAllocator
Sourcepub unsafe fn setDestinationArrayAllocator(
&self,
destination_array_allocator: &ProtocolObject<dyn MPSNDArrayAllocator>,
)
Available on crate feature MPSNDArray only.
pub unsafe fn setDestinationArrayAllocator( &self, destination_array_allocator: &ProtocolObject<dyn MPSNDArrayAllocator>, )
MPSNDArray only.Setter for destinationArrayAllocator.
Sourcepub unsafe fn encodeWithCoder(&self, coder: &NSCoder)
Available on crate feature MPSNDArray only.
pub unsafe fn encodeWithCoder(&self, coder: &NSCoder)
MPSNDArray only.Initialize a MPSNDArrayMultiaryKernel from a NSCoder
Parameter coder: The NSCoder that contains the serialized object
Safety
coder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
Available on crate feature MPSNDArray only.
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
MPSNDArray only.Create a copy with
Parameter zone: The NSZone in which to allocate the MPSNDArrayMultiaryKernel object
Parameter device: The device on which the new kernel will run. Pass nil for same device.
Returns: A valid MPSNDArrayMultiaryKernel, or nil if allocation failure.
Safety
zone must be a valid pointer or null.
pub unsafe fn resultStateForSourceArrays_sourceStates_destinationArray( &self, source_arrays: &NSArray<MPSNDArray>, source_states: Option<&NSArray<MPSState>>, destination_array: &MPSNDArray, ) -> Option<Retained<MPSState>>
MPSNDArray and MPSState only.Sourcepub unsafe fn destinationArrayDescriptorForSourceArrays_sourceState(
&self,
sources: &NSArray<MPSNDArray>,
state: Option<&MPSState>,
) -> Retained<MPSNDArrayDescriptor>
Available on crate features MPSNDArray and MPSState only.
pub unsafe fn destinationArrayDescriptorForSourceArrays_sourceState( &self, sources: &NSArray<MPSNDArray>, state: Option<&MPSState>, ) -> Retained<MPSNDArrayDescriptor>
MPSNDArray and MPSState only.Return a descriptor suitable for allocating a NSArray to receive the result
The object properties (kernelSize, offsets, edgeMode, etc.) should be properly configured as if the -encode call was about to be made, before this method is called. Those properties may affect the results.
Parameter sources: The list of sources passed into the -encode call
Parameter state: The source state object, if any passed to the -encode call
Returns: a valid MPSNDArrayDescriptor that may be used to create a MPSNDArray to used to hold the results of this kernel.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<AnyObject> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl AsRef<MPSKernel> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<MPSKernel> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl AsRef<MPSNDArrayMatrixMultiplication> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<MPSNDArrayMatrixMultiplication> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn as_ref(&self) -> &MPSNDArrayMatrixMultiplication
fn as_ref(&self) -> &MPSNDArrayMatrixMultiplication
Sourceimpl AsRef<MPSNDArrayMultiaryBase> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<MPSNDArrayMultiaryBase> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn as_ref(&self) -> &MPSNDArrayMultiaryBase
fn as_ref(&self) -> &MPSNDArrayMultiaryBase
Sourceimpl AsRef<MPSNDArrayMultiaryKernel> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<MPSNDArrayMultiaryKernel> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn as_ref(&self) -> &MPSNDArrayMultiaryKernel
fn as_ref(&self) -> &MPSNDArrayMultiaryKernel
Sourceimpl AsRef<MPSNDArrayQuantizedMatrixMultiplication> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<MPSNDArrayQuantizedMatrixMultiplication> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl AsRef<NSObject> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl AsRef<NSObject> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl Borrow<AnyObject> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Borrow<AnyObject> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl Borrow<MPSKernel> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Borrow<MPSKernel> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl Borrow<MPSNDArrayMatrixMultiplication> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Borrow<MPSNDArrayMatrixMultiplication> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn borrow(&self) -> &MPSNDArrayMatrixMultiplication
fn borrow(&self) -> &MPSNDArrayMatrixMultiplication
Sourceimpl Borrow<MPSNDArrayMultiaryBase> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Borrow<MPSNDArrayMultiaryBase> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn borrow(&self) -> &MPSNDArrayMultiaryBase
fn borrow(&self) -> &MPSNDArrayMultiaryBase
Sourceimpl Borrow<MPSNDArrayMultiaryKernel> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Borrow<MPSNDArrayMultiaryKernel> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn borrow(&self) -> &MPSNDArrayMultiaryKernel
fn borrow(&self) -> &MPSNDArrayMultiaryKernel
Sourceimpl Borrow<NSObject> for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Borrow<NSObject> for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl ClassType for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl ClassType for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceconst NAME: &'static str = "MPSNDArrayQuantizedMatrixMultiplication"
const NAME: &'static str = "MPSNDArrayQuantizedMatrixMultiplication"
Sourcetype Super = MPSNDArrayMatrixMultiplication
type Super = MPSNDArrayMatrixMultiplication
Sourcetype ThreadKind = <<MPSNDArrayQuantizedMatrixMultiplication as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSNDArrayQuantizedMatrixMultiplication as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl CopyingHelper for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcetype Result = MPSNDArrayQuantizedMatrixMultiplication
type Result = MPSNDArrayQuantizedMatrixMultiplication
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Debug for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl Deref for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Deref for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl Hash for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Hash for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl Message for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl Message for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl NSCoding for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl NSCoding for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl NSCopying for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl NSCopying for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl NSObjectProtocol for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl NSObjectProtocol for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl NSSecureCoding for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl PartialEq for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl PartialEq for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceimpl RefEncode for MPSNDArrayQuantizedMatrixMultiplication
Available on crate feature MPSNDArray only.
impl RefEncode for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.Sourceconst ENCODING_REF: Encoding = <MPSNDArrayMatrixMultiplication as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSNDArrayMatrixMultiplication as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.impl Eq for MPSNDArrayQuantizedMatrixMultiplication
MPSNDArray only.