pub struct MPSNNGradientStateNode { /* private fields */ }MPSNNGraphNodes only.Expand description
During training, each MPSNNFilterNode has a corresponding MPSNNGradientFilterNode for the gradient computation for trainable parameter update. The two communicate through a MPSNNGradientStateNode or subclass which carries information about the inference pass settings to the gradient pass. You can avoid managing these – there will be many! – by using -[MPSNNFilterNode gradientFilterWithSources:] to make the MPSNNGradientFilterNodes. That method will append the necessary extra information like MPSNNGradientState nodes and inference filter source image nodes to the object as needed.
See also Apple’s documentation
Implementations
Sourceimpl MPSNNGradientStateNode
Methods declared on superclass MPSNNStateNode.
impl MPSNNGradientStateNode
Methods declared on superclass MPSNNStateNode.
Methods from Deref<Target = MPSNNStateNode>
Sourcepub unsafe fn handle(&self) -> Option<Retained<ProtocolObject<dyn MPSHandle>>>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn handle(&self) -> Option<Retained<ProtocolObject<dyn MPSHandle>>>
MPSNeuralNetwork only.MPS resource identification
See MPSHandle protocol reference. Default: nil
Sourcepub unsafe fn setHandle(&self, handle: Option<&ProtocolObject<dyn MPSHandle>>)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setHandle(&self, handle: Option<&ProtocolObject<dyn MPSHandle>>)
MPSNeuralNetwork only.Setter for handle.
Sourcepub unsafe fn exportFromGraph(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn exportFromGraph(&self) -> bool
MPSNeuralNetwork only.Tag a state node for view later
Most state nodes are private to the graph. These alias memory heavily and consequently generally have invalid state when the graph exits. When exportFromGraph = YES, the image is preserved and made available through the [MPSNNGraph encode… resultStates:… list.
CAUTION: exporting an state from a graph prevents MPS from recycling memory. It will nearly always cause the amount of memory used by the graph to increase by the size of the state. There will probably be a performance regression accordingly. This feature should generally be used only when the node is needed as an input for further work and recomputing it is prohibitively costly.
Default: NO
Sourcepub unsafe fn setExportFromGraph(&self, export_from_graph: bool)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setExportFromGraph(&self, export_from_graph: bool)
MPSNeuralNetwork only.Setter for exportFromGraph.
Sourcepub unsafe fn synchronizeResource(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn synchronizeResource(&self) -> bool
MPSNeuralNetwork only.Set to true to cause the resource to be synchronized with the CPU
Ignored on non-MacOS.
Sourcepub unsafe fn setSynchronizeResource(&self, synchronize_resource: bool)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSynchronizeResource(&self, synchronize_resource: bool)
MPSNeuralNetwork only.Setter for synchronizeResource.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<AnyObject> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl AsRef<MPSNNGradientStateNode> for MPSCNNConvolutionGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSNNGradientStateNode> for MPSCNNConvolutionGradientStateNode
MPSNeuralNetwork only.Sourcefn as_ref(&self) -> &MPSNNGradientStateNode
fn as_ref(&self) -> &MPSNNGradientStateNode
Sourceimpl AsRef<MPSNNGradientStateNode> for MPSCNNConvolutionTransposeGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSNNGradientStateNode> for MPSCNNConvolutionTransposeGradientStateNode
MPSNeuralNetwork only.Sourcefn as_ref(&self) -> &MPSNNGradientStateNode
fn as_ref(&self) -> &MPSNNGradientStateNode
Sourceimpl AsRef<MPSNNGradientStateNode> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSNNGradientStateNode> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl AsRef<MPSNNStateNode> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSNNStateNode> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourcefn as_ref(&self) -> &MPSNNStateNode
fn as_ref(&self) -> &MPSNNStateNode
Sourceimpl AsRef<NSObject> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<NSObject> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl Borrow<AnyObject> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<AnyObject> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl Borrow<MPSNNGradientStateNode> for MPSCNNConvolutionGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSNNGradientStateNode> for MPSCNNConvolutionGradientStateNode
MPSNeuralNetwork only.Sourcefn borrow(&self) -> &MPSNNGradientStateNode
fn borrow(&self) -> &MPSNNGradientStateNode
Sourceimpl Borrow<MPSNNGradientStateNode> for MPSCNNConvolutionTransposeGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSNNGradientStateNode> for MPSCNNConvolutionTransposeGradientStateNode
MPSNeuralNetwork only.Sourcefn borrow(&self) -> &MPSNNGradientStateNode
fn borrow(&self) -> &MPSNNGradientStateNode
Sourceimpl Borrow<MPSNNStateNode> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSNNStateNode> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourcefn borrow(&self) -> &MPSNNStateNode
fn borrow(&self) -> &MPSNNStateNode
Sourceimpl Borrow<NSObject> for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<NSObject> for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl ClassType for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl ClassType for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceconst NAME: &'static str = "MPSNNGradientStateNode"
const NAME: &'static str = "MPSNNGradientStateNode"
Sourcetype Super = MPSNNStateNode
type Super = MPSNNStateNode
Sourcetype ThreadKind = <<MPSNNGradientStateNode as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSNNGradientStateNode as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl Debug for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Debug for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl Deref for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Deref for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl Hash for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Hash for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl Message for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl Message for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl NSObjectProtocol for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl NSObjectProtocol for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl PartialEq for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceimpl RefEncode for MPSNNGradientStateNode
Available on crate feature MPSNeuralNetwork only.
impl RefEncode for MPSNNGradientStateNode
MPSNeuralNetwork only.Sourceconst ENCODING_REF: Encoding = <MPSNNStateNode as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSNNStateNode as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSNNGradientStateNode
MPSNeuralNetwork only.impl Eq for MPSNNGradientStateNode
MPSNeuralNetwork only.