pub struct MPSNNImageNode { /* private fields */ }MPSNNGraphNodes only.Expand description
A placeholder node denoting the position of a MPSImage in a graph
MPS neural network graphs are made up of filter nodes connected by image (or state) nodes. An image node is produced by one filter but may be consumed by more than one filter.
Most image nodes will be created by MPS and made available through MPSNNFilterNode.resultImage. Image nodes that are not created by MPS (i.e. “the graph inputs”) must be created by you.
See also Apple’s documentation
Implementations
Sourceimpl MPSNNImageNode
impl MPSNNImageNode
Sourcepub unsafe fn initWithHandle(
this: Allocated<Self>,
handle: Option<&NSObject>,
) -> Retained<Self>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithHandle( this: Allocated<Self>, handle: Option<&NSObject>, ) -> Retained<Self>
MPSNeuralNetwork only.Safety
handle must implement MPSHandle.
Sourcepub unsafe fn nodeWithHandle(handle: Option<&NSObject>) -> Retained<Self>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn nodeWithHandle(handle: Option<&NSObject>) -> Retained<Self>
MPSNeuralNetwork only.Safety
handle must implement MPSHandle.
Sourcepub unsafe fn exportedNodeWithHandle(
handle: Option<&NSObject>,
) -> Retained<Self>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn exportedNodeWithHandle( handle: Option<&NSObject>, ) -> Retained<Self>
MPSNeuralNetwork only.Create a autoreleased MPSNNImageNode with exportFromGraph = YES.
Note: image is still temporary. See MPSNNImageNode.imageAllocator parameter.
Safety
handle must implement MPSHandle.
pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>
MPSNeuralNetwork only.Sourcepub unsafe fn handle(&self) -> Option<Retained<ProtocolObject<dyn MPSHandle>>>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn handle(&self) -> Option<Retained<ProtocolObject<dyn MPSHandle>>>
MPSNeuralNetwork only.MPS resource identifier
See MPSHandle protocol description. Default: nil
Sourcepub unsafe fn setHandle(&self, handle: Option<&ProtocolObject<dyn MPSHandle>>)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setHandle(&self, handle: Option<&ProtocolObject<dyn MPSHandle>>)
MPSNeuralNetwork only.Setter for handle.
Sourcepub unsafe fn format(&self) -> MPSImageFeatureChannelFormat
Available on crate features MPSNeuralNetwork and MPSCore and MPSCoreTypes only.
pub unsafe fn format(&self) -> MPSImageFeatureChannelFormat
MPSNeuralNetwork and MPSCore and MPSCoreTypes only.The preferred precision for the image
Default: MPSImageFeatureChannelFormatNone, meaning MPS should pick a format Typically, this is 16-bit floating-point.
Sourcepub unsafe fn setFormat(&self, format: MPSImageFeatureChannelFormat)
Available on crate features MPSNeuralNetwork and MPSCore and MPSCoreTypes only.
pub unsafe fn setFormat(&self, format: MPSImageFeatureChannelFormat)
MPSNeuralNetwork and MPSCore and MPSCoreTypes only.Setter for format.
Sourcepub unsafe fn imageAllocator(
&self,
) -> Retained<ProtocolObject<dyn MPSImageAllocator>>
Available on crate features MPSNeuralNetwork and MPSCore and MPSImage only.
pub unsafe fn imageAllocator( &self, ) -> Retained<ProtocolObject<dyn MPSImageAllocator>>
MPSNeuralNetwork and MPSCore and MPSImage only.Configurability for image allocation
Allows you to influence how the image is allocated Default: MPSTemporaryImage.defaultAllocator
Sourcepub unsafe fn setImageAllocator(
&self,
image_allocator: &ProtocolObject<dyn MPSImageAllocator>,
)
Available on crate features MPSNeuralNetwork and MPSCore and MPSImage only.
pub unsafe fn setImageAllocator( &self, image_allocator: &ProtocolObject<dyn MPSImageAllocator>, )
MPSNeuralNetwork and MPSCore and MPSImage only.Setter for imageAllocator.
Sourcepub unsafe fn exportFromGraph(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn exportFromGraph(&self) -> bool
MPSNeuralNetwork only.Tag a image node for view later
Most image nodes are private to the graph. These alias memory heavily and consequently generally have invalid state when the graph exits. When exportFromGraph = YES, the image is preserved and made available through the [MPSNNGraph encode… intermediateImages:… list.
CAUTION: exporting an image from a graph prevents MPS from recycling memory. It will nearly always cause the amount of memory used by the graph to increase by the size of the image. There will probably be a performance regression accordingly. This feature should generally be used only when the node is needed as an input for further work and recomputing it is prohibitively costly.
Default: NO
Sourcepub unsafe fn setExportFromGraph(&self, export_from_graph: bool)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setExportFromGraph(&self, export_from_graph: bool)
MPSNeuralNetwork only.Setter for exportFromGraph.
Sourcepub unsafe fn synchronizeResource(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn synchronizeResource(&self) -> bool
MPSNeuralNetwork only.Set to true to cause the resource to be synchronized with the CPU
It is not needed on iOS/tvOS devices, where it does nothing.
Sourcepub unsafe fn setSynchronizeResource(&self, synchronize_resource: bool)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSynchronizeResource(&self, synchronize_resource: bool)
MPSNeuralNetwork only.Setter for synchronizeResource.
Sourcepub unsafe fn stopGradient(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn stopGradient(&self) -> bool
MPSNeuralNetwork only.Stop training graph automatic creation at this node.
An inference graph of MPSNNFilterNodes, MPSNNStateNodes and MPSNNImageNodes can be automatically
converted to a training graph using -[MPSNNFilterNode trainingGraphWithSourceGradient:nodeHandler:].
Sometimes, an inference graph may contain extra nodes at start to do operations like resampling or range
adjustment that should not be part of the training graph. To prevent gradient operations for these extra
nodes from being included in the training graph, set
Sourcepub unsafe fn setStopGradient(&self, stop_gradient: bool)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setStopGradient(&self, stop_gradient: bool)
MPSNeuralNetwork only.Setter for stopGradient.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<AnyObject> for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl AsRef<MPSNNImageNode> for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSNNImageNode> for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl AsRef<NSObject> for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl AsRef<NSObject> for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl Borrow<AnyObject> for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<AnyObject> for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl Borrow<NSObject> for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl Borrow<NSObject> for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl ClassType for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl ClassType for MPSNNImageNode
MPSNeuralNetwork only.Sourceconst NAME: &'static str = "MPSNNImageNode"
const NAME: &'static str = "MPSNNImageNode"
Sourcetype ThreadKind = <<MPSNNImageNode as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSNNImageNode as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl Debug for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl Debug for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl Deref for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl Deref for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl Hash for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl Hash for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl Message for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl Message for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl NSObjectProtocol for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl NSObjectProtocol for MPSNNImageNode
MPSNeuralNetwork only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl PartialEq for MPSNNImageNode
MPSNeuralNetwork only.Sourceimpl RefEncode for MPSNNImageNode
Available on crate feature MPSNeuralNetwork only.
impl RefEncode for MPSNNImageNode
MPSNeuralNetwork only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSNNImageNode
MPSNeuralNetwork only.impl Eq for MPSNNImageNode
MPSNeuralNetwork only.