MPSPolygonBuffer

Struct MPSPolygonBuffer 

pub struct MPSPolygonBuffer { /* private fields */ }
๐Ÿ‘ŽDeprecated
Available on crate feature MPSPolygonBuffer only.
A vertex buffer and optional index and mask buffer for a set of polygons

See also Appleโ€™s documentation

Implementations

Source

impl MPSPolygonBuffer

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Initialize the polygon buffer

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Initialize the polygon buffer with an NSCoder. Buffer properties such as the vertex buffer, instance buffer, etc. are set to nil. Encode and decode these buffers along with the polygon buffer instead.

Safety

a_decoder possibly has further requirements.

pub unsafe fn polygonBuffer() -> Retained<Self>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.
pub unsafe fn copyWithZone(&self, zone: *mut NSZone) -> Retained<Self>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Create a a copy of this polygon buffer

Buffer properties of the polygon buffer such as the vertex buffer, instance, buffer, etc. are set to nil. Copy these buffers and assign them to the new polygon buffer or reassign the existing buffers to the new polygon buffer.

Parameter zone: This parameter is ignored. Memory zones are no longer used by Objective-C.

Safety

zone must be a valid pointer or null.

pub unsafe fn vertexBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Vertex buffer containing vertex data encoded as three 32 bit floats per vertex. Note that by default each vertex is aligned to the alignment of the vector_float3 type: 16 bytes. This can be changed using the vertexStride property. A vertex buffer must be provided before the acceleration structure is built.

When using triangle polygons, degenerate (zero or negative area) triangles are ignored during acceleration structure construction. This can be used to pad triangle indices if needed.

Quadrilateral polygons are internally treated as two triangles. If the quadrilateral has vertices v0, v1, v2, and v3, the two triangles will have vertices v0, v1, v2 and v0, v2, v3. A quadrilateral may be used to represent a triangle by repeating the last vertex. If the first triangle is degenerate (zero or negative area), the entire quadrilateral will be ignored. This can be used to pad quadrilateral indices if needed. All four vertices of a quadrilateral must be coplanar and the quadrilateral must be convex.

pub unsafe fn setVertexBuffer( &self, vertex_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for vertexBuffer.

Safety
  • vertex_buffer may need to be synchronized.
  • vertex_buffer may be unretained, you must ensure it is kept alive while in use.
  • vertex_buffer contents should be of the correct type.
pub unsafe fn vertexBufferOffset(&self) -> NSUInteger

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the vertex buffer. Defaults to 0 bytes. Must be aligned to 4 bytes.

pub unsafe fn setVertexBufferOffset(&self, vertex_buffer_offset: NSUInteger)

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for vertexBufferOffset.

pub unsafe fn indexBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Index buffer containing index data. Each index references a vertex in the vertex buffer. May be nil.

pub unsafe fn setIndexBuffer( &self, index_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for indexBuffer.

Safety
  • index_buffer may need to be synchronized.
  • index_buffer may be unretained, you must ensure it is kept alive while in use.
  • index_buffer contents should be of the correct type.
pub unsafe fn indexBufferOffset(&self) -> NSUInteger

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the index buffer. Defaults to 0 bytes. Must be aligned to a multiple of the index type. Changes to this property require rebuilding the acceleration structure.

pub unsafe fn setIndexBufferOffset(&self, index_buffer_offset: NSUInteger)

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for indexBufferOffset.

pub unsafe fn maskBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Mask buffer containing one uint32_t mask per polygon. May be nil. Otherwise, the mask type must be specified on the MPSRayIntersector with which it is used.

pub unsafe fn setMaskBuffer( &self, mask_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for maskBuffer.

Safety
  • mask_buffer may need to be synchronized.
  • mask_buffer may be unretained, you must ensure it is kept alive while in use.
  • mask_buffer contents should be of the correct type.
pub unsafe fn maskBufferOffset(&self) -> NSUInteger

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the mask buffer. Defaults to 0 bytes. Must be aligned to 4 bytes.

pub unsafe fn setMaskBufferOffset(&self, mask_buffer_offset: NSUInteger)

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for maskBufferOffset.

pub unsafe fn polygonCount(&self) -> NSUInteger

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Number of polygons. Changes to this property require rebuilding the acceleration structure.

pub unsafe fn setPolygonCount(&self, polygon_count: NSUInteger)

๐Ÿ‘ŽDeprecated
Available on crate feature MPSRayIntersector only.

Setter for polygonCount.

Source

impl MPSPolygonBuffer

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSRayIntersector only.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesnโ€™t recognize.

See Appleโ€™s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

๐Ÿ‘ŽDeprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable โ€œunder the hoodโ€.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Appleโ€™s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called โ€œlightweight genericsโ€, and thatโ€™s because they arenโ€™t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you donโ€™t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

โ“˜
use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}