MPSRNNMatrixInferenceLayer

Struct MPSRNNMatrixInferenceLayer 

pub struct MPSRNNMatrixInferenceLayer { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSRNNLayer only.
Dependencies: This depends on Metal.framework

The MPSRNNMatrixInferenceLayer specifies a recurrent neural network layer for inference on MPSMatrices. Currently two types of recurrent layers are supported: ones that operate with convolutions on images: MPSRNNImageInferenceLayerand one that operates on matrices: MPSRNNMatrixInferenceLayer.The former can be often used to implement the latter by using 1x1-matrices, but due to image size restrictions and performance, it is advisable to use MPSRNNMatrixInferenceLayerfor linear recurrent layers. A MPSRNNMatrixInferenceLayer is initialized using a MPSRNNLayerDescriptor,which further specifies the recurrent network layer, or an array of MPSRNNLayerDescriptors,which specifies a stack of recurrent layers, that can operate in parallel a subset of the inputs in a sequence of inputs and recurrent outputs. Note that currently stacks with bidirectionally traversing encode functions do not support starting from a previous set of recurrent states, but this can be achieved quite easily by defining two separate unidirectional stacks of layers, and running the same input sequence on them separately (one forwards and one backwards) and ultimately combining the two result sequences as desired with auxiliary functions. The input and output vectors in encode calls are stored as rows of the input and output matrices and MPSRNNMatrixInferenceLayer supports matrices with decreasing number of rows: The row-indices identify the different sequences that may be of different lengths - for example if we have three sequences: ( x1, x2, x3 ), ( y1, y2, y3, y4 ) and ( z1, z2 ) of vectors xi, yi and zi, then these can be inserted together as a batch to the sequence encoding kernel by using the matrices:

                           ( y1 )        ( y2 )        ( y3 )        ( y4 )
                      m1 = ( x1 ),  m2 = ( x2 ),  m3 = ( x3 ),  m4 =
                           ( z1 )        ( z2 )

If a recurrent output state is requested then it will contain the state corresponding to last inputs to each sequence and if all the intermediate states are requested (see storeAllIntermediateStates), then the shorter sequences will be propagated by copying the state of the previous output if the input vector is not present in the sequence - in the example above the output states would be:

                           ( s_y1 )        ( s_y2 )        ( s_y3 )        ( s_y4 )
                      s1 = ( s_x1 ),  s2 = ( s_x2 ),  s3 = ( s_x3 ),  s4 = ( s_x3 )
                           ( s_z1 )        ( s_z2 )        ( s_z2 )        ( s_z2 )

The mathematical operation described in the linear transformations of MPSRNNSingleGateDescriptorMPSLSTMDescriptorand MPSGRUDescriptorare y^T = W x^T < => y = x W^T, where x is the matrix containing the input vectors as rows, y is the matrix containing the output vectors as rows and W is the weight matrix.

See also Apple’s documentation

Implementations

Source

impl MPSRNNMatrixInferenceLayer

pub unsafe fn inputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of feature channels input vector/matrix.

pub unsafe fn outputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of feature channels in the output vector/matrix.

pub unsafe fn numberOfLayers(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

Number of layers in the filter-stack. This will be one when using initWithDevice:rnnDescriptor to initialize this filter and the number of entries in the array ‘rnnDescriptors’ when initializing this filter with initWithDevice:rnnDescriptors.

pub unsafe fn recurrentOutputIsTemporary(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

How output states from encodeSequenceToCommandBufferare constructed. Defaults to NO. For reference

See: MPSState.

pub unsafe fn setRecurrentOutputIsTemporary( &self, recurrent_output_is_temporary: bool, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn storeAllIntermediateStates(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

If YES then calls to encodeSequenceToCommandBufferreturn every recurrent state in the array: recurrentOutputStates. Defaults to NO.

pub unsafe fn setStoreAllIntermediateStates( &self, store_all_intermediate_states: bool, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn bidirectionalCombineMode(&self) -> MPSRNNBidirectionalCombineMode

Available on crate feature MPSNeuralNetwork only.

Defines how to combine the output-results, when encoding bidirectional layers using encodeBidirectionalSequenceToCommandBuffer.Defaults to MPSRNNBidirectionalCombineModeNone.

pub unsafe fn setBidirectionalCombineMode( &self, bidirectional_combine_mode: MPSRNNBidirectionalCombineMode, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithDevice_rnnDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, rnn_descriptor: &MPSRNNDescriptor, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Initializes a linear (fully connected) RNN kernel

Parameter device: The MTLDevice on which this MPSRNNMatrixLayer filter will be used

Parameter rnnDescriptor: The descriptor that defines the RNN layer

Returns: A valid MPSRNNMatrixInferenceLayer object or nil, if failure.

pub unsafe fn initWithDevice_rnnDescriptors( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, rnn_descriptors: &NSArray<MPSRNNDescriptor>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Initializes a kernel that implements a stack of linear (fully connected) RNN layers

Parameter device: The MTLDevice on which this MPSRNNMatrixLayer filter will be used

Parameter rnnDescriptors: An array of RNN descriptors that defines a stack of RNN layers, starting at index zero. The number of layers in stack is the number of entries in the array. All entries in the array must be valid MPSRNNDescriptors.

Returns: A valid MPSRNNMatrixInferenceLayer object or nil, if failure.

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeSequenceToCommandBuffer_sourceMatrices_sourceOffsets_destinationMatrices_destinationOffsets_recurrentInputState_recurrentOutputStates( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_matrices: &NSArray<MPSMatrix>, source_offsets: *mut NSUInteger, destination_matrices: &NSArray<MPSMatrix>, destination_offsets: *mut NSUInteger, recurrent_input_state: Option<&MPSRNNRecurrentMatrixState>, recurrent_output_states: Option<&NSMutableArray<MPSRNNRecurrentMatrixState>>, )

Available on crate features MPSNeuralNetwork and MPSMatrix and MPSState only.

Encode an MPSRNNMatrixInferenceLayer kernel (stack) for a sequence of inputs into a command buffer. Note that when encoding using this function the

See: layerSequenceDirection is ignored and the layer stack operates as if all layers were forward feeding layers. In order to run bidirectional sequences use encodeBidirectionalSequenceToCommandBuffer:sourceSequence:or alternatively run two layer stacks and combine results at the end using utility functions.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter sourceMatrices: An array of valid MPSMatrix objects containing the sequence of source matrices.

Parameter sourceOffsets: An array of byte-offsets into the sourceMatrices, if nil zeros are assumed and if not nil must contain offset for every matrix in sourceMatrices.

Parameter destinationMatrices: An array valid MPSMatrices to be overwritten by result matrix sequence. destinationMatrices may not alias sourceMatrices.

Parameter destinationOffsets: An array of byte-offsets into the destinationMatrices, if nil zeros are assumed and if not nil must contain offset for every matrix in destinationMatrices.

Parameter recurrentInputState: An optional state containing the output matrices and memory cells (for LSTMs) of the layer obtained from the previous input matrices in a sequence of inputs. Has to be the output of a previous call to this function or nil (assumed zero). Note: can be one of the states returned in intermediateRecurrentStates. Parameter recurrentOutputStates: An optional array that will contain the recurrent output states. If nil then the recurrent output state is discarded. If storeAllIntermediateStatesis YES, then all intermediate states of the sequence are returned in the array, the first one corresponding to the first input in the sequence, otherwise only the last recurrent output state is returned. If recurrentOutputIsTemporary is YES and then all returned recurrent states will be temporary.

See: MPSState:isTemporary. Example: In order to get a new state one can do the following:

                                                      MPSRNNRecurrentMatrixState* recurrent0 = nil;
                                                      [filter encodeToCommandBuffer: cmdBuf
                                                                       sourceMatrix: source0
                                                                  destinationMatrix: destination0
                                                                recurrentInputState: nil
                                                               recurrentOutputState: &recurrent0];

Then use it for the next input in sequence:

                                                      [filter encodeToCommandBuffer: cmdBuf
                                                                       sourceMatrix: source1
                                                                  destinationMatrix: destination1
                                                                recurrentInputState: recurrent0
                                                               recurrentOutputState: &recurrent0];

And discard recurrent output of the third input:

                                                      [filter encodeToCommandBuffer: cmdBuf
                                                                       sourceMatrix: source2
                                                                  destinationMatrix: destination2
                                                                recurrentInputState: recurrent0
                                                               recurrentOutputState: nil];
Safety
  • source_offsets must be a valid pointer or null.
  • destination_offsets must be a valid pointer or null.
pub unsafe fn encodeSequenceToCommandBuffer_sourceMatrices_destinationMatrices_recurrentInputState_recurrentOutputStates( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_matrices: &NSArray<MPSMatrix>, destination_matrices: &NSArray<MPSMatrix>, recurrent_input_state: Option<&MPSRNNRecurrentMatrixState>, recurrent_output_states: Option<&NSMutableArray<MPSRNNRecurrentMatrixState>>, )

Available on crate features MPSNeuralNetwork and MPSMatrix and MPSState only.
pub unsafe fn encodeBidirectionalSequenceToCommandBuffer_sourceSequence_destinationForwardMatrices_destinationBackwardMatrices( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_sequence: &NSArray<MPSMatrix>, destination_forward_matrices: &NSArray<MPSMatrix>, destination_backward_matrices: Option<&NSArray<MPSMatrix>>, )

Available on crate features MPSNeuralNetwork and MPSMatrix only.

Encode an MPSRNNMatrixInferenceLayer kernel stack for an input matrix sequences into a command buffer bidirectionally. The operation proceeds as follows: The first source matrix x0 is passed through all forward traversing layers in the stack, ie. those that were initialized with MPSRNNSequenceDirectionForward, recurrent input is assumed zero. This produces forward output yf0 and recurrent states hf00, hf01, hf02, … hf0n, one for each forward layer in the stack. Then x1 is passed to forward layers together with recurrent state hf00, hf01, …, hf0n, which produces yf1, and hf10,… This procedure is iterated until the last matrix in the input sequence x_(N-1), which produces forward output yf(N-1). The backwards layers iterate the same sequence backwards, starting from input x_(N-1) (recurrent state zero), that produces yb(N-1) and recurrent output hb(N-1)0, hf(N-1)1, … hb(N-1)m, one for each backwards traversing layer. Then the backwards layers handle input x_(N-2) using recurrent state hb(N-1)0, …, et cetera, until the first matrix of the sequence is computed, producing output yb0. The result of the operation is either pair of sequences ({yf0, yf1, … , yf(N-1)}, {yb0, yb1, … , yb(N-1)}) or a combined sequence, {(yf0 + yb0), … , (yf(N-1) + yb(N-1)) }, where ‘+’ stands either for sum, or concatenation along feature channels, as specified by bidirectionalCombineMode.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter sourceSequence: An array of valid MPSMatrix objects containing the source matrix sequence (x0, x1, … x_n-1).

Parameter destinationForwardMatrices: An array of valid MPSMatrices to be overwritten by result from forward input matrices. If bidirectionalCombineMode is either MPSRNNBidirectionalCombineModeAdd or MPSRNNBidirectionalCombineModeConcatenate, then will contain the combined results. destinationForwardMatrix may not alias with any of the source matrices.

Parameter destinationBackwardMatrices: If bidirectionalCombineMode is MPSRNNBidirectionalCombineModeNone, then must be an array of valid MPSMatrices that will be overwritten by result from backward input matrices. Otherwise this parameter is ignored and can be nil. destinationBackwardMatrices may not alias to any of the source matrices.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

NSSecureCoding compatability

See MPSKernel#initWithCoder. Parameter aDecoder: The NSCoder subclass with your serialized MPSRNNMatrixInferenceLayer

Parameter device: The MTLDevice on which to make the MPSRNNMatrixInferenceLayer

Returns: A new MPSRNNMatrixInferenceLayer object, or nil if failure.

Safety

a_decoder possibly has further requirements.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Make a copy of this kernel for a new device -

See: MPSKernel

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Source

impl MPSRNNMatrixInferenceLayer

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSRNNMatrixInferenceLayer

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}