MPSRNNMatrixTrainingLayer

Struct MPSRNNMatrixTrainingLayer 

pub struct MPSRNNMatrixTrainingLayer { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSRNNLayer only.
Dependencies: This depends on Metal.framework

The MPSRNNMatrixTrainingLayer specifies a recurrent neural network layer for training on MPSMatrices.

A MPSRNNMatrixTrainingLayer is initialized using a MPSRNNLayerDescriptor,which further specifies the recurrent network layer. The input and output vectors in encode calls are stored as rows of the input and output matrices and MPSRNNMatrixTrainingLayer supports matrices with decreasing number of rows: The row-indices identify the different sequences that may be of different lengths - for example if we have three sequences: ( x1, x2, x3 ), ( y1, y2, y3, y4 ) and ( z1, z2 ) of vectors xi, yi and zi, then these can be inserted together as a batch to the sequence encoding kernel by using the matrices:

                           ( y1 )        ( y2 )        ( y3 )        ( y4 )
                      m1 = ( x1 ),  m2 = ( x2 ),  m3 = ( x3 ),  m4 =
                           ( z1 )        ( z2 )

The gradient computation pass is then achieved by passing the corresponding gradient sequence from the previous layer ( dx1, dx2, dx3 ), ( dy1, dy2, dy3, dy4 ) and ( dz1, dz2 ) as matrices

                            ( dy1 )         ( dy2 )         ( dy3 )         ( dy4 )
                      dm1 = ( dx1 ),  dm2 = ( dx2 ),  dm3 = ( dx3 ),  dm4 =
                            ( dz1 )         ( dz2 )

The mathematical operation described in the linear transformations of MPSRNNSingleGateDescriptorMPSLSTMDescriptorand MPSGRUDescriptorare y^T = W x^T < => y = x W^T, where x is the matrix containing the input vectors as rows, y is the matrix containing the output vectors as rows and W is the weight matrix.

See also Apple’s documentation

Implementations

Source

impl MPSRNNMatrixTrainingLayer

pub unsafe fn inputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of feature channels input vector/matrix.

pub unsafe fn outputFeatureChannels(&self) -> NSUInteger

Available on crate feature MPSNeuralNetwork only.

The number of feature channels in the output vector/matrix.

pub unsafe fn storeAllIntermediateStates(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

If YES then calls to functions encodeForwardSequenceToCommandBufferand encodeGradientSequenceToCommandBufferreturn every recurrent state in the array: recurrentOutputStates. Defaults to NO.

pub unsafe fn setStoreAllIntermediateStates( &self, store_all_intermediate_states: bool, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn recurrentOutputIsTemporary(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

How recurrent output states from encodeForwardSequenceToCommandBufferand encodeGradientSequenceToCommandBuffer are constructed. Defaults to NO. For reference

See: MPSState.

pub unsafe fn setRecurrentOutputIsTemporary( &self, recurrent_output_is_temporary: bool, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn trainingStateIsTemporary(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

How training output states from encodeForwardSequenceToCommandBufferare constructed. Defaults to NO. For reference

See: MPSState.

pub unsafe fn setTrainingStateIsTemporary( &self, training_state_is_temporary: bool, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn accumulateWeightGradients(&self) -> bool

Available on crate feature MPSNeuralNetwork only.

If yes then the computed weight gradients are accumulated on top of existing values in calls to the gradient computation functions: encodeGradientSequenceToCommandBuffer. Defaults to NO.

pub unsafe fn setAccumulateWeightGradients( &self, accumulate_weight_gradients: bool, )

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithDevice_rnnDescriptor_trainableWeights( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, rnn_descriptor: &MPSRNNDescriptor, trainable_weights: &NSMutableArray<MPSMatrix>, ) -> Retained<Self>

Available on crate features MPSNeuralNetwork and MPSMatrix only.

Initializes a linear (fully connected) RNN kernel for training

Parameter device: The MTLDevice on which this MPSRNNMatrixLayer filter will be used

Parameter rnnDescriptor: The descriptor that defines the RNN layer

Parameter trainableWeights: An array where to store the weights of the layer as MPSMatrices. NOTE: The exact layout and number of matrices may vary between platforms and therefore you should not save out these weights directly, but instead use the function encodeCopyWeightsToCommandBuffer to identify the weights and biases for serialization. Typically you should pass here an initialized but empty NSMutableArray and when this function returns the array will have been populated with the weight matrices needed in the encode-calls, by using initial values from the datasources in rnnDescriptor.

Returns: A valid MPSRNNMatrixTrainingLayer object or nil, if failure.

pub unsafe fn createWeightGradientMatrices_dataType( &self, matrices_out: &NSMutableArray<MPSMatrix>, data_type: MPSDataType, )

Available on crate features MPSNeuralNetwork and MPSCoreTypes and MPSMatrix only.

Initializes a set of matrices that can be used in training for weight and bias gradient outputs in

See: encodeBackwardSequenceToCommandBuffer. Can be also used to easily create auxiliary matrices for example for ADAM and other advanced optimization schemes. The layout and number of matrices is the same as for the outputs of

See: initWithDevice, but the data type may differ. NOTE: These matrices cannot be used as weight matrices in the forward and backward encode calls, but matrices from initWithDevice() or createWeightMatrices() should be used instead.

Parameter matricesOut: An array where the newly created matrices will be stored, will be initialized to zero.

Parameter dataType: Datatype for the entries - currently MPSDataTypeFloat32 and MPSDataTypeFloat16 are supported.

pub unsafe fn createTemporaryWeightGradientMatrices_dataType_commandBuffer( &self, matrices_out: &NSMutableArray<MPSMatrix>, data_type: MPSDataType, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

Available on crate features MPSNeuralNetwork and MPSCoreTypes and MPSMatrix only.

As createWeightGradientMatrices,but the matrices will be temporary with readCount = 1, which means that they become invalid after the first encode call that reads them. Note also that as the matrices are temporary, their storage mode will be private which means that you can only access the data using a kernel on the GPU.

Parameter matricesOut: An array where the newly created matrices will be stored, will be initialized to zero.

Parameter dataType: Datatype for the entries - currently MPSDataTypeFloat32 and MPSDataTypeFloat16 are supported.

Parameter commandBuffer: The command buffer that the temporary matrices will live on.

pub unsafe fn createWeightMatrices( &self, matrices_out: &NSMutableArray<MPSMatrix>, )

Available on crate features MPSNeuralNetwork and MPSMatrix only.

Initializes a set of matrices that can be used in training for weight and bias matrices in the forward and backward passes. The layout, datatype and number of matrices is the same as for the outputs of

See: initWithDevice.

Parameter matricesOut: An array where the newly created matrices will be stored, will be initialized to zero.

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeCopyWeightsToCommandBuffer_weights_matrixId_matrix_copyFromWeightsToMatrix_matrixOffset( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, weights: &NSArray<MPSMatrix>, matrix_id: MPSRNNMatrixId, matrix: &MPSMatrix, copy_from_weights_to_matrix: bool, matrix_offset: MTLOrigin, )

Available on crate features MPSNeuralNetwork and MPSMatrix only.

Encode a copy kernel that copies one matrix from the trainable weight set to a matrix with standard layout, where the column index is the input feature channel index (in forward direction) and row index is the output feature channel index.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter weights: An array weights from

See: initWithDevice or

See: createWeightMatrices.

Parameter matrixId: Which matrix to copy - has to be a valid Id based on inputs defined in the rnnDescriptor of

See: initWithDevice.

Parameter matrix: The destination or source matrix that is used in the copy.

Parameter copyFromWeightsToMatrix: If YES then the copy direction is from the set of trainable ‘weights’ to ‘matrix’, otherwise the copy is done from ‘matrix’ to ‘weights’.

Parameter matrixOffset: A (valid) offset into matrix to be applied to the copy operation.

pub unsafe fn encodeForwardSequenceToCommandBuffer_sourceMatrices_sourceOffsets_destinationMatrices_destinationOffsets_trainingStates_recurrentInputState_recurrentOutputStates_weights( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_matrices: &NSArray<MPSMatrix>, source_offsets: *mut NSUInteger, destination_matrices: &NSArray<MPSMatrix>, destination_offsets: *mut NSUInteger, training_states: &NSMutableArray<MPSRNNMatrixTrainingState>, recurrent_input_state: Option<&MPSRNNRecurrentMatrixState>, recurrent_output_states: Option<&NSMutableArray<MPSRNNRecurrentMatrixState>>, weights: &NSArray<MPSMatrix>, )

Available on crate features MPSNeuralNetwork and MPSMatrix and MPSState only.

Encode an MPSRNNMatrixTrainingLayer forward pass kernel for a sequence of inputs into a command buffer.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter sourceMatrices: An array of valid MPSMatrix objects containing the sequence of source matrices.

Parameter sourceOffsets: An array of byte-offsets into the sourceMatrices, if nil zeros are assumed and if not nil must contain offset for every matrix in sourceMatrices.

Parameter destinationMatrices: An array valid MPSMatrices to be overwritten by result matrix sequence. destinationMatrices may not alias sourceMatrices.

Parameter destinationOffsets: An array of byte-offsets into the destinationMatrices, if nil zeros are assumed and if not nil must contain offset for every matrix in destinationMatrices.

Parameter trainingStates: An array containing the training states to be passed to the gradient computation encode function.

Parameter recurrentInputState: An optional state containing the output matrices and memory cells (for LSTMs) of the layer obtained from the previous input matrices in a sequence of inputs. Has to be the output of a previous call to this function or nil (assumed zero).

Parameter recurrentOutputStates: An array that will be appended with the recurrent output states. May not be nil. If recurrentOutputIsTemporary is YES and then all returned recurrent states will be temporary.

See: MPSState:isTemporary.

Parameter weights: An array of valid MPSMatrix objects containing the weights, should be the array that was produced either by

See: initWithDevice or

See: createWeightMatrices.

Safety
  • source_offsets must be a valid pointer or null.
  • destination_offsets must be a valid pointer or null.
pub unsafe fn encodeForwardSequenceToCommandBuffer_sourceMatrices_destinationMatrices_trainingStates_weights( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_matrices: &NSArray<MPSMatrix>, destination_matrices: &NSArray<MPSMatrix>, training_states: &NSMutableArray<MPSRNNMatrixTrainingState>, weights: &NSArray<MPSMatrix>, )

Available on crate features MPSNeuralNetwork and MPSMatrix and MPSState only.

Encode an MPSRNNMatrixTrainingLayer forward pass kernel for a sequence of inputs into a command buffer.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter sourceMatrices: An array of valid MPSMatrix objects containing the sequence of source matrices.

Parameter destinationMatrices: An array valid MPSMatrices to be overwritten by result matrix sequence. destinationMatrices may not alias sourceMatrices.

Parameter trainingStates: An array containing the training states to be passed to the gradient computation encode function.

Parameter weights: An array of valid MPSMatrix objects containing the weights, should be the array that was produced either by

See: initWithDevice or

See: createWeightMatrices.

pub unsafe fn encodeGradientSequenceToCommandBuffer_forwardSources_forwardSourceOffsets_sourceGradients_sourceGradientOffsets_destinationGradients_destinationOffsets_weightGradients_trainingStates_recurrentInputState_recurrentOutputStates_weights( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, forward_sources: &NSArray<MPSMatrix>, forward_source_offsets: *mut NSUInteger, source_gradients: &NSArray<MPSMatrix>, source_gradient_offsets: *mut NSUInteger, destination_gradients: Option<&NSArray<MPSMatrix>>, destination_offsets: *mut NSUInteger, weight_gradients: Option<&NSArray<MPSMatrix>>, training_states: &NSArray<MPSRNNMatrixTrainingState>, recurrent_input_state: Option<&MPSRNNRecurrentMatrixState>, recurrent_output_states: Option<&NSMutableArray<MPSRNNRecurrentMatrixState>>, weights: &NSArray<MPSMatrix>, )

Available on crate features MPSNeuralNetwork and MPSMatrix and MPSState only.

Encode an MPSRNNMatrixTrainingLayer gradient pass kernel for a sequence of input gradients into a command buffer. NOTE: The time sequence indexing follows the array indexing in the inputs: sourceGradients[0] has to contain the gradients corresponding to the first matrix in the forward pass corresponding to the current subsequence, which is typically sourceMatrices[0].

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter forwardSources: An array of MPSMatrix objects containing the sequence of source matrices of the forward pass.

Parameter forwardSourceOffsets: An array of byte-offsets into the forwardSources, if nil zeros are assumed and if not nil must contain offset for every matrix in forwardSources.

Parameter sourceGradients: An array of valid MPSMatrix objects containing the sequence of source gradient matrices.

Parameter sourceGradientOffsets: An array of byte-offsets into the sourceGradients, if nil zeros are assumed and if not nil must contain offset for every matrix in sourceGradients.

Parameter destinationGradients: An array valid MPSMatrix objects that will receive the backpropagated gradients, may be nil if not needed (for example first layer in graph).

Parameter destinationOffsets: An array of byte-offsets into the destinationGradients, if nil zeros are assumed and if not nil must contain offset for every matrix in destinationGradients.

Parameter weightGradients: An array of valid MPSMatrix objects that will receive the gradient wrt. weights and biases of the layer - should be the array that was produced either by

See: initWithDevice or

See: createWeightMatrices. May be nil in which case the gradients for the weights are not computed.

Parameter trainingStates: An array containing the training states from the forward pass - the array must contain the states corresponding to the input gradients is sourceGradients.

Parameter recurrentInputState: An optional state containing the output matrices and memory cells (for LSTMs) of the layer obtained from the previous input gradients in a sequence of inputs. Has to be the output of a previous call to this function or nil (assumed zero).

Parameter recurrentOutputStates: An array that will be appended with the recurrent output states. Can be nil. If recurrentOutputIsTemporary is YES and then all returned recurrent states will be temporary.

See: MPSState:isTemporary.

Parameter weights: An array of valid MPSMatrix objects containing the weights, should be the array that was produced either by

See: initWithDevice or

See: createWeightMatrices.

Safety
  • forward_source_offsets must be a valid pointer or null.
  • source_gradient_offsets must be a valid pointer or null.
  • destination_offsets must be a valid pointer or null.
pub unsafe fn encodeGradientSequenceToCommandBuffer_forwardSources_sourceGradients_destinationGradients_weightGradients_trainingStates_weights( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, forward_sources: &NSArray<MPSMatrix>, source_gradients: &NSArray<MPSMatrix>, destination_gradients: Option<&NSArray<MPSMatrix>>, weight_gradients: Option<&NSArray<MPSMatrix>>, training_states: &NSArray<MPSRNNMatrixTrainingState>, weights: &NSArray<MPSMatrix>, )

Available on crate features MPSNeuralNetwork and MPSMatrix and MPSState only.

Encode an MPSRNNMatrixTrainingLayer gradient pass kernel for a sequence of input gradients into a command buffer. NOTE: The time sequence indexing follows the array indexing in the inputs: sourceGradients[0] has to contain the gradients corresponding to the first matrix in the forward pass corresponding to the current subsequence, which is typically sourceMatrices[0].

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter forwardSources: An array of MPSMatrix objects containing the sequence of source matrices of the forward pass.

Parameter sourceGradients: An array of MPSMatrix objects containing the sequence of source gradient matrices.

Parameter destinationGradients: An array valid MPSMatrix objects that will receive the backpropagated gradients, may be nil if not needed (for example first layer in graph).

Parameter weightGradients: An array valid MPSMatrix objects that will receive the gradient wrt. weights and biases of the layer - should be the array that was produced either by

See: initWithDevice or

See: createWeightMatrices. May be nil in which case the gradients for the weights are not computed. NOTE: The weight gradients are accumulated on top of existing values so

Parameter trainingStates: An array containing the training states from the forward pass - the array must contain the states corresponding to the input gradients is sourceGradients.

Parameter weights: An array of valid MPSMatrix objects containing the weights, should be the array that was produced either by

See: initWithDevice or

See: createWeightMatrices.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

NSSecureCoding compatability

See MPSKernel#initWithCoder. Parameter aDecoder: The NSCoder subclass with your serialized MPSRNNMatrixTrainingLayer

Parameter device: The MTLDevice on which to make the MPSRNNMatrixTrainingLayer

Returns: A new MPSRNNMatrixTrainingLayer object, or nil if failure.

Safety

a_decoder possibly has further requirements.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Make a copy of this kernel for a new device -

See: MPSKernel

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Source

impl MPSRNNMatrixTrainingLayer

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSNeuralNetwork only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSRNNMatrixTrainingLayer

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSNeuralNetwork only.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}