MPSState

Struct MPSState 

pub struct MPSState { /* private fields */ }
Available on crate feature MPSState only.
Dependencies: This depends on Metal Framework

A semi-opaque data container for large storage in MPS CNN filters

Some MPS CNN kernels produce additional information beyond a MPSImage. These may be pooling indices where the result came from, convolution weights, or other information not contained in the usual MPSImage result from a MPSCNNKernel. A MPSState object typically contains one or more expensive MTLResources such as textures or buffers to store this information. It provides a base class with interfaces for managing this storage. Child classes may add additional functionality specific to their contents.

Some MPSState objects are temporary. Temporary state objects, like MPSTemporaryImages and Matrices, are for very short lived storage, perhaps just a few lines of code within the scope of a single MTLCommandBuffer. They are very efficient for storage, as several temporary objects can share the same memory over the course of a MTLCommandBuffer. This can improve both memory usage and time spent in the kernel wiring down memory and such. You may find that some large CNN tasks can not be computed without them, as non-temporary storage would simply take up too much memory.

In exchange, the lifetime of the underlying storage in temporary MPSState objects needs to be carefully managed. ARC often waits until the end of scope to release objects. Temporary storage often needs to be released sooner than that. Consequently the lifetime of the data in the underlying MTLResources is managed by a readCount property. Each time a MPSCNNKernel reads a temporary MPSState object the readCount is automatically decremented. When it reaches zero, the underlying storage is recycled for use by other MPS temporary objects, and the data is becomes undefined. If you need to consume the data multiple times, you should set the readCount to a larger number to prevent the data from becomming undefined. You may set the readCount to 0 yourself to return the storage to MPS, if for any reason, you realize that the MPSState object will no longer be used.

The contents of a temporary MPSState object are only valid from creation to the time the readCount reaches 0. The data is only valid for the MTLCommandBuffer on which it was created. Non-temporary MPSState objects are valid on any MTLCommandBuffer on the same device until they are released.

Finally, temporary MPSState objects are complicated to use with blit encoders. Your application should assume that the temporary MPSState is backed by a MTLHeap, and consequently needs a MTLFence to ensure that compute command encoders and other encoders do not trip over one another with heap based memory. MPS will almost never use a blit encoder for this reason. If you do need one, then you will need to make a new compute encoder to block on whatever previous compute encoder last used the heap block. (MPS will not tell you who previously used the heap block. That encoder is almost certainly long dead anyway.) If concurrent encoders are involved, then a barrier might be needed. Within that compute encoder, you will call -updateFence. End the compute encoder, make a blit encoder wait for the fence, do the blit, update a new fence, then make a new compute encoder, wait for the second fence, then you can continue. Possibly the second do-nothing compute encoder needs to be ended so MPS can be called. Frankly, we don’t bother with blit encoders and just write a compute operation for copy / clear as needed, or better yet find a way to eliminate the clear / copy pass so we don’t have to pay for it. Your application needs to use temporary MPSStates and MPSTemporaryImages. Memory costs skyrocket, otherwise. It is the blit encoder that is hopefully optional. Note: the most common use of a blit encoder, -synchronizeResource: can not encounter this problem because temporary images and states live in GPU private memory and can not be read by the CPU.

See also Apple’s documentation

Implementations

Source

impl MPSState

pub unsafe fn temporaryStateWithCommandBuffer_bufferSize( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, buffer_size: usize, ) -> Retained<Self>

Available on crate feature MPSCore only.

Create a MPSState holding a temporary MTLBuffer

Parameter cmdBuf: The command buffer against which the temporary resource is allocated

Parameter bufferSize: The size of the buffer in bytes

pub unsafe fn temporaryStateWithCommandBuffer_textureDescriptor( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, descriptor: &MTLTextureDescriptor, ) -> Retained<Self>

Available on crate feature MPSCore only.

Create a MPSState holding a temporary MTLTexture

Parameter cmdBuf: The command buffer against which the temporary resource is allocated

Parameter descriptor: A descriptor for the new temporary texture

pub unsafe fn temporaryStateWithCommandBuffer( cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, ) -> Retained<Self>

Available on crate feature MPSCore only.

Create a new autoreleased temporary state object without underlying resource

Parameter cmdBuf: The command buffer with which the temporary resource is associated

pub unsafe fn initWithDevice_bufferSize( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, buffer_size: usize, ) -> Retained<Self>

Available on crate feature MPSCore only.
pub unsafe fn initWithDevice_textureDescriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, descriptor: &MTLTextureDescriptor, ) -> Retained<Self>

Available on crate feature MPSCore only.
pub unsafe fn initWithResource( this: Allocated<Self>, resource: Option<&ProtocolObject<dyn MTLResource>>, ) -> Retained<Self>

Available on crate feature MPSCore only.

Create a MPSState with a non-temporary MTLResource

Parameter resource: A MTLBuffer or MTLTexture. May be nil.

Safety
  • resource may need to be synchronized.
  • resource may be unretained, you must ensure it is kept alive while in use.
pub unsafe fn init(this: Allocated<Self>) -> Option<Retained<Self>>

Available on crate feature MPSCore only.
pub unsafe fn initWithDevice_resourceList( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, resource_list: &MPSStateResourceList, ) -> Retained<Self>

Available on crate feature MPSCore only.

Initialize a non-temporary state to hold a number of textures and buffers

The allocation of each resource will be deferred until it is needed. This occurs when -resource or -resourceAtIndex: is called.

Parameter resourceList: The list of resources to create.

pub unsafe fn temporaryStateWithCommandBuffer_resourceList( command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, resource_list: &MPSStateResourceList, ) -> Retained<Self>

Available on crate feature MPSCore only.

Initialize a temporary state to hold a number of textures and buffers

The textures occur first in sequence

pub unsafe fn initWithResources( this: Allocated<Self>, resources: Option<&NSArray<ProtocolObject<dyn MTLResource>>>, ) -> Retained<Self>

Available on crate feature MPSCore only.

Create a state object with a list of MTLResources

Because MPS prefers deferred allocation of resources your application should use -initWithTextures:bufferSizes:bufferCount: whenever possible. This method is useful for cases when the MTLResources must be initialized by the CPU.

Safety
  • resources generic may need to be synchronized.
  • resources generic may be unretained, you must ensure it is kept alive while in use.
pub unsafe fn resourceCount(&self) -> NSUInteger

Available on crate feature MPSCore only.

Return the number of MTLResource objects held by the state

pub unsafe fn resourceAtIndex_allocateMemory( &self, index: NSUInteger, allocate_memory: bool, ) -> Option<Retained<ProtocolObject<dyn MTLResource>>>

Available on crate feature MPSCore only.

Get the MTLResource at the indicated index

By convention, except where otherwise documented, the MTLResources held by the MPSState are private to the MPSState object, owned by the MPSState subclass author. If the MPSState subclass author is MPS, then the identity (e.g. texture vs. buffer) and information contained in the resource should be considered implementation dependent. It may change by operating system version or device. If you are the author of the subclass then it is for your own use, and MPS will not look at it, except perhaps so as to pass it to a custom kernel. Otherwise, the method is made available to facilitate debugging and to allow you to write your own state objects. Provide accessors to read this information in a defined format.

Parameter index: The index of the MTLResource to retrieve

Parameter allocateMemory: It is very important to avoid allocating memory to hold MTLResources until it is absolutely necessary, especially when working with temporary MPSStates. When allocateMemory is set to NO and the resource has not yet been allocated, nil will be returned instead. If you just need information about the resource such as buffer size or MTLTexture properties, but not the resource itself, please use -bufferSizeAtIndex: or -textureInfoAtIndex: instead, as these will not force the creation of the MTLResource.

pub unsafe fn readCount(&self) -> NSUInteger

Available on crate feature MPSCore only.
pub unsafe fn setReadCount(&self, read_count: NSUInteger)

Available on crate feature MPSCore only.

Setter for readCount.

pub unsafe fn isTemporary(&self) -> bool

Available on crate feature MPSCore only.
pub unsafe fn label(&self) -> Option<Retained<NSString>>

Available on crate feature MPSCore only.

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Available on crate feature MPSCore only.

Setter for label.

This is copied when set.

pub unsafe fn bufferSizeAtIndex(&self, index: NSUInteger) -> NSUInteger

Available on crate feature MPSCore only.

Return the buffer size of the MTLBuffer at index or 0 if it is not a MTLBuffer

Does not force allocation of the MTLResource

pub unsafe fn textureInfoAtIndex( &self, index: NSUInteger, ) -> MPSStateTextureInfo

Available on crate feature MPSCore only.

Return the texture size {width,height,depth} or {0,0,0} if it is not a MTLTexture

Does not force allocation of the MTLResource

pub unsafe fn resourceTypeAtIndex( &self, index: NSUInteger, ) -> MPSStateResourceType

Available on crate feature MPSCore only.

Return YES if the resource at index is a buffer

Does not force allocation of the MTLResource

pub unsafe fn synchronizeOnCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

Available on crate feature MPSCore only.

Flush any copy of MTLResources held by the state from the device’s caches, and invalidate any CPU caches if needed.

This will call [id <MTLBlitEncoder

synchronizeResource: ] on the state’s MTLResources. This is necessary for all MTLStorageModeManaged resources. For other resources, including temporary resources (these are all MTLStorageModePrivate), nothing is done.

Parameter commandBuffer: The commandbuffer on which to synchronize

pub unsafe fn resourceSize(&self) -> NSUInteger

Available on crate feature MPSCore only.

Get the number of bytes used to allocate underyling MTLResources

This is the size of the backing store of underlying MTLResources. It does not include all storage used by the object, for example the storage used to hold the MPSState instantiation and MTLTexture or MTLBuffer is not included. It only measures the size of the allocation used to hold the texels in the texture or bytes in the buffer. This value is subject to change between different devices and operating systems.

Except when -initWithResource: is used, most MPSStates are allocated without a backing store. The backing store is allocated lazily when it is needed, typically when the .texture property is called. Consequently, in most cases, it should be inexpensive to make a MPSImage to see how much memory it will need, and release it if it is too large.

This method may fail in certain circumstances, such as when the MPSImage is created with -initWithTexture:featureChannels:, in which case 0 will be returned.

pub unsafe fn destinationImageDescriptorForSourceImages_sourceStates_forKernel_suggestedDescriptor( &self, source_images: &NSArray<MPSImage>, source_states: Option<&NSArray<MPSState>>, kernel: &MPSKernel, in_descriptor: &MPSImageDescriptor, ) -> Retained<MPSImageDescriptor>

Available on crate features MPSCore and MPSImage and MPSKernel only.

Determine padding and sizing of result images

A MPSState has the opportunity to reconfigure the MPSImageDescriptor used to create the filter result state and set the MPSKernel.offset to the correct value. By default, the MPSState does not modify the descriptor.

There is a order of operations defined for who may update the descriptor:

  1. Default padding code runs based on the MPSNNPaddingMethod in the MPSCNNKernel.padding. This creates the descriptor and picks a starting value for the MPSCNNKernel.offset.
  2. MPSStates are called in order to apply this function and update the offset.
  3. The MPSNNPadding custom padding method of the same name is called.

Some code that may prove helpful:

                  const int centeringPolicy = 0;  // When kernelSize is even: 0 pad bottom right. 1 pad top left.    Centers the kernel for even sized kernels.

                  typedef enum Style{
                      StyleValidOnly = -1,
                      StyleSame = 0,
                      StyleFull = 1
                  }Style;

                  // Typical destination size in one dimension for forward filters (most filters)
                  static int DestSize( int sourceSize, int stride, int filterWindowSize, Style style ){
                      sourceSize += style * (filterWindowSize - 1);       // adjust how many pixels we are allowed to read
                      return (sourceSize + stride - 1) / stride;          // sourceSize / stride, round up
                  }

                  // Typical destination size in one dimension for reverse filters (e.g. convolution transpose)
                  static int DestSizeReverse( int sourceSize, int stride, int filterWindowSize, Style style ){
                      return (sourceSize-1) * stride +        // center tap for the last N-1 results. Take stride into account
                              1 +                             // center tap for the first result
                              style * (filterWindowSize-1);   // add or subtract (or ignore) the filter extent
                  }

                  // Find the MPSOffset in one dimension
                  static int Offset( int sourceSize, int stride, int filterWindowSize, Style style ){
                      // The correction needed to adjust from position of left edge to center per MPSOffset definition
                      int correction = filterWindowSize / 2;

                      // exit if all we want is to start consuming pixels at the left edge of the image.
                      if( 0 )
                          return correction;

                      // Center the area consumed in the source image:
                      // Calculate the size of the destination image
                      int destSize = DestSize( sourceSize, stride, filterWindowSize, style ); // use DestSizeReverse here instead as appropriate

                      // calculate extent of pixels we need to read in source to populate the destination
                      int readSize = (destSize-1) * stride + filterWindowSize;

                      // calculate number of missing pixels in source
                      int extraSize = readSize - sourceSize;

                      // number of missing pixels on left side
                      int leftExtraPixels = (extraSize + centeringPolicy) / 2;

                      // account for the fact that the offset is based on the center pixel, not the left edge
                      return correction - leftExtraPixels;
                  }

Parameter sourceImages: The list of source images to be used

Parameter sourceStates: The list of source states to be used

Parameter kernel: The MPSKernel the padding method will be applied to. Set the kernel.offset

Parameter inDescriptor: MPS will prepare a starting guess based on the padding policy (exclusive of MPSNNPaddingMethodCustom) set for the object. You should adjust the offset and image size accordingly. It is on an autoreleasepool.

Returns: The MPSImageDescriptor to use to make a MPSImage to capture the results from the filter. The MPSImageDescriptor is assumed to be on an autoreleasepool. Your method must also set the kernel.offset property.

pub unsafe fn resource( &self, ) -> Option<Retained<ProtocolObject<dyn MTLResource>>>

👎Deprecated
Available on crate feature MPSCore only.

Get the private MTLResource underlying the MPSState

When the state is not directly initialized with a MTLResource, the actuall MTLResource creation is deferred. Especially with temporary resources, it is important to delay this creation as late as possible to avoid increasing the memory footprint. The memory is returned for reuse when the readCount = 0. Calling the -resource method will force the resource to be allocated, so you should not use it lightly, for purposes such as finding the MTLPixelFormat of a texture in the state.

By convention, except where otherwise documented, the MTLResources held by the MPSState are private to the MPSState object, owned by the MPSState subclass author. If the MPSState subclass author is MPS, then the identity (e.g. texture vs. buffer) and information contained in the resource should be considered implementation dependent. It may change by operating system version or device. If you are the author of the subclass then it is for your own use, and MPS will not look at it, except perhaps so as to pass it to a custom kernel. Otherwise, the method is made available to facilitate debugging and to allow you to write your own state objects.

Source

impl MPSState

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSCore only.
Source

impl MPSState

pub unsafe fn batch_increment_read_count( batch: Option<&MPSStateBatch>, amount: NSInteger, ) -> NSUInteger

Available on crate feature MPSCore only.

raise or lower the readcount of a batch by a set amount

In some circumstances, a MPSState may appear in a MPSStateBatch multiple times. This is particularly common when the MPSState serves as an accumulator across the entire batch, such as when accumulating gradients for convolution weight update or batch statistics for batch normalization. A naive function would then end up incrementing the state multiple times, probably leading to an error.

MPSStateBatchIncrementReadCount() will efficiently increment the readCounts of each object in the batch only once, avoiding this problem.

Parameter batch: The MPSStateBatch to increment

Parameter amount: The value to add to the read count for each unique state in the batch

Returns: The number of different objects in the batch

pub unsafe fn batch_synchronize( batch: &MPSStateBatch, cmd_buf: &ProtocolObject<dyn MTLCommandBuffer>, )

Available on crate feature MPSCore only.

Call [MTLBlitEncoder synchronizeResource:] on unique resources

pub unsafe fn batch_resource_size(batch: Option<&MPSStateBatch>) -> NSUInteger

Available on crate feature MPSCore only.

Call [MTLBlitEncoder resourceSize] on unique resources

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}