MPSTemporaryVector

Struct MPSTemporaryVector 

pub struct MPSTemporaryVector { /* private fields */ }
Available on crate feature MPSMatrix only.
A MPSVector allocated on GPU private memory.

It may alias one or more other MPSTemporaryVector objects. Undesired data destruction due to aliasing is avoided using the readCount property.

See also Apple’s documentation

Implementations

Source

impl MPSTemporaryVector

pub unsafe fn temporaryVectorWithCommandBuffer_descriptor( command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, descriptor: &MPSVectorDescriptor, ) -> Retained<Self>

Available on crate feature MPSCore only.

Initialize a MPSTemporaryVector for use on a MTLCommandBuffer

Parameter commandBuffer: The MTLCommandBuffer on which the MPSTemporaryMatrix will be exclusively used

Parameter descriptor: A valid MPSVectorDescriptor describing the MPSVector format to create

Returns: A valid MPSTemporaryVector. The object is not managed by a NSAutoreleasePool. The object will be released when the command buffer is committed. The underlying buffer will become invalid before this time due to the action of the readCount property. Please read and understand the use of the readCount property before using this object.

pub unsafe fn prefetchStorageWithCommandBuffer_descriptorList( command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, descriptor_list: &NSArray<MPSVectorDescriptor>, )

Available on crate feature MPSCore only.

Help MPS decide which allocations to make ahead of time

The buffer cache that underlies the MPSTemporaryVector can automatically allocate new storage as needed as you create new temporary vectors. However, sometimes a more global view of what you plan to make is useful for maximizing memory reuse to get the most efficient operation. This class method hints to the cache what the list of matrices will be.

It is never necessary to call this method. It is purely a performance and memory optimization.

Parameter commandBuffer: The command buffer on which the MPSTemporaryVector will be used

Parameter descriptorList: A NSArray of MPSVectorDescriptor objects, indicating vectors that will be created

pub unsafe fn initWithBuffer_descriptor( this: Allocated<Self>, buffer: &ProtocolObject<dyn MTLBuffer>, descriptor: &MPSVectorDescriptor, ) -> Retained<Self>

Available on crate feature MPSCore only.

*** unavailable

Safety
  • buffer may need to be synchronized.
  • buffer may be unretained, you must ensure it is kept alive while in use.
  • buffer contents should be of the correct type.
pub unsafe fn readCount(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of times a temporary vector may be read by a MPSMatrix… kernel before its contents become undefined.

MPSTemporaryVector objects must release their underlying buffers for reuse immediately after last use. So as to facilitate prompt convenient memory recycling, each time a MPSTemporaryVector is read by a MPSMatrix… -encode… method, its readCount is automatically decremented. When the readCount reaches 0, the underlying buffer is automatically made available for reuse to MPS for its own needs and for other MPSTemporaryVector objects prior to return from the -encode.. function. The contents of the buffer become undefined at this time.

By default, the readCount is initialized to 1, indicating a matrix that may be overwritten any number of times, but read only once.

You may change the readCount as desired to allow MPSMatrix kernels to read the MPSTemporaryVector additional times. However, it is an error to change the readCount once it is zero. It is an error to read or write to a MPSTemporaryVector with a zero readCount. You may set the readCount to 0 yourself to cause the underlying buffer to be returned to MPS. Writing to a MPSTemporaryVector does not adjust the readCount.

The Metal API Validation layer will assert if a MPSTemporaryVector is deallocated with non-zero readCount to help identify cases when resources are not returned promptly.

pub unsafe fn setReadCount(&self, read_count: NSUInteger)

Available on crate feature MPSCore only.

Setter for readCount.

Source

impl MPSTemporaryVector

Methods declared on superclass MPSVector.

pub unsafe fn initWithBuffer_offset_descriptor( this: Allocated<Self>, buffer: &ProtocolObject<dyn MTLBuffer>, offset: NSUInteger, descriptor: &MPSVectorDescriptor, ) -> Retained<Self>

Available on crate feature MPSCore only.

Initialize a MPSVector object with a MTLBuffer and an offset.

Parameter buffer: The MTLBuffer containing the data.

Parameter offset: The offset, in bytes, into the buffer at which data begins.

Parameter descriptor: The MPSVectorDescriptor.

Safety
  • buffer may need to be synchronized.
  • buffer may be unretained, you must ensure it is kept alive while in use.
  • buffer contents should be of the correct type.
pub unsafe fn initWithDevice_descriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, descriptor: &MPSVectorDescriptor, ) -> Retained<Self>

Available on crate feature MPSCore only.

Initialize a lazily backed MPSVector object with a descriptor

Parameter device: The device with which it will be used

Parameter descriptor: The shape and style of the matrix

Returns: A valid MPSVector object or nil

The vector object will be created, but the storage to hold the vector data will only be allocated when it is needed, typically when the data property is invoked. In conjunction with -resourceSize, this will allow you to estimate storage needs without actually creating the backing store for the vector.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSCore only.
Source

impl MPSTemporaryVector

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSCore only.

Methods from Deref<Target = MPSVector>

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

Available on crate feature MPSCore only.

The device on which the MPSVector will be used.

pub unsafe fn length(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of elements in the vector.

pub unsafe fn vectors(&self) -> NSUInteger

Available on crate feature MPSCore only.

The number of vectors in the MPSVector.

pub unsafe fn dataType(&self) -> MPSDataType

Available on crate features MPSCore and MPSCoreTypes only.

The type of the MPSVector data.

pub unsafe fn vectorBytes(&self) -> NSUInteger

Available on crate feature MPSCore only.

The stride, in bytes, between corresponding elements of consecutive vectors.

pub unsafe fn offset(&self) -> NSUInteger

Available on crate feature MPSCore only.

Byte-offset to the buffer where the vector data begins - see initWithBuffer:offset: descriptor: .

pub unsafe fn data(&self) -> Retained<ProtocolObject<dyn MTLBuffer>>

Available on crate feature MPSCore only.

An MTLBuffer to store the data.

pub unsafe fn synchronizeOnCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

Available on crate feature MPSCore only.

Flush the underlying MTLBuffer from the device’s caches, and invalidate any CPU caches if needed.

This will call [id <MTLBlitEncoder

synchronizeResource: ] on the vector’s MTLBuffer, if any. This is necessary for all MTLStorageModeManaged resources. For other resources, including temporary resources (these are all MTLStorageModePrivate), and buffers that have not yet been allocated, nothing is done. It is more efficient to use this method than to attempt to do this yourself with the data property.

Parameter commandBuffer: The commandbuffer on which to synchronize

pub unsafe fn resourceSize(&self) -> NSUInteger

Available on crate feature MPSCore only.

Get the number of bytes used to allocate underyling MTLResources

This is the size of the backing store of underlying MTLResources. It does not include all storage used by the object, for example the storage used to hold the MPSVector instantiation and MTLBuffer is not included. It only measures the size of the allocation used to hold the vector data in the buffer. This value is subject to change between different devices and operating systems.

Except when -initWithBuffer:descriptor: is used, most MPSVectors are allocated without a backing store. The backing store is allocated lazily when it is needed, typically when the .texture property is called. Consequently, in most cases, it should be inexpensive to make a MPSMatrix to see how much memory it will need, and release it if it is too large.

This method may fail in certain circumstances, such as when the MPSMatrix is created with -initWithBuffer:descriptor:. In such cases, 0 will be returned.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}