MPSTriangleAccelerationStructure

Struct MPSTriangleAccelerationStructure 

pub struct MPSTriangleAccelerationStructure { /* private fields */ }
👎Deprecated
Available on crate features MPSAccelerationStructure and MPSCore and MPSKernel and MPSPolygonAccelerationStructure and MPSTriangleAccelerationStructure only.
An acceleration structure built over triangles

See MPSPolygonAccelerationStructure for more information

See also Apple’s documentation

Implementations

Source

impl MPSTriangleAccelerationStructure

pub unsafe fn triangleCount(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Number of triangles. Changes to this property require rebuilding the acceleration structure.

Note that this property is an alias for the polygonCount property.

pub unsafe fn setTriangleCount(&self, triangle_count: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for triangleCount.

Source

impl MPSTriangleAccelerationStructure

Methods declared on superclass MPSAccelerationStructure.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

👎Deprecated
Available on crate feature MPSRayIntersector only.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Initialize the acceleration structure with a Metal device

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Initialize the acceleration structure with an NSCoder and a Metal device. Buffer properties such as the vertex buffer, instance buffer, etc. are set to nil. Encode and decode these buffers along with the acceleration structure instead.

Safety

a_decoder possibly has further requirements.

pub unsafe fn initWithGroup( this: Allocated<Self>, group: &MPSAccelerationStructureGroup, ) -> Retained<Self>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

Initialize the acceleration structure with an acceleration structure group, if the acceleration structure will be used in an instance hierarchy.

The Metal device is determined from the acceleration structure group. All acceleration structures in the instance hierarchy must share the same group.

pub unsafe fn initWithCoder_group( this: Allocated<Self>, a_decoder: &NSCoder, group: &MPSAccelerationStructureGroup, ) -> Option<Retained<Self>>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

Initialize the acceleration structure with an NSCoder and an acceleration structure group, if the acceleration structure will be used in an instance hierarchy. All acceleration structures in the instance hierarchy must share the same group. Buffer properties such as the vertex buffer, instance buffer, etc. are set to nil. Encode and decode these buffers along with the acceleration structure instead.

Safety

a_decoder possibly has further requirements.

Source

impl MPSTriangleAccelerationStructure

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSRayIntersector only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

Source

impl MPSTriangleAccelerationStructure

Methods declared on superclass NSObject.

pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSRayIntersector only.

Methods from Deref<Target = MPSPolygonAccelerationStructure>

pub unsafe fn polygonType(&self) -> MPSPolygonType

👎Deprecated
Available on crate feature MPSRayIntersector only.

The type of polygon. Defaults to MPSPolygonTypeTriangle. Changes to this property require rebuilding the acceleration structure.

pub unsafe fn setPolygonType(&self, polygon_type: MPSPolygonType)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for polygonType.

pub unsafe fn vertexStride(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, between consecutive vertices in the vertex buffer. Defaults to 0 bytes, indicating that the vertices are packed according to the natural alignment of the vector_float3 type: 16 bytes.

This can be used to skip past any additional per-vertex data which may be stored alongside the position such as the vertex normal and texture coordinates. Must be a multiple of 4 bytes, and must be at least 12 bytes. Changes to this property require rebuilding the acceleration structure.

pub unsafe fn setVertexStride(&self, vertex_stride: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for vertexStride.

pub unsafe fn indexType(&self) -> MPSDataType

👎Deprecated
Available on crate features MPSRayIntersector and MPSCoreTypes only.

Index type. Defaults to MPSDataTypeUInt32. Only MPSDataTypeUInt16 and MPSDataTypeUInt32 are supported.

pub unsafe fn setIndexType(&self, index_type: MPSDataType)

👎Deprecated
Available on crate features MPSRayIntersector and MPSCoreTypes only.

Setter for indexType.

pub unsafe fn vertexBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Vertex buffer containing vertex data encoded as three 32 bit floats per vertex. Note that by default each vertex is aligned to the alignment of the vector_float3 type: 16 bytes. This can be changed using the vertexStride property. A vertex buffer must be provided before the acceleration structure is built.

When using triangle polygons, degenerate (zero or negative area) triangles are ignored during acceleration structure construction. This can be used to pad triangle indices if needed.

Quadrilateral polygons are internally treated as two triangles. If the quadrilateral has vertices v0, v1, v2, and v3, the two triangles will have vertices v0, v1, v2 and v0, v2, v3. A quadrilateral may be used to represent a triangle by repeating the last vertex. If the first triangle is degenerate (zero or negative area), the entire quadrilateral will be ignored. This can be used to pad quadrilateral indices if needed. All four vertices of a quadrilateral must be coplanar and the quadrilateral must be convex.

This is an alias for polygonBuffers[0].vertexBuffer. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setVertexBuffer( &self, vertex_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for vertexBuffer.

Safety
  • vertex_buffer may need to be synchronized.
  • vertex_buffer may be unretained, you must ensure it is kept alive while in use.
  • vertex_buffer contents should be of the correct type.
pub unsafe fn vertexBufferOffset(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the vertex buffer. Defaults to 0 bytes. Must be aligned to 4 bytes.

This is an alias for polygonBuffers[0].vertexBufferOffset. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setVertexBufferOffset(&self, vertex_buffer_offset: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for vertexBufferOffset.

pub unsafe fn indexBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Index buffer containing index data. Each index references a vertex in the vertex buffer. May be nil.

This is an alias for polygonBuffers[0].indexBuffer. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setIndexBuffer( &self, index_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for indexBuffer.

Safety
  • index_buffer may need to be synchronized.
  • index_buffer may be unretained, you must ensure it is kept alive while in use.
  • index_buffer contents should be of the correct type.
pub unsafe fn indexBufferOffset(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the index buffer. Defaults to 0 bytes. Must be aligned to a multiple of the index type. Changes to this property require rebuilding the acceleration structure.

This is an alias for polygonBuffers[0].indexBufferOffset. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setIndexBufferOffset(&self, index_buffer_offset: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for indexBufferOffset.

pub unsafe fn maskBuffer( &self, ) -> Option<Retained<ProtocolObject<dyn MTLBuffer>>>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Mask buffer containing one uint32_t mask per polygon. May be nil. Otherwise, the mask type must be specified on the MPSRayIntersector with which it is used.

This is an alias for polygonBuffers[0].maskBuffer. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setMaskBuffer( &self, mask_buffer: Option<&ProtocolObject<dyn MTLBuffer>>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for maskBuffer.

Safety
  • mask_buffer may need to be synchronized.
  • mask_buffer may be unretained, you must ensure it is kept alive while in use.
  • mask_buffer contents should be of the correct type.
pub unsafe fn maskBufferOffset(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Offset, in bytes, into the mask buffer. Defaults to 0 bytes. Must be aligned to 4 bytes.

This is an alias for polygonBuffers[0].maskBufferOffset. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setMaskBufferOffset(&self, mask_buffer_offset: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for maskBufferOffset.

pub unsafe fn polygonCount(&self) -> NSUInteger

👎Deprecated
Available on crate feature MPSRayIntersector only.

Number of polygons. Changes to this property require rebuilding the acceleration structure.

This is an alias for polygonBuffers[0].polygonCount. There must be exactly one polygon buffer to use this property, or the polygonBuffers property must be nil, in which case an MPSPolygonBuffer will be created automatically.

pub unsafe fn setPolygonCount(&self, polygon_count: NSUInteger)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for polygonCount.

pub unsafe fn polygonBuffers( &self, ) -> Option<Retained<NSArray<MPSPolygonBuffer>>>

👎Deprecated
Available on crate features MPSRayIntersector and MPSPolygonBuffer only.

Array of polygon buffers. Each buffer contains a vertex buffer and optional index and mask buffer for an array of polygons. Changing the length of this array requires rebuilding the acceleration structure.

Using more than one MPSPolygonBuffer will reduce performance. It is better to concatenate these buffers into a single vertex buffer, index buffer, and mask buffer and use a single MPSPolygonBuffer if possible. This also applies when using an MPSInstanceAccelerationStructure: each instance or subclass of MPSPolygonAccelerationStructure in an instance hierarchy should use the same vertex buffer, index buffer, and mask buffer, although each acceleration structure may use different offsets into these buffers. This allows for the vertex, index, and mask buffers to be bound directly instead of indirectly through an argument buffer.

There must be at least one MPSPolygonBuffer. On argument buffer tier 1 devices, there must be be exactly one MPSPolygonBuffer. Use the argumentBuffersSupport property of the MTLDevice to check for support.

pub unsafe fn setPolygonBuffers( &self, polygon_buffers: Option<&NSArray<MPSPolygonBuffer>>, )

👎Deprecated
Available on crate features MPSRayIntersector and MPSPolygonBuffer only.

Setter for polygonBuffers.

Methods from Deref<Target = MPSAccelerationStructure>

pub unsafe fn group(&self) -> Retained<MPSAccelerationStructureGroup>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

The group this acceleration structure was created with

pub unsafe fn status(&self) -> MPSAccelerationStructureStatus

👎Deprecated
Available on crate feature MPSRayIntersector only.

Status indicating whether the acceleration structure has finished building

pub unsafe fn usage(&self) -> MPSAccelerationStructureUsage

👎Deprecated
Available on crate feature MPSRayIntersector only.

Acceleration structure usage options. Changes to this property require rebuilding the acceleration structure. Defaults to MPSAccelerationStructureUsageNone.

pub unsafe fn setUsage(&self, usage: MPSAccelerationStructureUsage)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Setter for usage.

pub unsafe fn rebuild(&self)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Rebuild the acceleration structure

This method must be called before any intersection tests can be scheduled with this acceleration structure. Before calling this method, fill out the properties of the acceleration structure such as vertex buffer, instance buffer, etc. The acceleration structure should be rebuilt when its contents (e.g. vertices in a triangle acceleration structure) have been modified significantly and must be rebuilt when properties such as triangle count, vertex stride, etc. have changed. When the contents of the acceleration structure have only been modified slightly, it may be cheaper to refit the acceleration structure instead.

This method blocks until the acceleration structure has been rebuilt. Until the rebuild has completed, the acceleration structure cannot be copied, encoded with NSSecureCoding, rebuilt, or refit. Before this method can be called, any pending GPU writes to the vertex buffer, index buffer, etc. must be completed (and, for managed buffers, synchronized). Any prior intersection tests must also be completed before the acceleration structure can be rebuilt.

pub unsafe fn rebuildWithCompletionHandler( &self, completion_handler: MPSAccelerationStructureCompletionHandler, )

👎Deprecated
Available on crate features MPSRayIntersector and block2 only.

Rebuild the acceleration structure asynchronously

This method must be called before any intersection tests can be scheduled with this acceleration structure. Before calling this method, fill out the properties of the acceleration structure such as vertex buffer, instance buffer, etc. The acceleration structure should be rebuilt when its contents (e.g. vertices in a triangle acceleration structure) have been modified significantly and must be rebuilt when properties such as triangle count, vertex stride, etc. have changed. When the contents of the acceleration structure have only been modified slightly, it may be cheaper to refit the acceleration structure instead.

Until the rebuild has completed, the acceleration structure cannot be copied, encoded with NSSecureCoding, rebuilt, or refit. Before this method can be called, any pending GPU writes to the vertex buffer, index buffer, etc. must be completed (and, for managed buffers, synchronized). Any prior intersection tests must also be completed before the acceleration structure can be rebuilt.

Safety

completion_handler must be a valid pointer.

pub unsafe fn encodeRefitToCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )

👎Deprecated
Available on crate feature MPSRayIntersector only.

Refit the existing acceleration structure to new data

This method is used to refit the acceleration structure to new vertex data, index data, instance data, etc. while preserving the existing acceleration structure topology. This is typically much faster than a full rebuild of the acceleration structure. Refitting can also be pipelined with other GPU work such as ray intersection.

Until the command buffer has completed, the acceleration structure cannot be copied, encoded with NSSecureCoding, or rebuilt. Changes to properties such as the triangle count or instance count might not be reflected. These changes require that the acceleration structure be rebuilt instead. The acceleration structure must be rebuilt at least once before this method can be called.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

👎Deprecated
Available on crate feature MPSRayIntersector only.

Create a a copy of this acceleration structure

The acceleration structure may be copied to a different Metal device. Buffer properties of the acceleration structure such as the vertex buffer, instance, buffer, etc. are set to nil. Copy these buffers to the new Metal device and assign them to the new acceleration structure instead. Do not copy the acceleration structure until any prior refit or rebuild operations have completed.

Parameter zone: This parameter is ignored. Memory zones are no longer used by Objective-C.

Parameter device: New Metal device

Safety

zone must be a valid pointer or null.

pub unsafe fn copyWithZone_group( &self, zone: *mut NSZone, group: &MPSAccelerationStructureGroup, ) -> Retained<Self>

👎Deprecated
Available on crate features MPSRayIntersector and MPSAccelerationStructureGroup only.

Create a a copy of this acceleration structure

The acceleration structure may be copied with a different acceleration structure group. Buffer properties of the acceleration structure such as the vertex buffer, instance buffer, etc. are set to nil. Copy these buffers with the new Metal device and assign them to the new acceleration structure instead. Do not copy the acceleration structure until any prior refit or rebuild operations have completed.

Parameter zone: This parameter is ignored. Memory zones are no longer used by Objective-C.

Parameter group: New acceleration structure group

Safety

zone must be a valid pointer or null.

pub unsafe fn encodeWithCoder(&self, coder: &NSCoder)

👎Deprecated
Available on crate feature MPSRayIntersector only.

Encode the acceleration structure with an NSCoder

Buffer properties such as the vertex buffer, index buffer, etc. are not be encoded. Encode and decode these buffers along with the acceleration structure instead. Do not encode the acceleration structure until any prior refit or rebuild operations have completed.

Parameter coder: An archiver object

Safety

coder possibly has further requirements.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}