pub struct MPSVectorDescriptor { /* private fields */ }MPSMatrix only.Expand description
Dependencies: This depends on Metal.framework
A MPSVectorDescriptor describes the length and data type of a an array of 1-dimensional vectors. All vectors are stored as contiguous arrays of data.
See also Apple’s documentation
Implementations
Sourceimpl MPSVectorDescriptor
impl MPSVectorDescriptor
Sourcepub unsafe fn length(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn length(&self) -> NSUInteger
MPSCore only.The number of elements in the vector.
Sourcepub unsafe fn setLength(&self, length: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setLength(&self, length: NSUInteger)
MPSCore only.Setter for length.
Sourcepub unsafe fn vectors(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn vectors(&self) -> NSUInteger
MPSCore only.The number of vectors.
Sourcepub unsafe fn dataType(&self) -> MPSDataType
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn dataType(&self) -> MPSDataType
MPSCore and MPSCoreTypes only.The type of the data which makes up the values of the vector.
Sourcepub unsafe fn setDataType(&self, data_type: MPSDataType)
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn setDataType(&self, data_type: MPSDataType)
MPSCore and MPSCoreTypes only.Setter for dataType.
Sourcepub unsafe fn vectorBytes(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn vectorBytes(&self) -> NSUInteger
MPSCore only.The stride, in bytes, between corresponding elements of consecutive vectors. Must be a multiple of the element size
Sourcepub unsafe fn vectorDescriptorWithLength_dataType(
length: NSUInteger,
data_type: MPSDataType,
) -> Retained<Self>
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn vectorDescriptorWithLength_dataType( length: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Create a MPSVectorDescriptor with the specified length and data type.
Parameter length: The number of elements in a single vector.
Parameter dataType: The type of the data to be stored in the vector.
Use this function for creating a descriptor of a MPSVector object containing a single vector.
Sourcepub unsafe fn vectorDescriptorWithLength_vectors_vectorBytes_dataType(
length: NSUInteger,
vectors: NSUInteger,
vector_bytes: NSUInteger,
data_type: MPSDataType,
) -> Retained<Self>
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn vectorDescriptorWithLength_vectors_vectorBytes_dataType( length: NSUInteger, vectors: NSUInteger, vector_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Create a MPSVectorDescriptor with the specified length and data type.
Parameter length: The number of elements in a single vector.
Parameter vectors: The number of vectors in the MPSVector object.
Parameter vectorBytes: The number of bytes between starting elements of consecutive
vectors.
Parameter dataType: The type of the data to be stored in the vector.
For performance considerations the optimal stride between vectors may not necessarily be equal to the vector length. The MPSVectorDescriptor class provides a method which may be used to determine this value, see the vectorBytesForLength API.
Sourcepub unsafe fn vectorBytesForLength_dataType(
length: NSUInteger,
data_type: MPSDataType,
) -> usize
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn vectorBytesForLength_dataType( length: NSUInteger, data_type: MPSDataType, ) -> usize
MPSCore and MPSCoreTypes only.Return the recommended stride, in bytes, to be used for an array of vectors of a given length.
Parameter length: The number of elements in a single vector.
Parameter dataType: The type of vector data values.
To achieve best performance the optimal stride between vectors within an array of vectors is not necessarily equivalent to the number of elements per vector. This method returns the stride, in bytes, which gives best performance for a given vector length. Using this stride to construct your array is recommended, but not required (provided that the stride used is still large enough to allocate a full vector of data).
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl AsRef<AnyObject> for MPSVectorDescriptor
MPSCore only.Sourceimpl AsRef<MPSVectorDescriptor> for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl AsRef<MPSVectorDescriptor> for MPSVectorDescriptor
MPSCore only.Sourceimpl AsRef<NSObject> for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl AsRef<NSObject> for MPSVectorDescriptor
MPSCore only.Sourceimpl Borrow<AnyObject> for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl Borrow<AnyObject> for MPSVectorDescriptor
MPSCore only.Sourceimpl Borrow<NSObject> for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl Borrow<NSObject> for MPSVectorDescriptor
MPSCore only.Sourceimpl ClassType for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl ClassType for MPSVectorDescriptor
MPSCore only.Sourceconst NAME: &'static str = "MPSVectorDescriptor"
const NAME: &'static str = "MPSVectorDescriptor"
Sourcetype ThreadKind = <<MPSVectorDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSVectorDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl Debug for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl Debug for MPSVectorDescriptor
MPSCore only.Sourceimpl Deref for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl Deref for MPSVectorDescriptor
MPSCore only.Sourceimpl Hash for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl Hash for MPSVectorDescriptor
MPSCore only.Sourceimpl Message for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl Message for MPSVectorDescriptor
MPSCore only.Sourceimpl NSObjectProtocol for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl NSObjectProtocol for MPSVectorDescriptor
MPSCore only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl PartialEq for MPSVectorDescriptor
MPSCore only.Sourceimpl RefEncode for MPSVectorDescriptor
Available on crate feature MPSCore only.
impl RefEncode for MPSVectorDescriptor
MPSCore only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSVectorDescriptor
MPSCore only.impl Eq for MPSVectorDescriptor
MPSCore only.