pub struct MPSCNNLossDescriptor { /* private fields */ }MPSCNNLoss only.Expand description
Dependencies: This depends on Metal.framework.
The MPSCNNLossDescriptor specifies a loss filter descriptor. The same descriptor can be used to initialize both the MPSCNNLoss and the MPSNNLossGradient filters.
See also Apple’s documentation
Implementations
Sourceimpl MPSCNNLossDescriptor
impl MPSCNNLossDescriptor
Sourcepub unsafe fn lossType(&self) -> MPSCNNLossType
Available on crate features MPSNeuralNetwork and MPSCNNTypes only.
pub unsafe fn lossType(&self) -> MPSCNNLossType
MPSNeuralNetwork and MPSCNNTypes only.The type of a loss filter.
This parameter specifies the type of a loss filter.
Sourcepub unsafe fn setLossType(&self, loss_type: MPSCNNLossType)
Available on crate features MPSNeuralNetwork and MPSCNNTypes only.
pub unsafe fn setLossType(&self, loss_type: MPSCNNLossType)
MPSNeuralNetwork and MPSCNNTypes only.Setter for lossType.
Sourcepub unsafe fn reductionType(&self) -> MPSCNNReductionType
Available on crate features MPSNeuralNetwork and MPSCNNTypes only.
pub unsafe fn reductionType(&self) -> MPSCNNReductionType
MPSNeuralNetwork and MPSCNNTypes only.The type of a reduction operation performed in the loss filter.
This parameter specifies the type of a reduction operation performed in the loss filter.
Sourcepub unsafe fn setReductionType(&self, reduction_type: MPSCNNReductionType)
Available on crate features MPSNeuralNetwork and MPSCNNTypes only.
pub unsafe fn setReductionType(&self, reduction_type: MPSCNNReductionType)
MPSNeuralNetwork and MPSCNNTypes only.Setter for reductionType.
Sourcepub unsafe fn reduceAcrossBatch(&self) -> bool
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn reduceAcrossBatch(&self) -> bool
MPSNeuralNetwork only.If set to YES then the reduction operation is applied also across the batch-index dimension, ie. the loss value is summed over images in the batch and the result of the reduction is written on the first loss image in the batch while the other loss images will be set to zero. If set to NO, then no reductions are performed across the batch dimension and each image in the batch will contain the loss value associated with that one particular image. NOTE: If reductionType == MPSCNNReductionTypeNone, then this flag has no effect on results, that is no reductions are done in this case. NOTE: If reduceAcrossBatch is set to YES and reductionType == MPSCNNReductionTypeMean then the final forward loss value is computed by first summing over the components and then by dividing the result with: number of feature channels * width * height * number of images in the batch. The default value is NO.
Sourcepub unsafe fn setReduceAcrossBatch(&self, reduce_across_batch: bool)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setReduceAcrossBatch(&self, reduce_across_batch: bool)
MPSNeuralNetwork only.Setter for reduceAcrossBatch.
Sourcepub unsafe fn weight(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn weight(&self) -> c_float
MPSNeuralNetwork only.The scale factor to apply to each element of a result.
Each element of a result is multiplied by the weight value. The default value is 1.0f.
Sourcepub unsafe fn setWeight(&self, weight: c_float)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setWeight(&self, weight: c_float)
MPSNeuralNetwork only.Setter for weight.
Sourcepub unsafe fn labelSmoothing(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn labelSmoothing(&self) -> c_float
MPSNeuralNetwork only.The label smoothing parameter. The default value is 0.0f.
This parameter is valid only for the loss functions of the following type(s): MPSCNNLossFunctionTypeSoftmaxCrossEntropy, MPSCNNLossFunctionTypeSigmoidCrossEntropy.
MPSCNNLossFunctionTypeSoftmaxCrossEntropy: given labels (ground truth), it is applied in the following way: labels = labelSmoothing > 0 ? labels * (1 - labelSmoothing) + labelSmoothing / numberOfClasses : labels
MPSCNNLossFunctionTypeSigmoidCrossEntropy: given labels (ground truth), it is applied in the following way: labels = labelSmoothing > 0 ? labels * (1 - labelSmoothing) + 0.5 * labelSmoothing : labels
Sourcepub unsafe fn setLabelSmoothing(&self, label_smoothing: c_float)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setLabelSmoothing(&self, label_smoothing: c_float)
MPSNeuralNetwork only.Setter for labelSmoothing.
Sourcepub unsafe fn numberOfClasses(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn numberOfClasses(&self) -> NSUInteger
MPSNeuralNetwork only.The number of classes parameter. The default value is 1.
This parameter is valid only for the loss functions of the following type(s): MPSCNNLossFunctionTypeSoftmaxCrossEntropy.
Given labels (ground truth), it is applied in the following way: labels = labelSmoothing > 0 ? labels * (1 - labelSmoothing) + labelSmoothing / numberOfClasses : labels
Sourcepub unsafe fn setNumberOfClasses(&self, number_of_classes: NSUInteger)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setNumberOfClasses(&self, number_of_classes: NSUInteger)
MPSNeuralNetwork only.Setter for numberOfClasses.
Sourcepub unsafe fn epsilon(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn epsilon(&self) -> c_float
MPSNeuralNetwork only.The epsilon parameter. The default value is 1e-7.
This parameter is valid only for the loss functions of the following type(s): MPSCNNLossTypeLog.
Given predictions and labels (ground truth), it is applied in the following way: -(labels * log(predictions + epsilon)) - ((1 - labels) * log(1 - predictions + epsilon))
Sourcepub unsafe fn setEpsilon(&self, epsilon: c_float)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setEpsilon(&self, epsilon: c_float)
MPSNeuralNetwork only.Setter for epsilon.
Sourcepub unsafe fn delta(&self) -> c_float
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn delta(&self) -> c_float
MPSNeuralNetwork only.The delta parameter. The default value is 1.0f.
This parameter is valid only for the loss functions of the following type(s): MPSCNNLossTypeHuber.
Given predictions and labels (ground truth), it is applied in the following way: if (|predictions - labels| < = delta, loss = 0.5f * predictions^2 if (|predictions - labels| > delta, loss = 0.5 * delta^2 + delta * (|predictions - labels| - delta)
Sourcepub unsafe fn setDelta(&self, delta: c_float)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setDelta(&self, delta: c_float)
MPSNeuralNetwork only.Setter for delta.
pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>
MPSNeuralNetwork only.Sourcepub unsafe fn cnnLossDescriptorWithType_reductionType(
loss_type: MPSCNNLossType,
reduction_type: MPSCNNReductionType,
) -> Retained<MPSCNNLossDescriptor>
Available on crate features MPSNeuralNetwork and MPSCNNTypes only.
pub unsafe fn cnnLossDescriptorWithType_reductionType( loss_type: MPSCNNLossType, reduction_type: MPSCNNReductionType, ) -> Retained<MPSCNNLossDescriptor>
MPSNeuralNetwork and MPSCNNTypes only.Make a descriptor for a MPSCNNLoss or MPSNNLossGradient object.
Parameter lossType: The type of a loss filter.
Parameter reductionType: The type of a reduction operation to apply.
This argument is ignored in the MPSNNLossGradient filter.
Returns: A valid MPSCNNLossDescriptor object or nil, if failure.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl AsRef<AnyObject> for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl AsRef<MPSCNNLossDescriptor> for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSCNNLossDescriptor> for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl AsRef<NSObject> for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl AsRef<NSObject> for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl Borrow<AnyObject> for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Borrow<AnyObject> for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl Borrow<NSObject> for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Borrow<NSObject> for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl ClassType for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl ClassType for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceconst NAME: &'static str = "MPSCNNLossDescriptor"
const NAME: &'static str = "MPSCNNLossDescriptor"
Sourcetype ThreadKind = <<MPSCNNLossDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSCNNLossDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl CopyingHelper for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourcetype Result = MPSCNNLossDescriptor
type Result = MPSCNNLossDescriptor
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Debug for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl Deref for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Deref for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl Hash for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Hash for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl Message for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl Message for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl NSCopying for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl NSCopying for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl NSObjectProtocol for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl NSObjectProtocol for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl PartialEq for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceimpl RefEncode for MPSCNNLossDescriptor
Available on crate feature MPSNeuralNetwork only.
impl RefEncode for MPSCNNLossDescriptor
MPSNeuralNetwork only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSCNNLossDescriptor
MPSNeuralNetwork only.impl Eq for MPSCNNLossDescriptor
MPSNeuralNetwork only.