pub struct MPSImageDescriptor { /* private fields */ }MPSImage only.Expand description
Dependencies: This depends on Metal.framework
A MPSImageDescriptor object describes a attributes of MPSImage and is used to create one (see MPSImage discussion below)
See also Apple’s documentation
Implementations
Sourceimpl MPSImageDescriptor
impl MPSImageDescriptor
Sourcepub unsafe fn width(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn width(&self) -> NSUInteger
MPSCore only.The width of the CNN image.
The formal width of the CNN image in pixels. Default = 1.
Sourcepub unsafe fn setWidth(&self, width: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setWidth(&self, width: NSUInteger)
MPSCore only.Setter for width.
Sourcepub unsafe fn height(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn height(&self) -> NSUInteger
MPSCore only.The height of the CNN image.
The formal height of the CNN image in pixels. Default = 1.
Sourcepub unsafe fn setHeight(&self, height: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setHeight(&self, height: NSUInteger)
MPSCore only.Setter for height.
Sourcepub unsafe fn featureChannels(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn featureChannels(&self) -> NSUInteger
MPSCore only.The number of feature channels per pixel. Default = 1.
Sourcepub unsafe fn setFeatureChannels(&self, feature_channels: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setFeatureChannels(&self, feature_channels: NSUInteger)
MPSCore only.Setter for featureChannels.
Sourcepub unsafe fn numberOfImages(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn numberOfImages(&self) -> NSUInteger
MPSCore only.The number of images for batch processing. Default = 1.
Sourcepub unsafe fn setNumberOfImages(&self, number_of_images: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setNumberOfImages(&self, number_of_images: NSUInteger)
MPSCore only.Setter for numberOfImages.
Sourcepub unsafe fn pixelFormat(&self) -> MTLPixelFormat
Available on crate feature MPSCore only.
pub unsafe fn pixelFormat(&self) -> MTLPixelFormat
MPSCore only.The MTLPixelFormat expected for the underlying texture.
Sourcepub unsafe fn channelFormat(&self) -> MPSImageFeatureChannelFormat
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn channelFormat(&self) -> MPSImageFeatureChannelFormat
MPSCore and MPSCoreTypes only.The storage format to use for each channel in the image.
Sourcepub unsafe fn setChannelFormat(
&self,
channel_format: MPSImageFeatureChannelFormat,
)
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn setChannelFormat( &self, channel_format: MPSImageFeatureChannelFormat, )
MPSCore and MPSCoreTypes only.Setter for channelFormat.
Sourcepub unsafe fn cpuCacheMode(&self) -> MTLCPUCacheMode
Available on crate feature MPSCore only.
pub unsafe fn cpuCacheMode(&self) -> MTLCPUCacheMode
MPSCore only.Options to specify CPU cache mode of texture resource. Default = MTLCPUCacheModeDefaultCache
Sourcepub unsafe fn setCpuCacheMode(&self, cpu_cache_mode: MTLCPUCacheMode)
Available on crate feature MPSCore only.
pub unsafe fn setCpuCacheMode(&self, cpu_cache_mode: MTLCPUCacheMode)
MPSCore only.Setter for cpuCacheMode.
Sourcepub unsafe fn storageMode(&self) -> MTLStorageMode
Available on crate feature MPSCore only.
pub unsafe fn storageMode(&self) -> MTLStorageMode
MPSCore only.To specify storage mode of texture resource.
Storage mode options:
Default = MTLStorageModeShared on iOS
MTLStorageModeManaged on Mac OSX
MTLStorageModeShared not supported on Mac OSX.
See Metal headers for synchronization requirements when using StorageModeManagedSourcepub unsafe fn setStorageMode(&self, storage_mode: MTLStorageMode)
Available on crate feature MPSCore only.
pub unsafe fn setStorageMode(&self, storage_mode: MTLStorageMode)
MPSCore only.Setter for storageMode.
Sourcepub unsafe fn usage(&self) -> MTLTextureUsage
Available on crate feature MPSCore only.
pub unsafe fn usage(&self) -> MTLTextureUsage
MPSCore only.Description of texture usage. Default = MTLTextureUsageShaderRead/Write
Sourcepub unsafe fn setUsage(&self, usage: MTLTextureUsage)
Available on crate feature MPSCore only.
pub unsafe fn setUsage(&self, usage: MTLTextureUsage)
MPSCore only.Setter for usage.
Sourcepub unsafe fn imageDescriptorWithChannelFormat_width_height_featureChannels(
channel_format: MPSImageFeatureChannelFormat,
width: NSUInteger,
height: NSUInteger,
feature_channels: NSUInteger,
) -> Retained<Self>
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn imageDescriptorWithChannelFormat_width_height_featureChannels( channel_format: MPSImageFeatureChannelFormat, width: NSUInteger, height: NSUInteger, feature_channels: NSUInteger, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Create a MPSImageDescriptor for a single read/write cnn image.
Sourcepub unsafe fn imageDescriptorWithChannelFormat_width_height_featureChannels_numberOfImages_usage(
channel_format: MPSImageFeatureChannelFormat,
width: NSUInteger,
height: NSUInteger,
feature_channels: NSUInteger,
number_of_images: NSUInteger,
usage: MTLTextureUsage,
) -> Retained<Self>
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn imageDescriptorWithChannelFormat_width_height_featureChannels_numberOfImages_usage( channel_format: MPSImageFeatureChannelFormat, width: NSUInteger, height: NSUInteger, feature_channels: NSUInteger, number_of_images: NSUInteger, usage: MTLTextureUsage, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Create a MPSImageDescriptor for a read/write cnn image with option to set usage and batch size (numberOfImages).
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSImageDescriptor
Available on crate feature MPSCore only.
impl AsRef<AnyObject> for MPSImageDescriptor
MPSCore only.Sourceimpl AsRef<MPSImageDescriptor> for MPSImageDescriptor
Available on crate feature MPSCore only.
impl AsRef<MPSImageDescriptor> for MPSImageDescriptor
MPSCore only.Sourceimpl AsRef<NSObject> for MPSImageDescriptor
Available on crate feature MPSCore only.
impl AsRef<NSObject> for MPSImageDescriptor
MPSCore only.Sourceimpl Borrow<AnyObject> for MPSImageDescriptor
Available on crate feature MPSCore only.
impl Borrow<AnyObject> for MPSImageDescriptor
MPSCore only.Sourceimpl Borrow<NSObject> for MPSImageDescriptor
Available on crate feature MPSCore only.
impl Borrow<NSObject> for MPSImageDescriptor
MPSCore only.Sourceimpl ClassType for MPSImageDescriptor
Available on crate feature MPSCore only.
impl ClassType for MPSImageDescriptor
MPSCore only.Sourceconst NAME: &'static str = "MPSImageDescriptor"
const NAME: &'static str = "MPSImageDescriptor"
Sourcetype ThreadKind = <<MPSImageDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSImageDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSImageDescriptor
Available on crate feature MPSCore only.
impl CopyingHelper for MPSImageDescriptor
MPSCore only.Sourcetype Result = MPSImageDescriptor
type Result = MPSImageDescriptor
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSImageDescriptor
Available on crate feature MPSCore only.
impl Debug for MPSImageDescriptor
MPSCore only.Sourceimpl Deref for MPSImageDescriptor
Available on crate feature MPSCore only.
impl Deref for MPSImageDescriptor
MPSCore only.Sourceimpl Hash for MPSImageDescriptor
Available on crate feature MPSCore only.
impl Hash for MPSImageDescriptor
MPSCore only.Sourceimpl Message for MPSImageDescriptor
Available on crate feature MPSCore only.
impl Message for MPSImageDescriptor
MPSCore only.Sourceimpl NSCopying for MPSImageDescriptor
Available on crate feature MPSCore only.
impl NSCopying for MPSImageDescriptor
MPSCore only.Sourceimpl NSObjectProtocol for MPSImageDescriptor
Available on crate feature MPSCore only.
impl NSObjectProtocol for MPSImageDescriptor
MPSCore only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSImageDescriptor
Available on crate feature MPSCore only.
impl PartialEq for MPSImageDescriptor
MPSCore only.Sourceimpl RefEncode for MPSImageDescriptor
Available on crate feature MPSCore only.
impl RefEncode for MPSImageDescriptor
MPSCore only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSImageDescriptor
MPSCore only.impl Eq for MPSImageDescriptor
MPSCore only.