#[repr(transparent)]pub struct MPSImageEdgeMode(pub NSUInteger);Available on crate feature
MPSCoreTypes only.Expand description
Tuple Fields
0: NSUIntegerImplementations
Sourceimpl MPSImageEdgeMode
impl MPSImageEdgeMode
Sourcepub const Zero: Self
Available on crate feature MPSCore only.
pub const Zero: Self
MPSCore only.Out of bound pixels are (0,0,0,1) for image with pixel format without alpha channel and (0,0,0,0) for image with pixel format that has an alpha channel
Sourcepub const Clamp: Self
Available on crate feature MPSCore only.
pub const Clamp: Self
MPSCore only.Out of bound pixels are clamped to nearest edge pixel
Sourcepub const Mirror: Self
Available on crate feature MPSCore only.
pub const Mirror: Self
MPSCore only.Out of bound pixels are mirrored wrt. the nearest edge pixel center - ie. the edge of the image is not repeated. NOTE: The only filter that currently supports this mode is MPSNNPad- using this with other filters results in undefined behavior.
Sourcepub const MirrorWithEdge: Self
Available on crate feature MPSCore only.
pub const MirrorWithEdge: Self
MPSCore only.Out of bound pixels are mirrored wrt. the nearest edge pixel nearest border - ie. the edge of the image is repeated. NOTE: The only filter that currently supports this mode is MPSNNPad- using this with other filters results in undefined behavior.