pub struct MPSMatrix { /* private fields */ }MPSMatrix only.Expand description
Dependencies: This depends on Metal.framework
A MPSMatrix object describes a set of 2-dimensional arrays of data and provides storage for its values. MPSMatrix objects serve as inputs and outputs of MPSMatrixKernel objects.
Implementation note: A MPSMatrix object maintains its internal storage using a MTLBuffer object and thus the same rules for maintaining coherency of a MTLBuffer’s data between CPU memory and GPU memory apply to a MPSMatrix. An MPSMatrix object’s data refers to an array of matrices. Data is assumed to be ordered by matrix first, followed by row, followed by column.
For example, index [i,j] of the k’th matrix of an MPSMatrix is located at byte offset: k * matrixBytes + i * rowBytes + j * sizeof(dataType)
Where matrixBytes is a multiple of rowBytes at least equal to rows * rowBytes.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrix
impl MPSMatrix
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
Available on crate feature MPSCore only.
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
MPSCore only.The device on which the MPSMatrix will be used.
Sourcepub unsafe fn rows(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn rows(&self) -> NSUInteger
MPSCore only.The number of rows in a matrix in the MPSMatrix.
Sourcepub unsafe fn columns(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn columns(&self) -> NSUInteger
MPSCore only.The number of columns in a matrix in the MPSMatrix.
Sourcepub unsafe fn matrices(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn matrices(&self) -> NSUInteger
MPSCore only.The number of matrices in the MPSMatrix.
Sourcepub unsafe fn dataType(&self) -> MPSDataType
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn dataType(&self) -> MPSDataType
MPSCore and MPSCoreTypes only.The type of the MPSMatrix data.
Sourcepub unsafe fn rowBytes(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn rowBytes(&self) -> NSUInteger
MPSCore only.The stride, in bytes, between corresponding elements of consecutive rows.
Sourcepub unsafe fn matrixBytes(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn matrixBytes(&self) -> NSUInteger
MPSCore only.The stride, in bytes, between corresponding elements of consecutive matrices.
Sourcepub unsafe fn offset(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn offset(&self) -> NSUInteger
MPSCore only.Byte-offset to the buffer where the matrix data begins - see initWithBuffer:offset: descriptor: .
Sourcepub unsafe fn data(&self) -> Retained<ProtocolObject<dyn MTLBuffer>>
Available on crate feature MPSCore only.
pub unsafe fn data(&self) -> Retained<ProtocolObject<dyn MTLBuffer>>
MPSCore only.An MTLBuffer to store the data.
Sourcepub unsafe fn initWithBuffer_descriptor(
this: Allocated<Self>,
buffer: &ProtocolObject<dyn MTLBuffer>,
descriptor: &MPSMatrixDescriptor,
) -> Retained<Self>
Available on crate feature MPSCore only.
pub unsafe fn initWithBuffer_descriptor( this: Allocated<Self>, buffer: &ProtocolObject<dyn MTLBuffer>, descriptor: &MPSMatrixDescriptor, ) -> Retained<Self>
MPSCore only.Initialize a MPSMatrix object with a MTLBuffer.
Parameter buffer: The MTLBuffer object which contains the data to use for the
MPSMatrix. May not be NULL.
Parameter descriptor: The MPSMatrixDescriptor. May not be NULL.
Returns: A valid MPSMatrix object or nil, if failure.
This function returns a MPSMatrix object which uses the supplied MTLBuffer. The dimensions and stride of the matrix are specified by the MPSMatrixDescriptor object.
The provided MTLBuffer must have enough storage to hold
(descriptor.matrices-1) * descriptor.matrixBytes + (descriptor.rows-1) * descriptor.rowBytes + descriptor.columns * (element size) bytes.
Safety
buffermay need to be synchronized.buffermay be unretained, you must ensure it is kept alive while in use.buffercontents should be of the correct type.
Sourcepub unsafe fn initWithBuffer_offset_descriptor(
this: Allocated<Self>,
buffer: &ProtocolObject<dyn MTLBuffer>,
offset: NSUInteger,
descriptor: &MPSMatrixDescriptor,
) -> Retained<Self>
Available on crate feature MPSCore only.
pub unsafe fn initWithBuffer_offset_descriptor( this: Allocated<Self>, buffer: &ProtocolObject<dyn MTLBuffer>, offset: NSUInteger, descriptor: &MPSMatrixDescriptor, ) -> Retained<Self>
MPSCore only.Initialize a MPSMatrix object with a MTLBuffer at a given offset.
Parameter buffer: The MTLBuffer object which contains the data to use for the
MPSMatrix. May not be NULL.
Parameter offset: The offset, in bytes, into the buffer at which the data begins.
Parameter descriptor: The MPSMatrixDescriptor describing the shape of the matrix.
Safety
buffermay need to be synchronized.buffermay be unretained, you must ensure it is kept alive while in use.buffercontents should be of the correct type.
Sourcepub unsafe fn initWithDevice_descriptor(
this: Allocated<Self>,
device: &ProtocolObject<dyn MTLDevice>,
descriptor: &MPSMatrixDescriptor,
) -> Retained<Self>
Available on crate feature MPSCore only.
pub unsafe fn initWithDevice_descriptor( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, descriptor: &MPSMatrixDescriptor, ) -> Retained<Self>
MPSCore only.Initialize a MPSMatrix object with a descriptor. Allocate the buffer.
Parameter device: The device with which it will be used
Parameter descriptor: The shape and style of the matrix
Returns: A valid MPSMatrix object or nil
The matrix object will be created, but the storage to hold the matrix data will only be allocated when it is needed, typically when the data property is invoked. In conjunction with -resourceSize, this will allow you to estimate storage needs without actually creating the backing store for the matrix.
pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>
MPSCore only.Sourcepub unsafe fn synchronizeOnCommandBuffer(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
)
Available on crate feature MPSCore only.
pub unsafe fn synchronizeOnCommandBuffer( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, )
MPSCore only.Flush the underlying MTLBuffer from the device’s caches, and invalidate any CPU caches if needed.
This will call [id <MTLBlitEncoder
synchronizeResource: ] on the matrix’s MTLBuffer, if any. This is necessary for all MTLStorageModeManaged resources. For other resources, including temporary resources (these are all MTLStorageModePrivate), and buffers that have not yet been allocated, nothing is done. It is more efficient to use this method than to attempt to do this yourself with the data property.
Parameter commandBuffer: The commandbuffer on which to synchronize
Sourcepub unsafe fn resourceSize(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn resourceSize(&self) -> NSUInteger
MPSCore only.Get the number of bytes used to allocate underyling MTLResources
This is the size of the backing store of underlying MTLResources. It does not include all storage used by the object, for example the storage used to hold the MPSMatrix instantiation and MTLBuffer is not included. It only measures the size of the allocation used to hold the matrix data in the buffer. This value is subject to change between different devices and operating systems.
Except when -initWithBuffer:descriptor: is used, most MPSMatrixes are allocated without a backing store. The backing store is allocated lazily when it is needed, typically when the .texture property is called. Consequently, in most cases, it should be inexpensive to make a MPSImage to see how much memory it will need, and release it if it is too large.
This method may fail in certain circumstances, such as when the MPSImage is created with -initWithTexture:featureChannels:. In such cases, 0 will be returned.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<MPSMatrix> for MPSTemporaryMatrix
Available on crate feature MPSCore only.
impl AsRef<MPSMatrix> for MPSTemporaryMatrix
MPSCore only.Sourceimpl Borrow<MPSMatrix> for MPSTemporaryMatrix
Available on crate feature MPSCore only.
impl Borrow<MPSMatrix> for MPSTemporaryMatrix
MPSCore only.Sourceimpl ClassType for MPSMatrix
Available on crate feature MPSCore only.
impl ClassType for MPSMatrix
MPSCore only.Sourceconst NAME: &'static str = "MPSMatrix"
const NAME: &'static str = "MPSMatrix"
Sourcetype ThreadKind = <<MPSMatrix as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrix as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl NSObjectProtocol for MPSMatrix
Available on crate feature MPSCore only.
impl NSObjectProtocol for MPSMatrix
MPSCore only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl RefEncode for MPSMatrix
Available on crate feature MPSCore only.
impl RefEncode for MPSMatrix
MPSCore only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrix
MPSCore only.impl Eq for MPSMatrix
MPSCore only.