pub struct MPSMatrixDescriptor { /* private fields */ }MPSMatrix only.Expand description
Dependencies: This depends on Metal.framework
A MPSMatrixDescriptor describes the sizes, strides, and data type of a an array of 2-dimensional matrices. All storage is assumed to be in “matrix-major”. See the description for MPSMatrix for further details.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixDescriptor
impl MPSMatrixDescriptor
Sourcepub unsafe fn rows(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn rows(&self) -> NSUInteger
MPSCore only.The number of rows in a matrix.
Sourcepub unsafe fn setRows(&self, rows: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setRows(&self, rows: NSUInteger)
MPSCore only.Setter for rows.
Sourcepub unsafe fn columns(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn columns(&self) -> NSUInteger
MPSCore only.The number of columns in a matrix.
Sourcepub unsafe fn setColumns(&self, columns: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setColumns(&self, columns: NSUInteger)
MPSCore only.Setter for columns.
Sourcepub unsafe fn matrices(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn matrices(&self) -> NSUInteger
MPSCore only.The number of matrices.
Sourcepub unsafe fn dataType(&self) -> MPSDataType
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn dataType(&self) -> MPSDataType
MPSCore and MPSCoreTypes only.The type of the data which makes up the values of the matrix.
Sourcepub unsafe fn setDataType(&self, data_type: MPSDataType)
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn setDataType(&self, data_type: MPSDataType)
MPSCore and MPSCoreTypes only.Setter for dataType.
Sourcepub unsafe fn rowBytes(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn rowBytes(&self) -> NSUInteger
MPSCore only.The stride, in bytes, between corresponding elements of consecutive rows. Must be a multiple of the element size.
Sourcepub unsafe fn setRowBytes(&self, row_bytes: NSUInteger)
Available on crate feature MPSCore only.
pub unsafe fn setRowBytes(&self, row_bytes: NSUInteger)
MPSCore only.Setter for rowBytes.
Sourcepub unsafe fn matrixBytes(&self) -> NSUInteger
Available on crate feature MPSCore only.
pub unsafe fn matrixBytes(&self) -> NSUInteger
MPSCore only.The stride, in bytes, between corresponding elements of consecutive matrices. Must be a multiple of rowBytes.
Sourcepub unsafe fn matrixDescriptorWithDimensions_columns_rowBytes_dataType(
rows: NSUInteger,
columns: NSUInteger,
row_bytes: NSUInteger,
data_type: MPSDataType,
) -> Retained<Self>
👎DeprecatedAvailable on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn matrixDescriptorWithDimensions_columns_rowBytes_dataType( rows: NSUInteger, columns: NSUInteger, row_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Create a MPSMatrixDescriptor with the specified dimensions and data type.
Parameter rows: The number of rows of the matrix.
Parameter columns: The number of columns of the matrix.
Parameter rowBytes: The number of bytes between starting elements of consecutive
rows. Must be a multiple of the element size.
Parameter dataType: The type of the data to be stored in the matrix.
For performance considerations the optimal row stride may not necessarily be equal to the number of columns in the matrix. The MPSMatrix class provides a method which may be used to determine this value, see the rowBytesForColumns API in the MPSMatrix class. The number of matrices described is initialized to 1.
pub unsafe fn matrixDescriptorWithRows_columns_rowBytes_dataType( rows: NSUInteger, columns: NSUInteger, row_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Sourcepub unsafe fn matrixDescriptorWithRows_columns_matrices_rowBytes_matrixBytes_dataType(
rows: NSUInteger,
columns: NSUInteger,
matrices: NSUInteger,
row_bytes: NSUInteger,
matrix_bytes: NSUInteger,
data_type: MPSDataType,
) -> Retained<Self>
Available on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn matrixDescriptorWithRows_columns_matrices_rowBytes_matrixBytes_dataType( rows: NSUInteger, columns: NSUInteger, matrices: NSUInteger, row_bytes: NSUInteger, matrix_bytes: NSUInteger, data_type: MPSDataType, ) -> Retained<Self>
MPSCore and MPSCoreTypes only.Create a MPSMatrixDescriptor with the specified dimensions and data type.
Parameter rows: The number of rows of a single matrix.
Parameter columns: The number of columns of a single matrix.
Parameter matrices: The number of matrices in the MPSMatrix object.
Parameter rowBytes: The number of bytes between starting elements of consecutive
rows. Must be a multiple of the element size.
Parameter matrixBytes: The number of bytes between starting elements of consecutive
matrices. Must be a multiple of rowBytes.
Parameter dataType: The type of the data to be stored in the matrix.
For performance considerations the optimal row stride may not necessarily be equal to the number of columns in the matrix. The MPSMatrix class provides a method which may be used to determine this value, see the rowBytesForColumns API in the MPSMatrix class.
Sourcepub unsafe fn rowBytesFromColumns_dataType(
columns: NSUInteger,
data_type: MPSDataType,
) -> usize
👎DeprecatedAvailable on crate features MPSCore and MPSCoreTypes only.
pub unsafe fn rowBytesFromColumns_dataType( columns: NSUInteger, data_type: MPSDataType, ) -> usize
MPSCore and MPSCoreTypes only.Return the recommended row stride, in bytes, for a given number of columns.
Parameter columns: The number of columns in the matrix for which the recommended
row stride, in bytes, is to be determined.
Parameter dataType: The type of matrix data values.
To achieve best performance the optimal stride between rows of a matrix is not necessarily equivalent to the number of columns. This method returns the row stride, in bytes, which gives best performance for a given number of columns. Using this row stride to construct your array is recommended, but not required (provided that the stride used is still large enough to allocate a full row of data).
pub unsafe fn rowBytesForColumns_dataType( columns: NSUInteger, data_type: MPSDataType, ) -> usize
MPSCore and MPSCoreTypes only.Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl AsRef<AnyObject> for MPSMatrixDescriptor
MPSCore only.Sourceimpl AsRef<MPSMatrixDescriptor> for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl AsRef<MPSMatrixDescriptor> for MPSMatrixDescriptor
MPSCore only.Sourceimpl AsRef<NSObject> for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl AsRef<NSObject> for MPSMatrixDescriptor
MPSCore only.Sourceimpl Borrow<AnyObject> for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl Borrow<AnyObject> for MPSMatrixDescriptor
MPSCore only.Sourceimpl Borrow<NSObject> for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl Borrow<NSObject> for MPSMatrixDescriptor
MPSCore only.Sourceimpl ClassType for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl ClassType for MPSMatrixDescriptor
MPSCore only.Sourceconst NAME: &'static str = "MPSMatrixDescriptor"
const NAME: &'static str = "MPSMatrixDescriptor"
Sourcetype ThreadKind = <<MPSMatrixDescriptor as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixDescriptor as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl Debug for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl Debug for MPSMatrixDescriptor
MPSCore only.Sourceimpl Deref for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl Deref for MPSMatrixDescriptor
MPSCore only.Sourceimpl Hash for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl Hash for MPSMatrixDescriptor
MPSCore only.Sourceimpl Message for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl Message for MPSMatrixDescriptor
MPSCore only.Sourceimpl NSObjectProtocol for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl NSObjectProtocol for MPSMatrixDescriptor
MPSCore only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl PartialEq for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl PartialEq for MPSMatrixDescriptor
MPSCore only.Sourceimpl RefEncode for MPSMatrixDescriptor
Available on crate feature MPSCore only.
impl RefEncode for MPSMatrixDescriptor
MPSCore only.Sourceconst ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <NSObject as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixDescriptor
MPSCore only.impl Eq for MPSMatrixDescriptor
MPSCore only.