pub struct MPSMatrixFullyConnectedGradient { /* private fields */ }MPSCore and MPSKernel and MPSMatrix and MPSMatrixFullyConnected and MPSMatrixTypes only.Expand description
Dependencies: This depends on Metal.framework.
Computes the gradient of the fully connected layer with respect to either the weights and bias terms or the input feature vectors.
An MPSMatrixFullyConnectedGradient kernel may be used to compute the gradients corresponding to a MPSMatrixFullyConnected kernel. The properties, input, and weight data must match those values used in the forward computation. This kernel does not compute the gradient of any non-identity activation function which may have been applied in the forward kernel. Such a kernel must be expressed using both MPSMatrixFullyConnected and MPSMatrixNeuron if a gradient is to be computed.
See also Apple’s documentation
Implementations
Sourceimpl MPSMatrixFullyConnectedGradient
impl MPSMatrixFullyConnectedGradient
Sourcepub unsafe fn sourceNumberOfFeatureVectors(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceNumberOfFeatureVectors(&self) -> NSUInteger
MPSNeuralNetwork only.The number of input vectors which make up the input array. This is equivalent to the number of rows in both the input matrix and the source gradient matrix.
This value should be equal to the corresponding value in the forward fully connected kernel.
Sourcepub unsafe fn setSourceNumberOfFeatureVectors(
&self,
source_number_of_feature_vectors: NSUInteger,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSourceNumberOfFeatureVectors( &self, source_number_of_feature_vectors: NSUInteger, )
MPSNeuralNetwork only.Setter for sourceNumberOfFeatureVectors.
Sourcepub unsafe fn sourceOutputFeatureChannels(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceOutputFeatureChannels(&self) -> NSUInteger
MPSNeuralNetwork only.The number of feature channels in the output of the forward fully connected layer. This is equivalent to the number of columns in both the weight matrix and the source gradient matrix.
This value should be equal to the corresponding value in the forward fully connected kernel.
Sourcepub unsafe fn setSourceOutputFeatureChannels(
&self,
source_output_feature_channels: NSUInteger,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSourceOutputFeatureChannels( &self, source_output_feature_channels: NSUInteger, )
MPSNeuralNetwork only.Setter for sourceOutputFeatureChannels.
Sourcepub unsafe fn sourceInputFeatureChannels(&self) -> NSUInteger
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn sourceInputFeatureChannels(&self) -> NSUInteger
MPSNeuralNetwork only.The number of feature channels in the input to the forward fully connected layer. This is equivalent to the number of columns in the input matrix.
This value should be equal to the corresponding value in the forward fully connected kernel.
Sourcepub unsafe fn setSourceInputFeatureChannels(
&self,
source_input_feature_channels: NSUInteger,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setSourceInputFeatureChannels( &self, source_input_feature_channels: NSUInteger, )
MPSNeuralNetwork only.Setter for sourceInputFeatureChannels.
Sourcepub unsafe fn alpha(&self) -> c_double
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn alpha(&self) -> c_double
MPSNeuralNetwork only.Scale factor to apply to the product. This value should be equal to the corresponding value in the forward fully connected kernel.
Sourcepub unsafe fn setAlpha(&self, alpha: c_double)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn setAlpha(&self, alpha: c_double)
MPSNeuralNetwork only.Setter for alpha.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
MPSNeuralNetwork only.Sourcepub unsafe fn encodeGradientForDataToCommandBuffer_gradientMatrix_weightMatrix_resultGradientForDataMatrix(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
gradient_matrix: &MPSMatrix,
weight_matrix: &MPSMatrix,
result_gradient_for_data_matrix: &MPSMatrix,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeGradientForDataToCommandBuffer_gradientMatrix_weightMatrix_resultGradientForDataMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, gradient_matrix: &MPSMatrix, weight_matrix: &MPSMatrix, result_gradient_for_data_matrix: &MPSMatrix, )
MPSNeuralNetwork only.Encode a MPSMatrixFullyConnectedGradient object to a command buffer and produce the gradient of the loss function with respect to the input data.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.
Parameter gradientMatrix: A valid MPSMatrix object which specifies the input gradient.
Parameter weightMatrix: A valid MPSMatrix object which specifies the weight array.
Parameter resultGradientForDataMatrix: A valid MPSMatrix object which specifies the result gradient.
This operation computes the resulting gradient of the loss function with respect to the forward kernel’s input data. weightMatrix should contain the same values used to compute the result of the forward kernel.
Sourcepub unsafe fn encodeGradientForWeightsAndBiasToCommandBuffer_gradientMatrix_inputMatrix_resultGradientForWeightMatrix_resultGradientForBiasVector(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
gradient_matrix: &MPSMatrix,
input_matrix: &MPSMatrix,
result_gradient_for_weight_matrix: &MPSMatrix,
result_gradient_for_bias_vector: Option<&MPSVector>,
)
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn encodeGradientForWeightsAndBiasToCommandBuffer_gradientMatrix_inputMatrix_resultGradientForWeightMatrix_resultGradientForBiasVector( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, gradient_matrix: &MPSMatrix, input_matrix: &MPSMatrix, result_gradient_for_weight_matrix: &MPSMatrix, result_gradient_for_bias_vector: Option<&MPSVector>, )
MPSNeuralNetwork only.Encode a MPSMatrixFullyConnectedGradient object to a command buffer and produce the gradient of the loss function with respect to the weight matrix and bias vector.
Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded kernel.
Parameter gradientMatrix: A valid MPSMatrix object which specifies the input gradient.
Parameter inputMatrix: A valid MPSMatrix object which specifies the input array.
Parameter resultGradientForWeightMatrix: A valid MPSMatrix object which specifies the resulting gradients
with respect to the weights.
Parameter resultGradientForBiasVector: A valid MPSVector object which specifies the resulting gradients
with respect to the bias terms. If NULL these values will not be
returned.
This operation computes the resulting gradient of the loss function with respect to the forward kernel’s weight data. inputMatrix should contain the same values used to compute the result of the forward kernel.
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
MPSNeuralNetwork only.NSSecureCoding compatability
See
MPSKernel#initWithCoder.
Parameter aDecoder: The NSCoder subclass with your serialized MPSMatrixFullyConnectedGradient
Parameter device: The MTLDevice on which to make the MPSMatrixFullyConnectedGradient object.
Returns: A new MPSMatrixFullyConnected object, or nil if failure.
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
MPSNeuralNetwork only.Make a copy of this kernel for a new device -
See: MPSKernel
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: A pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Sourceimpl MPSMatrixFullyConnectedGradient
Methods declared on superclass MPSKernel.
impl MPSMatrixFullyConnectedGradient
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
Available on crate feature MPSNeuralNetwork only.
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
MPSNeuralNetwork only.Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSMatrixBinaryKernel>
Sourcepub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
pub unsafe fn primarySourceMatrixOrigin(&self) -> MTLOrigin
The origin, relative to [0, 0] in the primary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setPrimarySourceMatrixOrigin(
&self,
primary_source_matrix_origin: MTLOrigin,
)
pub unsafe fn setPrimarySourceMatrixOrigin( &self, primary_source_matrix_origin: MTLOrigin, )
Setter for primarySourceMatrixOrigin.
Sourcepub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
pub unsafe fn secondarySourceMatrixOrigin(&self) -> MTLOrigin
The origin, relative to [0, 0] in the secondary source matrix, at which to start reading values. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setSecondarySourceMatrixOrigin(
&self,
secondary_source_matrix_origin: MTLOrigin,
)
pub unsafe fn setSecondarySourceMatrixOrigin( &self, secondary_source_matrix_origin: MTLOrigin, )
Setter for secondarySourceMatrixOrigin.
Sourcepub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
pub unsafe fn resultMatrixOrigin(&self) -> MTLOrigin
The origin, relative to [0, 0] in the result matrix, at which to start writing results. This property is modifiable and defaults to [0, 0] at initialization time. If a different origin is desired then this should be modified prior to encoding the kernel. The z value must be 0.
Sourcepub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
pub unsafe fn setResultMatrixOrigin(&self, result_matrix_origin: MTLOrigin)
Setter for resultMatrixOrigin.
Sourcepub unsafe fn batchStart(&self) -> NSUInteger
pub unsafe fn batchStart(&self) -> NSUInteger
The index of the first matrix in the batch. This property is modifiable and defaults to 0 at initialization time. If batch processing should begin at a different matrix this value should be modified prior to encoding the kernel.
Sourcepub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)
Setter for batchStart.
Sourcepub unsafe fn batchSize(&self) -> NSUInteger
pub unsafe fn batchSize(&self) -> NSUInteger
The number of matrices in the batch to process. This property is modifiable and by default allows all matrices available at encoding time to be processed. If a single matrix should be processed set this value to 1.
Sourcepub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)
Setter for batchSize.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
Ivar::load instead.Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<AnyObject> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl AsRef<MPSKernel> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSKernel> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl AsRef<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourcefn as_ref(&self) -> &MPSMatrixBinaryKernel
fn as_ref(&self) -> &MPSMatrixBinaryKernel
Sourceimpl AsRef<MPSMatrixFullyConnectedGradient> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<MPSMatrixFullyConnectedGradient> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl AsRef<NSObject> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl AsRef<NSObject> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl Borrow<AnyObject> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<AnyObject> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl Borrow<MPSKernel> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSKernel> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl Borrow<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<MPSMatrixBinaryKernel> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourcefn borrow(&self) -> &MPSMatrixBinaryKernel
fn borrow(&self) -> &MPSMatrixBinaryKernel
Sourceimpl Borrow<NSObject> for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Borrow<NSObject> for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl ClassType for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl ClassType for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceconst NAME: &'static str = "MPSMatrixFullyConnectedGradient"
const NAME: &'static str = "MPSMatrixFullyConnectedGradient"
Sourcetype Super = MPSMatrixBinaryKernel
type Super = MPSMatrixBinaryKernel
Sourcetype ThreadKind = <<MPSMatrixFullyConnectedGradient as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSMatrixFullyConnectedGradient as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl CopyingHelper for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourcetype Result = MPSMatrixFullyConnectedGradient
type Result = MPSMatrixFullyConnectedGradient
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Debug for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl Deref for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Deref for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl Hash for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Hash for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl Message for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl Message for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl NSCoding for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl NSCoding for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl NSCopying for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl NSCopying for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl NSObjectProtocol for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl NSObjectProtocol for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
isKindOfClass directly, or cast your objects with AnyObject::downcast_refSourcefn isMemberOfClass(&self, cls: &AnyClass) -> bool
fn isMemberOfClass(&self, cls: &AnyClass) -> bool
Sourcefn respondsToSelector(&self, aSelector: Sel) -> bool
fn respondsToSelector(&self, aSelector: Sel) -> bool
Sourcefn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
fn conformsToProtocol(&self, aProtocol: &AnyProtocol) -> bool
Sourcefn debugDescription(&self) -> Retained<NSObject>
fn debugDescription(&self) -> Retained<NSObject>
Sourceimpl NSSecureCoding for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl NSSecureCoding for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl PartialEq for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl PartialEq for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceimpl RefEncode for MPSMatrixFullyConnectedGradient
Available on crate feature MPSNeuralNetwork only.
impl RefEncode for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.Sourceconst ENCODING_REF: Encoding = <MPSMatrixBinaryKernel as ::objc2::RefEncode>::ENCODING_REF
const ENCODING_REF: Encoding = <MPSMatrixBinaryKernel as ::objc2::RefEncode>::ENCODING_REF
impl DowncastTarget for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.impl Eq for MPSMatrixFullyConnectedGradient
MPSNeuralNetwork only.