MPSMatrixRandom

Struct MPSMatrixRandom 

pub struct MPSMatrixRandom { /* private fields */ }
Available on crate features MPSCore and MPSKernel and MPSMatrixRandom only.
Kernels that implement random number generation.

See also Apple’s documentation

Implementations

Source

impl MPSMatrixRandom

pub unsafe fn destinationDataType(&self) -> MPSDataType

Available on crate features MPSMatrix and MPSCoreTypes only.

The type of the data which makes up the values of the result. Supported values are: MPSDataTypeUInt32 MPSDataTypeFloat32

Default is MPSDataTypeUInt32

pub unsafe fn distributionType(&self) -> MPSMatrixRandomDistribution

Available on crate feature MPSMatrix only.

The distribution from which to generate random values.

Default is MPSMatrixRandomDistributionDefault

pub unsafe fn batchStart(&self) -> NSUInteger

Available on crate feature MPSMatrix only.

The starting index in the destination batch.

pub unsafe fn setBatchStart(&self, batch_start: NSUInteger)

Available on crate feature MPSMatrix only.

Setter for batchStart.

pub unsafe fn batchSize(&self) -> NSUInteger

Available on crate feature MPSMatrix only.

The size of the batch to process.

pub unsafe fn setBatchSize(&self, batch_size: NSUInteger)

Available on crate feature MPSMatrix only.

Setter for batchSize.

pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>

Available on crate feature MPSMatrix only.
pub unsafe fn encodeToCommandBuffer_destinationVector( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, destination_vector: &MPSVector, )

Available on crate feature MPSMatrix only.

Encode a MPSMatrixRandom kernel into a command Buffer.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter destinationVector: A valid MPSVector to contain the result.

pub unsafe fn encodeToCommandBuffer_destinationMatrix( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, destination_matrix: &MPSMatrix, )

Available on crate feature MPSMatrix only.

Encode a MPSMatrixRandom kernel into a command Buffer.

Parameter commandBuffer: A valid MTLCommandBuffer to receive the encoded filter

Parameter destinationMatrix: A valid MPSMatrix to contain the result.

Source

impl MPSMatrixRandom

Methods declared on superclass MPSKernel.

pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>

Available on crate feature MPSMatrix only.

Called by NSCoder to decode MPSKernels

This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.

Safety

a_decoder possibly has further requirements.

pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>

Available on crate feature MPSMatrix only.

NSSecureCoding compatability

While the standard NSSecureCoding/NSCoding method -initWithCoder: should work, since the file can’t know which device your data is allocated on, we have to guess and may guess incorrectly. To avoid that problem, use initWithCoder:device instead.

Parameter aDecoder: The NSCoder subclass with your serialized MPSKernel

Parameter device: The MTLDevice on which to make the MPSKernel

Returns: A new MPSKernel object, or nil if failure.

Safety

a_decoder possibly has further requirements.

Source

impl MPSMatrixRandom

Methods declared on superclass NSObject.

pub unsafe fn init(this: Allocated<Self>) -> Retained<Self>

Available on crate feature MPSMatrix only.
pub unsafe fn new() -> Retained<Self>

Available on crate feature MPSMatrix only.

Methods from Deref<Target = MPSKernel>

pub unsafe fn options(&self) -> MPSKernelOptions

Available on crate feature MPSCoreTypes only.

The set of options used to run the kernel. subsubsection_options

pub unsafe fn setOptions(&self, options: MPSKernelOptions)

Available on crate feature MPSCoreTypes only.

Setter for options.

pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>

The device on which the kernel will be used

pub unsafe fn label(&self) -> Option<Retained<NSString>>

A string to help identify this object.

pub unsafe fn setLabel(&self, label: Option<&NSString>)

Setter for label.

This is copied when set.

pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>

Make a copy of this MPSKernel for a new device

-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:

Parameter zone: The NSZone in which to allocate the object

Parameter device: The device for the new MPSKernel. If nil, then use self.device.

Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.

Safety

zone must be a valid pointer or null.

Methods from Deref<Target = NSObject>

pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !

Handle messages the object doesn’t recognize.

See Apple’s documentation for details.

Methods from Deref<Target = AnyObject>

pub fn class(&self) -> &'static AnyClass

Dynamically find the class of this object.

Panics

May panic if the object is invalid (which may be the case for objects returned from unavailable init/new methods).

Example

Check that an instance of NSObject has the precise class NSObject.

use objc2::ClassType;
use objc2::runtime::NSObject;

let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());
pub unsafe fn get_ivar<T>(&self, name: &str) -> &T
where T: Encode,

👎Deprecated: this is difficult to use correctly, use Ivar::load instead.

Use Ivar::load instead.

Safety

The object must have an instance variable with the given name, and it must be of type T.

See Ivar::load_ptr for details surrounding this.

pub fn downcast_ref<T>(&self) -> Option<&T>
where T: DowncastTarget,

Attempt to downcast the object to a class of type T.

This is the reference-variant. Use Retained::downcast if you want to convert a retained object to another type.

Mutable classes

Some classes have immutable and mutable variants, such as NSString and NSMutableString.

When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.

So using this method to convert a NSString to a NSMutableString, while not unsound, is generally frowned upon unless you created the string yourself, or the API explicitly documents the string to be mutable.

See Apple’s documentation on mutability and on isKindOfClass: for more details.

Generic classes

Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.

You can, however, safely downcast to generic collections where all the type-parameters are AnyObject.

Panics

This works internally by calling isKindOfClass:. That means that the object must have the instance method of that name, and an exception will be thrown (if CoreFoundation is linked) or the process will abort if that is not the case. In the vast majority of cases, you don’t need to worry about this, since both root objects NSObject and NSProxy implement this method.

Examples

Cast an NSString back and forth from NSObject.

use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};

let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();

Try (and fail) to cast an NSObject to an NSString.

use objc2_foundation::{NSObject, NSString};

let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());

Try to cast to an array of strings.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();

This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.

Downcast when processing each element instead.

use objc2_foundation::{NSArray, NSObject, NSString};

let arr = NSArray::from_retained_slice(&[NSObject::new()]);

for elem in arr {
    if let Some(data) = elem.downcast_ref::<NSString>() {
        // handle `data`
    }
}