MPSRayOriginMaskDirectionMaxDistance

Struct MPSRayOriginMaskDirectionMaxDistance 

#[repr(C)]
pub struct MPSRayOriginMaskDirectionMaxDistance { pub origin: MPSPackedFloat3, pub mask: c_uint, pub direction: MPSPackedFloat3, pub maxDistance: c_float, }
Available on crate feature MPSRayIntersectorTypes only.
Represents a 3D ray with an origin, a direction, and a mask to filter out intersections

This type is available from the Metal Shading Language by including the MetalPerformanceShaders/MetalPerformanceShaders.h header.

See also Apple’s documentation

Fields

origin: MPSPackedFloat3

Ray origin. The intersection test will be skipped if the origin contains NaNs or infinities.

mask: c_uint

Ray mask which is bitwise AND-ed with instance and primitive masks to filter out intersections. The intersection test will be skipped if the mask is zero.

direction: MPSPackedFloat3

Ray direction. Does not need to be normalized. The intersection test will be skipped if the direction has length zero or contains NaNs or infinities.

maxDistance: c_float

Maximum intersection distance from the origin along the ray direction. May be infinite. The intersection test will be skipped if the maximum distance is less than zero or NaN.