MPSRayPackedOriginDirection

Struct MPSRayPackedOriginDirection 

#[repr(C)]
pub struct MPSRayPackedOriginDirection { pub origin: MPSPackedFloat3, pub direction: MPSPackedFloat3, }
Available on crate features MPSRayIntersector and MPSRayIntersectorTypes only.
Represents a 3D ray with an origin and a direction

This type is available from the Metal Shading Language by including the MetalPerformanceShaders/MetalPerformanceShaders.h header.

See also Apple’s documentation

Fields

origin: MPSPackedFloat3

Ray origin. The intersection test will be skipped if the origin contains NaNs or infinities.

direction: MPSPackedFloat3

Ray direction. Does not need to be normalized. The intersection test will be skipped if the direction has length zero or contains NaNs or infinities.