pub struct MPSTemporalAA { /* private fields */ }MPSCore and MPSKernel and MPSRayIntersector and MPSTemporalAA only.Expand description
Reduces aliasing in an image by accumulating samples over multiple frames
The color for the previous frame will be sampled using the provided motion vector texture and blended with the current frame according to the blendFactor property. The colors from the previous frame will be clamped to the color-space bounding box formed by the center pixel’s neighbors to avoid reprojection artifacts, and the motion vector texture will be dilated to avoid aliased silhouette edges under motion.
For the best result, the sample positions produced by the renderer should be jittered every frame, ideally using a low discrepency sequence. This will ensure that different sample positions along edges will be visited over time even if the camera is not moving. This will also reduce aliasing due to textures and high-frequency shading.
For reference, see “High-Quality Temporal Supersampling” by Karis.
See also Apple’s documentation
Implementations
Sourceimpl MPSTemporalAA
impl MPSTemporalAA
Sourcepub unsafe fn blendFactor(&self) -> c_float
pub unsafe fn blendFactor(&self) -> c_float
How much to blend the current frame with the previous frame during temporal antialiasing. The final value is given by current * blendFactor + previous * (1 - blendFactor). Must be between zero and one, inclusive. Defaults to 0.1.
Sourcepub unsafe fn setBlendFactor(&self, blend_factor: c_float)
pub unsafe fn setBlendFactor(&self, blend_factor: c_float)
Setter for blendFactor.
pub unsafe fn initWithDevice( this: Allocated<Self>, device: &ProtocolObject<dyn MTLDevice>, ) -> Retained<Self>
Sourcepub unsafe fn initWithCoder_device(
this: Allocated<Self>,
a_decoder: &NSCoder,
device: &ProtocolObject<dyn MTLDevice>,
) -> Option<Retained<Self>>
pub unsafe fn initWithCoder_device( this: Allocated<Self>, a_decoder: &NSCoder, device: &ProtocolObject<dyn MTLDevice>, ) -> Option<Retained<Self>>
Safety
a_decoder possibly has further requirements.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Safety
zone must be a valid pointer or null.
Sourcepub unsafe fn encodeWithCoder(&self, coder: &NSCoder)
pub unsafe fn encodeWithCoder(&self, coder: &NSCoder)
Safety
coder possibly has further requirements.
Sourcepub unsafe fn encodeToCommandBuffer_sourceTexture_previousTexture_destinationTexture_motionVectorTexture_depthTexture(
&self,
command_buffer: &ProtocolObject<dyn MTLCommandBuffer>,
source_texture: &ProtocolObject<dyn MTLTexture>,
previous_texture: &ProtocolObject<dyn MTLTexture>,
destination_texture: &ProtocolObject<dyn MTLTexture>,
motion_vector_texture: Option<&ProtocolObject<dyn MTLTexture>>,
depth_texture: Option<&ProtocolObject<dyn MTLTexture>>,
)
pub unsafe fn encodeToCommandBuffer_sourceTexture_previousTexture_destinationTexture_motionVectorTexture_depthTexture( &self, command_buffer: &ProtocolObject<dyn MTLCommandBuffer>, source_texture: &ProtocolObject<dyn MTLTexture>, previous_texture: &ProtocolObject<dyn MTLTexture>, destination_texture: &ProtocolObject<dyn MTLTexture>, motion_vector_texture: Option<&ProtocolObject<dyn MTLTexture>>, depth_texture: Option<&ProtocolObject<dyn MTLTexture>>, )
Encode temporal antialiasing a command buffer
The motion vector texture must be at least a two channel texture representing how many texels each texel in the source image(s) have moved since the previous frame. The remaining channels will be ignored if present. This texture may be nil, in which case the motion vector is assumed to be zero, which is suitable for static images.
The depth texture must contain the depth values for directly visible geometry for the current frame for each pixel. The first channel must store the depth value from zero to infinity. The depth texture may be nil, but this will prevent motion vectors from being dilated and may introduce aliasing along silhouette edges.
The destination texture should be used as the previous texture in the next frame.
Parameter commandBuffer: Command buffer to encode into
Parameter sourceTexture: Current frame to denoise
Parameter previousTexture: Previous denoised frame to reproject into current
frame
Parameter destinationTexture: Output blended image
Parameter motionVectorTexture: Motion vector texture
Parameter depthTexture: The depth values for the current frame
Safety
source_texturemay need to be synchronized.source_texturemay be unretained, you must ensure it is kept alive while in use.previous_texturemay need to be synchronized.previous_texturemay be unretained, you must ensure it is kept alive while in use.destination_texturemay need to be synchronized.destination_texturemay be unretained, you must ensure it is kept alive while in use.motion_vector_texturemay need to be synchronized.motion_vector_texturemay be unretained, you must ensure it is kept alive while in use.depth_texturemay need to be synchronized.depth_texturemay be unretained, you must ensure it is kept alive while in use.
Sourceimpl MPSTemporalAA
Methods declared on superclass MPSKernel.
impl MPSTemporalAA
Methods declared on superclass MPSKernel.
Sourcepub unsafe fn initWithCoder(
this: Allocated<Self>,
a_decoder: &NSCoder,
) -> Option<Retained<Self>>
pub unsafe fn initWithCoder( this: Allocated<Self>, a_decoder: &NSCoder, ) -> Option<Retained<Self>>
Called by NSCoder to decode MPSKernels
This isn’t the right interface to decode a MPSKernel, but it is the one that NSCoder uses. To enable your NSCoder (e.g. NSKeyedUnarchiver) to set which device to use extend the object to adopt the MPSDeviceProvider protocol. Otherwise, the Metal system default device will be used.
Safety
a_decoder possibly has further requirements.
Methods from Deref<Target = MPSKernel>
Sourcepub unsafe fn options(&self) -> MPSKernelOptions
Available on crate feature MPSCoreTypes only.
pub unsafe fn options(&self) -> MPSKernelOptions
MPSCoreTypes only.The set of options used to run the kernel. subsubsection_options
Sourcepub unsafe fn setOptions(&self, options: MPSKernelOptions)
Available on crate feature MPSCoreTypes only.
pub unsafe fn setOptions(&self, options: MPSKernelOptions)
MPSCoreTypes only.Setter for options.
Sourcepub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
pub unsafe fn device(&self) -> Retained<ProtocolObject<dyn MTLDevice>>
The device on which the kernel will be used
Sourcepub unsafe fn label(&self) -> Option<Retained<NSString>>
pub unsafe fn label(&self) -> Option<Retained<NSString>>
A string to help identify this object.
Sourcepub unsafe fn copyWithZone_device(
&self,
zone: *mut NSZone,
device: Option<&ProtocolObject<dyn MTLDevice>>,
) -> Retained<Self>
pub unsafe fn copyWithZone_device( &self, zone: *mut NSZone, device: Option<&ProtocolObject<dyn MTLDevice>>, ) -> Retained<Self>
Make a copy of this MPSKernel for a new device
-copyWithZone: will call this API to make a copy of the MPSKernel on the same device. This interface may also be called directly to make a copy of the MPSKernel on a new device. Typically, the same MPSKernels should not be used to encode kernels on multiple command buffers from multiple threads. Many MPSKernels have mutable properties that might be changed by the other thread while this one is trying to encode. If you need to use a MPSKernel from multiple threads make a copy of it for each additional thread using -copyWithZone: or -copyWithZone:device:
Parameter zone: The NSZone in which to allocate the object
Parameter device: The device for the new MPSKernel. If nil, then use
self.device.
Returns: a pointer to a copy of this MPSKernel. This will fail, returning nil if the device is not supported. Devices must be MTLFeatureSet_iOS_GPUFamily2_v1 or later.
Safety
zone must be a valid pointer or null.
Methods from Deref<Target = NSObject>
Sourcepub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
pub fn doesNotRecognizeSelector(&self, sel: Sel) -> !
Handle messages the object doesn’t recognize.
See Apple’s documentation for details.
Methods from Deref<Target = AnyObject>
Sourcepub fn class(&self) -> &'static AnyClass
pub fn class(&self) -> &'static AnyClass
Dynamically find the class of this object.
Panics
May panic if the object is invalid (which may be the case for objects
returned from unavailable init/new methods).
Example
Check that an instance of NSObject has the precise class NSObject.
use objc2::ClassType;
use objc2::runtime::NSObject;
let obj = NSObject::new();
assert_eq!(obj.class(), NSObject::class());Sourcepub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
👎Deprecated: this is difficult to use correctly, use Ivar::load instead.
pub unsafe fn get_ivar<T>(&self, name: &str) -> &Twhere
T: Encode,
this is difficult to use correctly, use Ivar::load instead.
Use Ivar::load instead.
Safety
The object must have an instance variable with the given name, and it
must be of type T.
See Ivar::load_ptr for details surrounding this.
Sourcepub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
pub fn downcast_ref<T>(&self) -> Option<&T>where
T: DowncastTarget,
Attempt to downcast the object to a class of type T.
This is the reference-variant. Use Retained::downcast if you want
to convert a retained object to another type.
Mutable classes
Some classes have immutable and mutable variants, such as NSString
and NSMutableString.
When some Objective-C API signature says it gives you an immutable class, it generally expects you to not mutate that, even though it may technically be mutable “under the hood”.
So using this method to convert a NSString to a NSMutableString,
while not unsound, is generally frowned upon unless you created the
string yourself, or the API explicitly documents the string to be
mutable.
See Apple’s documentation on mutability and on
isKindOfClass: for more details.
Generic classes
Objective-C generics are called “lightweight generics”, and that’s because they aren’t exposed in the runtime. This makes it impossible to safely downcast to generic collections, so this is disallowed by this method.
You can, however, safely downcast to generic collections where all the
type-parameters are AnyObject.
Panics
This works internally by calling isKindOfClass:. That means that the
object must have the instance method of that name, and an exception
will be thrown (if CoreFoundation is linked) or the process will abort
if that is not the case. In the vast majority of cases, you don’t need
to worry about this, since both root objects NSObject and
NSProxy implement this method.
Examples
Cast an NSString back and forth from NSObject.
use objc2::rc::Retained;
use objc2_foundation::{NSObject, NSString};
let obj: Retained<NSObject> = NSString::new().into_super();
let string = obj.downcast_ref::<NSString>().unwrap();
// Or with `downcast`, if we do not need the object afterwards
let string = obj.downcast::<NSString>().unwrap();Try (and fail) to cast an NSObject to an NSString.
use objc2_foundation::{NSObject, NSString};
let obj = NSObject::new();
assert!(obj.downcast_ref::<NSString>().is_none());Try to cast to an array of strings.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
// This is invalid and doesn't type check.
let arr = arr.downcast_ref::<NSArray<NSString>>();This fails to compile, since it would require enumerating over the array to ensure that each element is of the desired type, which is a performance pitfall.
Downcast when processing each element instead.
use objc2_foundation::{NSArray, NSObject, NSString};
let arr = NSArray::from_retained_slice(&[NSObject::new()]);
for elem in arr {
if let Some(data) = elem.downcast_ref::<NSString>() {
// handle `data`
}
}Trait Implementations
Sourceimpl AsRef<AnyObject> for MPSTemporalAA
impl AsRef<AnyObject> for MPSTemporalAA
Sourceimpl AsRef<MPSKernel> for MPSTemporalAA
impl AsRef<MPSKernel> for MPSTemporalAA
Sourceimpl AsRef<MPSTemporalAA> for MPSTemporalAA
impl AsRef<MPSTemporalAA> for MPSTemporalAA
Sourceimpl AsRef<NSObject> for MPSTemporalAA
impl AsRef<NSObject> for MPSTemporalAA
Sourceimpl Borrow<AnyObject> for MPSTemporalAA
impl Borrow<AnyObject> for MPSTemporalAA
Sourceimpl Borrow<MPSKernel> for MPSTemporalAA
impl Borrow<MPSKernel> for MPSTemporalAA
Sourceimpl Borrow<NSObject> for MPSTemporalAA
impl Borrow<NSObject> for MPSTemporalAA
Sourceimpl ClassType for MPSTemporalAA
impl ClassType for MPSTemporalAA
Sourceconst NAME: &'static str = "MPSTemporalAA"
const NAME: &'static str = "MPSTemporalAA"
Sourcetype ThreadKind = <<MPSTemporalAA as ClassType>::Super as ClassType>::ThreadKind
type ThreadKind = <<MPSTemporalAA as ClassType>::Super as ClassType>::ThreadKind
Sourceimpl CopyingHelper for MPSTemporalAA
impl CopyingHelper for MPSTemporalAA
Sourcetype Result = MPSTemporalAA
type Result = MPSTemporalAA
Self if the type has no
immutable counterpart. Read moreSourceimpl Debug for MPSTemporalAA
impl Debug for MPSTemporalAA
Sourceimpl Deref for MPSTemporalAA
impl Deref for MPSTemporalAA
Sourceimpl Hash for MPSTemporalAA
impl Hash for MPSTemporalAA
Sourceimpl Message for MPSTemporalAA
impl Message for MPSTemporalAA
Sourceimpl NSCoding for MPSTemporalAA
impl NSCoding for MPSTemporalAA
Sourceunsafe fn encodeWithCoder(&self, coder: &NSCoder)
unsafe fn encodeWithCoder(&self, coder: &NSCoder)
NSCoder only.Sourceimpl NSCopying for MPSTemporalAA
impl NSCopying for MPSTemporalAA
Sourceimpl NSObjectProtocol for MPSTemporalAA
impl NSObjectProtocol for MPSTemporalAA
Sourcefn isEqual(&self, other: Option<&AnyObject>) -> bool
fn isEqual(&self, other: Option<&AnyObject>) -> bool
Sourcefn hash(&self) -> usize
fn hash(&self) -> usize
Sourcefn isKindOfClass(&self, cls: &AnyClass) -> bool
fn isKindOfClass(&self, cls: &AnyClass) -> bool
Sourcefn is_kind_of<T>(&self) -> bool
fn is_kind_of<T>(&self) -> bool
use isKindOfClass directly, or cast your objects with AnyObject::downcast_ref